 /**
 * @return {number}
 */
function random(n){return (Math.floor (Math.random() * n));}

 /**
 * @constructor
 * @param {number} r
 * @param {number} g
 * @param {number} b
 */
function TColor(r,g,b){
	this.r = r;
	this.g = g;
	this.b = b;
	this.asString = function(){
		return "rgb("+this.r+","+this.g+","+this.b+")";
	}
}

 /**
 * @constructor
 */
function TSettings() {
	this.HQ = false;
	this.load();
}

 /**
 * @param {boolean} aHQ
 */
TSettings.prototype.setHQ = function (aHQ) {
	this.HQ = aHQ;
	this.save();
}

TSettings.prototype.load = function () {
	var cookies = document.cookie.split(";");
	for (var i = 0; i < cookies.length; i++) {
		var name = cookies[i].substr(0, cookies[i].indexOf("="));
		var value = cookies[i].substr(cookies[i].indexOf("=") + 1);
		name = name.replace(/^\s+|\s+$/g, "");
		if (name == "neonflow_settings_hq") this.HQ = JSON.parse(value);
	}
}

TSettings.prototype.save = function () {
	var expiryDate = new Date();
	expiryDate.setDate(expiryDate.getDate() + 100000);
	sExpiry = "; expires=" + expiryDate.toUTCString();
	document.cookie = "neonflow_settings_hq=" + JSON.stringify(this.HQ) + sExpiry;
}

/**
 * @constructor
 * @param {number=} aX
 * @param {number=} aY
 */
function TPoint(aX, aY) {
	this.X = aX || 0;
	this.Y = aY || 0;
}

/**
 * @constructor
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 */
function TControl(aCaption, aLeft, aTop, aWidth, aHeight) {
	this.Caption = aCaption || "";
	this.Left = aLeft || 0;
	this.Top = aTop || 0;
	this.Width = aWidth || 0;
	this.Height = aHeight || 0;
	this.Visible = true;
	this.Enabled = true;
	this.NeedRepaint = true;
	this.Parent = null;
	this.Font = "10px Arial";
	this.FontColor = "white";
	this.Color = "black";
	this.ShadowColor = "transparent";
	this.ShadowOffsetX = 0;
	this.ShadowOffsetY = 0;
	this.ShadowBlur = 0;
}

TControl.prototype.show = function () {
	this.Visible = true;
	this.NeedRepaint = true;
}

TControl.prototype.hide = function () {
	this.Visible = false;
	this.NeedRepaint = true;
}

TControl.prototype.enable = function () {
	this.Enabled = true;
	this.NeedRepaint = true;
}

TControl.prototype.disable = function () {
	this.Enabled = false;
	this.mouseOut();
	this.NeedRepaint = true;
}

TControl.prototype.paint = function () {}

 /**
 * @param {string} aCaption
 */
TControl.prototype.setCaption = function (aCaption) {
	if (!aCaption) return;
	this.Caption = aCaption;
	this.NeedRepaint = true;
}

TControl.prototype.getCaption = function () {
	return this.Caption;
}

/**
 * @return {boolean}
 */
TControl.prototype.isMouseOver = function () {
	return ((gMouse.X >= this.getLeft()) && (gMouse.X <= this.getLeft() + this.Width) && (gMouse.Y >= this.getTop()) && (gMouse.Y <= this.getTop() + this.Height));
}

 /**
 * @param {TControl} aParent
 */
TControl.prototype.setParent = function (aParent) {
	if (!aParent) return;
	this.Parent = aParent;
}

 /**
 * @param {string} aFont
 */
TControl.prototype.setFont = function (aFont) {
	if (!aFont) return;
	this.Font = aFont;
	this.NeedRepaint = true;
}

 /**
 * @param {string} aFontColor
 */
TControl.prototype.setFontColor = function (aFontColor) {
	this.FontColor = aFontColor || "white";
	this.NeedRepaint = true;
}

 /**
 * @param {string} aColor
 */
TControl.prototype.setColor = function (aColor) {
	this.Color = aColor || "black";
	this.NeedRepaint = true;
}

 /**
 * @param {number} aLeft
 */
TControl.prototype.setLeft = function (aLeft) {
	this.Left = aLeft || 0;
	this.NeedRepaint = true;
	this.mouseMove(gMouse.X, gMouse.Y);
}

 /**
 * @param {number} aTop
 */
TControl.prototype.setTop = function (aTop) {
	this.Top = aTop || 0;
	this.NeedRepaint = true;
	this.mouseMove(gMouse.X, gMouse.Y);
}

TControl.prototype.getWidth = function () {
	return this.Width;
}

TControl.prototype.getHeight = function () {
	return this.Height;
}

 /**
 * @param {number} aLeft
 * @param {number} aTop
 */
TControl.prototype.setPosition = function (aLeft, aTop) {
	this.Left = aLeft || 0;
	this.Top = aTop || 0;
	this.NeedRepaint = true;
	this.mouseMove(gMouse.X, gMouse.Y);
}

 /**
 * @param {string} Color
 * @param {number} xOffset
 * @param {number} yOffset
 * @param {number} Blur
 */
TControl.prototype.setShadow = function (Color, xOffset, yOffset, Blur) {
	this.ShadowColor = Color || "black";
	this.ShadowOffsetX = xOffset || 0;
	this.ShadowOffsetY = yOffset || 0;
	this.ShadowBlur = Blur || 0;
	this.NeedRepaint = true;
}

 /**
 * @param {number} x
 * @param {number} y
 */
TControl.prototype.mouseMove = function (x, y) {}

TControl.prototype.mouseOut = function () {}

 /**
 * @param {boolean} left
 */
TControl.prototype.mouseDown = function (left) {}

 /**
 * @param {boolean} left
 */
TControl.prototype.mouseUp = function (left) {}

 /**
 * @return {boolean}
 */
TControl.prototype.update = function () {
	if (!this.NeedRepaint) return false;
	this.NeedRepaint = false;
	return true;
}

 /**
 * @return {boolean}
 */
TControl.prototype.getVisible = function () {
	if (!this.Parent) return this.Visible;
	else return (this.Visible && this.Parent.getVisible());
}

/**
 * @return {boolean}
 */
TControl.prototype.getEnabled = function () {
	if (!this.Parent) return this.Enabled;
	else return (this.Enabled && this.Parent.getEnabled());
}

 /**
 * @return {number}
 */
TControl.prototype.getLeft = function () {
	if (!this.Parent) return this.Left;
	else return parseInt(this.Parent.getLeft() + this.Left, 10);
}

 /**
 * @return {number}
 */
TControl.prototype.getTop = function () {
	if (!this.Parent) return this.Top;
	else return parseInt(this.Parent.getTop() + this.Top, 10);
}

/**
 * @constructor
 * @extends TControl
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
*/
function TPanel(aLeft, aTop, aWidth, aHeight) {
	TControl.call(this, "", aLeft, aTop, aWidth, aHeight);
	this.Controls = [];
}
TPanel.prototype = Object.create(TControl.prototype);
TPanel.prototype.constructor = TPanel;

TPanel.prototype.paintBack = function () {
	gContext.beginPath();
	gContext.rect(this.getLeft(), this.getTop(), this.Width - 1, this.Height - 1);
	if (gSettings.HQ) gContext.globalAlpha = 0.8;
	try {
		gContext.fillStyle = this.Color;
		gContext.fill();
		gContext.lineWidth = 1;
		gContext.strokeStyle = "silver";
		gContext.stroke();
	} finally {
		if (gSettings.HQ) gContext.globalAlpha = 1;
	}
}

TPanel.prototype.paint = function () {
	if (!this.getVisible()) return;
	this.paintBack();
	gContext.save();
	gContext.clip();
	try {
		for (var i = 0; i < this.Controls.length; i++) this.Controls[i].paint();
	} finally {
		gContext.restore();
	}
}

/**
 * @return {boolean}
 */
TPanel.prototype.update = function () {
	if (!this.getVisible()) return false;
	for (var i = 0; i < this.Controls.length; i++) this.NeedRepaint = this.Controls[i].update() || this.NeedRepaint;
	return TControl.prototype.update.call(this);
}

/**
 * @param {TControl} control
 */
TPanel.prototype.addControl = function (control) {
	if (!control) return;
	this.Controls.push(control);
	control.setParent(this);
	this.NeedRepaint = true;
}

/**
 * @param {TControl} control
 * @param {number} position
 */
TPanel.prototype.insertControl = function (control, position) {
	if (!control) return;
	if ((position < 0) || (position >= this.Controls.length)) this.addControl(control);
	else {
		this.Controls.splice(position, 0, control);
		control.setParent(this);
		this.NeedRepaint = true;
	}
}

/**
 * @param {TControl} control
 * @return {boolean}
 */
TPanel.prototype.delControl = function (control) {
	if (!control) return false;
	control.setParent(null);
	control.hide();
	for (var i = 0; i < this.Controls.length; i++) {
		if (this.Controls[i] === control) {
			this.Controls.splice(i, 1);
			this.NeedRepaint = true;
			return true;
		}
	}
	return false;
}

TPanel.prototype.mouseOut = function () {
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseOut();
}

 /**
 * @param {number} x
 * @param {number} y
 */
TPanel.prototype.mouseMove = function (x, y) {
	if (!this.getEnabled()) return;
	for (var i = 0; i < this.Controls.length; i++) {
		if (this.Controls[i].isMouseOver()) this.Controls[i].mouseMove(x, y);
		else this.Controls[i].mouseOut();
	}
}

 /**
 * @param {boolean} left
 */
TPanel.prototype.mouseDown = function (left) {
	if (!this.getEnabled()) return;
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseDown(left);
}

 /**
 * @param {boolean} left
 */
TPanel.prototype.mouseUp = function (left) {
	if (!this.getEnabled()) return;
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseUp(left);
}

/**
 * @constructor
 */
function TImageList() {
	this.Images = [];
}

TImageList.prototype.count = function () {
	return this.Images.length;
};

 /**
 * @param {string} aPath
 * @return {number} index
 */
TImageList.prototype.add = function (aPath) {
	if (!aPath) return -1;
	var image = new Image();
	image.src = aPath;
	if (image.height == 0) return -1;
	this.Images.push(image);
	return (this.Images.length - 1);
};

 /**
 * @param {number} index
 * @return {Image}
 */
TImageList.prototype.get = function (index) {
	if (index < 0) return null;
	if (index >= this.Images.length) return null;
	return this.Images[index];
}

/** @enum {number} */
var TTextAlignment = {taLeft: 1, taCenter: 2, taRight: 3}

/**
 * @constructor
 * @extends TControl
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {TTextAlignment=} aTextAlignment
*/
function TLabel(aCaption, aLeft, aTop, aTextAlignment) {
	TControl.call(this, aCaption, aLeft, aTop, 0, 0);
	this.Font = "bold 20px Arial";
	this.MaxWidth = 0;
	this.TextAlignment = aTextAlignment || TTextAlignment.taLeft;
}
TLabel.prototype = Object.create(TControl.prototype);
TLabel.prototype.constructor = TLabel;

 /**
 * @param {TTextAlignment} aAlignment
 */
TLabel.prototype.setTextAlignment = function (aAlignment) {
	if (!aAlignment) return;
	this.TextAlignment = aAlignment;
	this.NeedRepaint = true;
}

 /**
 * @param {number} aMaxWidth
 */
TLabel.prototype.setMaxWidth = function (aMaxWidth) {
	this.MaxWidth = aMaxWidth || 0;
	this.NeedRepaint = true;
}

 /**
 * @return {number}
 */
TLabel.prototype.getWidth = function () {
	if (this.Caption == "") return 0;
	gContext.font = this.Font;
	gContext.fillStyle = this.FontColor;
	if (this.MaxWidth == 0) {
		return gContext.measureText(this.Caption).width;
	} else {
		var txtWidth = gContext.measureText(this.Caption).width;
		if (txtWidth > this.MaxWidth) return this.MaxWidth;
		else return txtWidth;
	}
}

TLabel.prototype.paint = function () {
	if (!this.getVisible()) return;
	if (this.Caption == "") return;
	gContext.font = this.Font;
	gContext.fillStyle = this.FontColor;
	gContext.textBaseline = "middle";
	if (this.TextAlignment == TTextAlignment.taLeft) gContext.textAlign = "left";
	else if (this.TextAlignment == TTextAlignment.taRight) gContext.textAlign = "right";
	else gContext.textAlign = "center";

	gContext.save();
	try {
		gContext.shadowColor = this.ShadowColor;
		gContext.shadowOffsetX = this.ShadowOffsetX;
		gContext.shadowOffsetY = this.ShadowOffsetY;
		gContext.shadowBlur = this.ShadowBlur;

		if (this.MaxWidth == 0) {
			gContext.fillText(this.Caption, this.getLeft(), this.getTop());
		} else {
			var yOffset = 0;
			var lineHeight = 30;
			var words = this.Caption.split(" ");
			var line = words[0];
			for (var i = 1; i < words.length; i++) {
				if (gContext.measureText(line + " " + words[i]).width > this.MaxWidth) {
					gContext.fillText(line, this.getLeft(), this.getTop() + yOffset);
					line = words[i];
					yOffset += lineHeight;
				} else line = line + " " + words[i];
			}
			gContext.fillText(line, this.getLeft(), this.getTop() + yOffset);
		}
	} finally {
		gContext.restore();
	}
}

/** @enum {number} */
var TNeonColor = {nePink: 1, neGreen: 2, neBlue: 3, neYellow: 4, neOrange: 5}

/**
 * @constructor
 * @extends TControl
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {TNeonColor} aColor
 * @param {boolean} aFlickering
 */
function TNeonLabel(aCaption, aLeft, aTop, aColor, aFlickering) {
	TControl.call(this, aCaption, aLeft, aTop, 0, 0);
	this.Font = "bold 30px Arial";
	this.NeonColor = aColor || TNeonColor.nePink;
	this.FlickerLength = 0;
	this.Flickering = false;
	if (gSettings.HQ) this.Flickering = aFlickering;
}
TNeonLabel.prototype = Object.create(TControl.prototype);
TNeonLabel.prototype.constructor = TNeonLabel;

 /**
 * @return {boolean}
 */
TNeonLabel.prototype.update = function () {
	if (gSettings.HQ && this.Flickering) {
		if (this.FlickerLength > 0) {
			this.FlickerLength -= 1;
			this.NeedRepaint = true;
		} else if (random(50) == 0) {
			this.FlickerLength = random(2) + 1;
			this.NeedRepaint = true;
		}
	}
	return TControl.prototype.update.call(this);
}

TNeonLabel.prototype.paint = function () {
	if (!this.getVisible()) return;
	if (this.Caption == "") return;
	gContext.font = this.Font;
	gContext.fillStyle = this.FontColor;
	gContext.textBaseline = "middle";
	gContext.textAlign = "center";

	gContext.save();
	try {
		if (this.FlickerLength == 0) {
			gContext.beginPath();
			var txtWidth = gContext.measureText(this.Caption).width;
			var txtHeight = 30;
			var neonBorder = txtHeight * 1;
			var yOffset = txtHeight * 3;
			var txtX = this.getLeft();
			var txtY = this.getTop() - yOffset;
			gContext.rect(txtX - txtWidth/2 - neonBorder, this.getTop() - txtHeight/2 - 1.5*neonBorder, txtWidth + 2*neonBorder, txtHeight + 3*neonBorder);
			gContext.clip();

			gContext.shadowOffsetX = 0;
			gContext.shadowOffsetY = yOffset;

			switch (this.NeonColor) {
			case TNeonColor.neGreen:
				gContext.shadowColor = "#39FF14";
				break;
			case TNeonColor.neBlue:
				gContext.shadowColor = "#22CEFF";
				break;
			case TNeonColor.neYellow:
				gContext.shadowColor = "#FFFF15";
				break;
			case TNeonColor.neOrange:
				gContext.shadowColor = "#FF6A3C";
				break;
			case TNeonColor.nePink:
			default:
				gContext.shadowColor = "#FF00DE";
				break;
			}

//			if (gSettings.HQ) {
				gContext.shadowBlur = txtHeight * 1.5;
				gContext.fillText(this.Caption, txtX, txtY);
				gContext.shadowBlur = txtHeight;
				gContext.fillText(this.Caption, txtX, txtY);
				gContext.shadowBlur = txtHeight * 0.75;
				gContext.fillText(this.Caption, txtX, txtY);

				gContext.shadowColor = "#FFFFFF";
				gContext.shadowBlur = txtHeight * 0.75;
				gContext.fillText(this.Caption, txtX, txtY);
				gContext.shadowBlur = txtHeight * 0.5;
				gContext.fillText(this.Caption, txtX, txtY);
				gContext.shadowBlur = txtHeight * 0.25;

				txtY += gContext.shadowOffsetY;
				gContext.shadowOffsetY = 0;
				gContext.fillStyle = "white";
//			}
			gContext.fillText(this.Caption, txtX, txtY);
		} else {
			gContext.fillStyle = "#404040";
			gContext.fillText(this.Caption, this.getLeft(), this.getTop());
		}
	} finally {
		gContext.restore();
	}
}

/**
 * @constructor
 * @extends TLabel
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 */
function TCaptionLabel(aCaption, aLeft, aTop) {
	TLabel.call(this, aCaption, aLeft, aTop, TTextAlignment.taCenter);
	this.Font = "bold 30px Arial";
	this.MaxWidth = 0;
	this.ShadowColor = "silver";
	this.ShadowOffsetX = 0;
	this.ShadowOffsetY = 0;
	this.ShadowBlur = 15;
}
TCaptionLabel.prototype = Object.create(TLabel.prototype);
TCaptionLabel.prototype.constructor = TCaptionLabel;

/**
 * @constructor
 * @extends TControl
 * @param {string} aPath
 * @param {number} aLeft
 * @param {number} aTop
*/
function TStaticImage(aPath, aLeft, aTop) {
	TControl.call(this, "", aLeft, aTop, 0, 0);
	this.Image = new Image();
	this.setImage(aPath);
}
TStaticImage.prototype = Object.create(TControl.prototype);
TStaticImage.prototype.constructor = TStaticImage;

TStaticImage.prototype.paint = function () {
	if (!this.getVisible()) return;
	gContext.drawImage(this.Image, this.getLeft(), this.getTop());
}

 /**
 * @param {string} aPath
 */
TStaticImage.prototype.setImage = function (aPath) {
	this.Image.src = aPath;
	this.Height = this.Image.height;
	this.Width = this.Image.width;
}

/**
 * @constructor
 * @extends TControl
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 * @param {function(string, number, boolean=)} callbackClick
 * @param {number=} aID
 * @param {Object=} aImageList
 * @param {number=} aImageIndex
*/
function TButton(aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aID, aImageList, aImageIndex) {
	TControl.call(this, aCaption, aLeft, aTop, aWidth, aHeight);
	this.OnClick = callbackClick;
	this.ID = aID || 0;
	this.TButtonState = {bsDisabled: 0, bsNormal: 1, bsMouse: 2, bsDown: 3}
	this.ButtonState = this.TButtonState.bsNormal;
	this.Font = "12px Arial";
	this.ImageList = null;
	this.ImageIndex = -1;
	if (aImageList) {
		this.ImageList = aImageList;
		if (isNaN(aImageIndex)) return;
		if ((aImageIndex >= 0) && (aImageIndex < aImageList.count())) this.ImageIndex = aImageIndex;
	}
}
TButton.prototype = Object.create(TControl.prototype);
TButton.prototype.constructor = TButton;

TButton.prototype.hide = function () {
	TControl.prototype.hide.call(this);
	if ((this.ButtonState == this.TButtonState.bsMouse) || (this.ButtonState == this.TButtonState.bsDown)) this.ButtonState = this.TButtonState.bsNormal;
}

TButton.prototype.enable = function () {
	this.ButtonState = this.TButtonState.bsNormal;
	this.NeedRepaint = true;
}

TButton.prototype.disable = function () {
	this.ButtonState = this.TButtonState.bsDisabled;
	this.NeedRepaint = true;
}

 /**
 * @param {number} x
 * @param {number} y
 */
TButton.prototype.mouseMove = function (x, y) {
	if (!this.getVisible()) return;
	if (!this.getEnabled()) return;
	if (this.isMouseOver()) {
		if (this.ButtonState == this.TButtonState.bsNormal) {
			this.ButtonState = this.TButtonState.bsMouse;
			this.NeedRepaint = true;
		}
	} else {
		if ((this.ButtonState == this.TButtonState.bsMouse) || (this.ButtonState == this.TButtonState.bsDown)) {
			this.ButtonState = this.TButtonState.bsNormal;
			this.NeedRepaint = true;
		}
	}
}

TButton.prototype.mouseOut = function () {
	if ((this.ButtonState == this.TButtonState.bsNormal) || (this.ButtonState == this.TButtonState.bsDisabled)) return;
	this.ButtonState = this.TButtonState.bsNormal;
	this.NeedRepaint = true;
}

 /**
 * @param {boolean} left
 */
TButton.prototype.mouseDown = function (left) {
	if (!this.getVisible()) return;
	if (!this.getEnabled()) return;
	if (!left) return;
	if (this.ButtonState != this.TButtonState.bsMouse) return;
	this.ButtonState = this.TButtonState.bsDown;
	this.NeedRepaint = true;
}

 /**
 * @param {boolean} left
 */
TButton.prototype.mouseUp = function (left) {
	if (!this.getVisible()) return;
	if (!this.getEnabled()) return;
	if (!left) return;
	if (this.ButtonState != this.TButtonState.bsDown) return;
	this.ButtonState = this.TButtonState.bsMouse;
	this.NeedRepaint = true;
	this.OnClick.call(this, this.Caption, this.ID);
	if (this.ButtonState != this.TButtonState.bsDisabled) {
		if (!this.getEnabled()) this.ButtonState = this.TButtonState.bsNormal;
		else this.mouseMove(gMouse.X, gMouse.Y);
		this.NeedRepaint = true;
	}
}

 /**
 * @param {number} aImageIndex
 */
TButton.prototype.setImageIndex = function (aImageIndex) {
	this.ImageIndex = -1;
	if (isNaN(aImageIndex)) return;
	if (!(this.ImageList)) return;
	if ((aImageIndex >= 0) && (aImageIndex < this.ImageList.count())) {
		this.ImageIndex = aImageIndex;
		this.NeedRepaint = true;
	}
}

TButton.prototype.paintBack = function () {
	gContext.beginPath();
	var fillGradient = gContext.createLinearGradient(this.getLeft(), this.getTop(), this.getLeft(), this.getTop() + this.Height);
	switch (this.ButtonState) {
	case this.TButtonState.bsMouse:
	case this.TButtonState.bsDown:
		fillGradient.addColorStop(0, '#8a7bc3');
		fillGradient.addColorStop(0.5, '#ada2d6');
		fillGradient.addColorStop(1, '#8a7bc3');
		break;
	case this.TButtonState.bsDisabled:
		fillGradient.addColorStop(0, '#a5a8ad');
		fillGradient.addColorStop(1, '#a5a8ad');
		break;
	case this.TButtonState.bsNormal:
		fillGradient.addColorStop(0, '#838385');
		fillGradient.addColorStop(0.5, '#a5a8ad');
		fillGradient.addColorStop(1, '#838385');
		break;
	}
	gContext.lineWidth = 1;
	gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height);
	gContext.fillStyle = fillGradient;
	gContext.strokeStyle = "white";
	gContext.fill();
	gContext.stroke();
}

TButton.prototype.paintCaption = function () {
	if ((this.Caption == "") && (this.ImageIndex < 0)) return;
	var textOffset = 0;
	if (this.ButtonState == this.TButtonState.bsDown) textOffset = 2;
	gContext.font = this.Font;
	if (this.ButtonState == this.TButtonState.bsDisabled) gContext.fillStyle = "#707070";
	else gContext.fillStyle = this.FontColor;

	gContext.save();
	gContext.clip();
	try {
		gContext.textBaseline = "middle";

		if (this.ImageIndex < 0) { // text only
			gContext.textAlign = "center";
			gContext.fillText(this.Caption, this.getLeft() + this.Width / 2 + textOffset, this.getTop() +2+ this.Height / 2 + textOffset);
		} else if (this.Caption == "") { // image only
			var image = this.ImageList.get(this.ImageIndex);
			gContext.drawImage(image, this.getLeft() + (this.Width - image.width) / 2 + textOffset, this.getTop() + (this.Height - image.height) / 2 + textOffset);
		} else { // text and image
			var image = this.ImageList.get(this.ImageIndex);
			var metrics = gContext.measureText(this.Caption);
			var totalWidth = metrics.width + image.width + 10;
			gContext.textAlign = "left";
			gContext.fillText(this.Caption, this.getLeft() + (this.Width - totalWidth) / 2 + textOffset, this.getTop() +2+ this.Height / 2 + textOffset);
			gContext.drawImage(image, this.getLeft() + (this.Width - totalWidth) / 2 + totalWidth - image.width + textOffset, this.getTop() + (this.Height - image.height) / 2 + textOffset);
		}
	} finally {
		gContext.restore();
	}
}

TButton.prototype.paint = function () {
	if (!this.getVisible()) return;
	this.paintBack();
	this.paintCaption();
}

/**
 * @constructor
 * @extends TButton
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 * @param {function(string, number, boolean=)} callbackClick
 * @param {number=} aID
 * @param {Object=} aImageList
 * @param {number=} aImageIndex
*/
function TStandardButton(aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aID, aImageList, aImageIndex) {
	TButton.call(this, aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aID, aImageList, aImageIndex);
	this.Font = "bold 20px Arial";
	this.ShadowLineWidth = 0;
}
TStandardButton.prototype = Object.create(TButton.prototype);
TStandardButton.prototype.constructor = TStandardButton;

 /**
 * @return {boolean}
 */
TStandardButton.prototype.update = function () {
	if (this.ShadowLineWidth > 0) {
		if (this.ButtonState == this.TButtonState.bsNormal) {
			if (gSettings.HQ) this.ShadowLineWidth -= 1;
			else this.ShadowLineWidth = 0;
		}
		else if (this.ButtonState == this.TButtonState.bsDisabled) this.ShadowLineWidth = 0;
		this.NeedRepaint = true;
	}
	return TControl.prototype.update.call(this);
}

 /**
 * @param {number} x
 * @param {number} y
 */
TStandardButton.prototype.mouseMove = function (x, y) {
	TButton.prototype.mouseMove.call(this, x, y);
	if (this.ButtonState == this.TButtonState.bsMouse) this.ShadowLineWidth=this.Height/5;
}

TStandardButton.prototype.paintBack = function () {
	if (!gSettings.HQ) {
		TButton.prototype.paintBack.call(this);
		return;
	}
	gContext.beginPath();
	var fillGradient = gContext.createLinearGradient(this.getLeft(), this.getTop(), this.getLeft(), this.getTop() + this.Height);
	switch (this.ButtonState) {
	case this.TButtonState.bsMouse:
	case this.TButtonState.bsDown:
	case this.TButtonState.bsNormal:
		fillGradient.addColorStop(0, '#838385');
		fillGradient.addColorStop(0.5, '#a5a8ad');
		fillGradient.addColorStop(1, '#838385');
		break;
	case this.TButtonState.bsDisabled:
		fillGradient.addColorStop(0, '#a5a8ad');
		fillGradient.addColorStop(1, '#a5a8ad');
		break;
	}
	gContext.lineWidth = 1;
	gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height);
	gContext.fillStyle = fillGradient;
	gContext.strokeStyle = "white";
	gContext.fill();
	gContext.stroke();

	if (this.ShadowLineWidth > 0) {
		gContext.save();
		gContext.clip();
		try {
			gContext.beginPath();
			gContext.shadowOffsetX = 0;
			gContext.shadowOffsetY = 2 * this.Height;

			gContext.strokeStyle = "white";
			gContext.shadowColor = "#000000";
			gContext.shadowBlur = this.Height * 1.5;
			gContext.lineWidth = this.ShadowLineWidth;
			gContext.lineCap = "round";
			var shadowWidth = this.Width - 2 * this.Height / 3;
			if (shadowWidth < 20) shadowWidth = 20;
			var XOffset = (this.Width - shadowWidth) / 2;
			gContext.moveTo(this.getLeft() + XOffset, this.getTop() - this.Height - this.Height / 2);
			gContext.lineTo(this.getLeft() + this.Width - XOffset, this.getTop() - this.Height - this.Height / 2);
			gContext.stroke();
			gContext.shadowBlur = this.Height;
			gContext.stroke();

		} finally {
			gContext.restore();
			gContext.beginPath();
			gContext.lineWidth = 1;
			gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height);
		}
	}
}

/**
 * @constructor
 * @extends TStandardButton
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 * @param {function(string, number, boolean=)} callbackClick
 * @param {number=} aID
 * @param {Object=} aImageList
*/
function TDiffButton(aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aID, aImageList) {
	TStandardButton.call(this, aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aID, aImageList);
	this.Gridsize = aID;
	this.Levels = 0;
	this.Solved = 0;
	this.Perfectly = 0;
	this.GridSizeImageIndex = 6;
	this.LevelsImageIndex = 5;
	this.SolvedImageIndex = 3;
	this.PerfectlyImageIndex = 1;
	this.ShadowColor = "#404040";
	this.ShadowOffsetX = 1;
	this.ShadowOffsetY = 1;
	this.ShadowBlur = 2;
}
TDiffButton.prototype = Object.create(TStandardButton.prototype);
TDiffButton.prototype.constructor = TDiffButton;

 /**
 * @param {number} gridSizeIndex
 * @param {number} levelsIndex
 * @param {number} solvedIndex
 * @param {number} perfectlyIndex
 */
TDiffButton.prototype.setImageIndexes = function (gridSizeIndex, levelsIndex, solvedIndex, perfectlyIndex) {
	this.GridSizeImageIndex = gridSizeIndex;
	this.LevelsImageIndex = levelsIndex;
	this.SolvedImageIndex = solvedIndex;
	this.PerfectlyImageIndex = perfectlyIndex;
	this.NeedRepaint = true;
}

 /**
 * @param {number} aLevels
 */
TDiffButton.prototype.setLevels = function (aLevels) {
	this.Levels = 0 || aLevels;
	this.Visible = (this.Levels > 0);
	this.NeedRepaint = true;
}

 /**
 * @param {number} aSolved
 */
TDiffButton.prototype.setSolved = function (aSolved) {
	this.Solved = 0 || aSolved;
	this.NeedRepaint = true;
}

 /**
 * @param {number} aPerfectly
 */
TDiffButton.prototype.setPerfectly = function (aPerfectly) {
	this.Perfectly = 0 || aPerfectly;
	this.NeedRepaint = true;
}

TDiffButton.prototype.paintCaption = function () {
	if (this.Caption == "") return;
	var textOffset = 0;
	if (this.ButtonState == this.TButtonState.bsDown) textOffset = 2;
	gContext.fillStyle = this.FontColor;
	gContext.save();
	gContext.clip();
	try {
		gContext.textBaseline = "middle";
		gContext.textAlign = "left";
		gContext.font = "bold 16px Arial";
		var iconTop = 25;
		var textTop = iconTop + 12;

		var image = this.ImageList.get(this.GridSizeImageIndex);
		var metrics = gContext.measureText(this.Gridsize + "x" + this.Gridsize);
		var totalWidth = metrics.width + image.width + 4;
		var offsetImg = (this.Width / 4 - totalWidth) / 2;
		var offsetText = offsetImg + image.width + 4;
		gContext.drawImage(image, this.getLeft() + offsetImg + textOffset, this.getTop() + iconTop + textOffset);
		gContext.fillText(this.Gridsize + "x" + this.Gridsize, this.getLeft() + offsetText + textOffset, this.getTop() + textTop + textOffset);

		var image = this.ImageList.get(this.LevelsImageIndex);
		var metrics = gContext.measureText(this.Levels);
		var totalWidth = metrics.width + image.width + 4;
		var offsetImg = this.Width / 4 + (this.Width / 4 - totalWidth) / 2;
		var offsetText = offsetImg + image.width + 4;
		gContext.drawImage(image, this.getLeft() + offsetImg + textOffset, this.getTop() + iconTop + textOffset);
		gContext.fillText(this.Levels, this.getLeft() + offsetText + textOffset, this.getTop() + textTop + textOffset);

		var image = this.ImageList.get(this.SolvedImageIndex);
		var metrics = gContext.measureText(this.Solved);
		var totalWidth = metrics.width + image.width + 4;
		var offsetImg = this.Width / 2 + (this.Width / 4 - totalWidth) / 2;
		var offsetText = offsetImg + image.width + 4;
		gContext.drawImage(image, this.getLeft() + offsetImg + textOffset, this.getTop() + iconTop + textOffset);
		gContext.fillText(this.Solved, this.getLeft() + offsetText + textOffset, this.getTop() + textTop + textOffset);

		var image = this.ImageList.get(this.PerfectlyImageIndex);
		var metrics = gContext.measureText(this.Perfectly);
		var totalWidth = metrics.width + image.width + 4;
		var offsetImg = this.Width / 4 * 3 + (this.Width / 4 - totalWidth) / 2;
		var offsetText = offsetImg + image.width + 4;
		gContext.drawImage(image, this.getLeft() + offsetImg + textOffset, this.getTop() + iconTop + textOffset);
		gContext.fillText(this.Perfectly, this.getLeft() + offsetText + textOffset, this.getTop() + textTop + textOffset);

		gContext.shadowColor = this.ShadowColor;
		gContext.shadowOffsetX = this.ShadowOffsetX;
		gContext.shadowOffsetY = this.ShadowOffsetY;
		gContext.shadowBlur = this.ShadowBlur;
		gContext.font = this.Font;
		gContext.textBaseline = "top";
		gContext.textAlign = "center";
		gContext.fillText(this.Caption, this.getLeft() + this.Width / 2 + textOffset, this.getTop() + 4 + textOffset);
	} finally {
		gContext.restore();
	}
}

/**
 * @constructor
 * @extends TButton
 * @param {string} aCaption
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 * @param {function(string, number, boolean=)} callbackClick
 * @param {boolean=} aChecked
*/
function TCheckbox(aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, aChecked) {
	TButton.call(this, aCaption, aLeft, aTop, aWidth, aHeight, callbackClick, 0);
	this.Font = "bold 20px Arial";
	this.Checked = false || aChecked;
	this.BoxSize = 20;
}
TCheckbox.prototype = Object.create(TButton.prototype);
TCheckbox.prototype.constructor = TCheckbox;

 /**
 * @param {boolean} left
 */
TCheckbox.prototype.mouseUp = function (left) {
	if (!this.getVisible()) return;
	if (!left) return;
	if (this.ButtonState != this.TButtonState.bsDown) return;
	this.ButtonState = this.TButtonState.bsMouse;
	this.Checked = !this.Checked;
	this.NeedRepaint = true;
	this.OnClick.call(this, this.Caption, this.ID);
}

 /**
 * @const
 */
var gSolvedImage = new Image();
gSolvedImage.src = "images/OK30.png";

TCheckbox.prototype.paintBack = function () {
	gContext.beginPath();
	gContext.lineWidth = 3;
	var top = this.getTop() + (this.Height - this.BoxSize)/2;
	gContext.rect(this.getLeft() + ~~(gContext.lineWidth/2), top, this.BoxSize, this.BoxSize);
	gContext.strokeStyle = "silver";
	gContext.fillStyle = "#404040";
	if ((this.ButtonState==this.TButtonState.bsMouse)||(this.ButtonState==this.TButtonState.bsDown)) gContext.fill();
	gContext.stroke();
	if (this.Checked) gContext.drawImage(gSolvedImage, this.getLeft(), top-8);

	gContext.beginPath();
	gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height);
}

TCheckbox.prototype.paintCaption = function () {
	if (this.Caption == "") return;
	gContext.font = this.Font;
	if (this.ButtonState == this.TButtonState.bsDisabled) gContext.fillStyle = "#707070";
	else gContext.fillStyle = this.FontColor;

	gContext.save();
	gContext.clip();
	try {
		gContext.textBaseline = "middle";
		gContext.textAlign = "left";
		gContext.fillText(this.Caption, this.getLeft() + this.BoxSize + 10, this.getTop() + 2 + this.Height / 2);
	} finally {
		gContext.restore();
	}
}

/**
 * @constructor
 * @extends TPanel
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 */
function TPage(aLeft, aTop, aWidth, aHeight) {
	TPanel.call(this, aLeft, aTop, aWidth, aHeight);
	this.Controls = [];
}
TPage.prototype = Object.create(TPanel.prototype);
TPage.prototype.constructor = TPage;

TPage.prototype.paintBack = function () {
	if (!this.getVisible()) return;
	gContext.beginPath();
	gContext.rect(this.getLeft(), this.getTop(), this.Width-1, this.Height-1);
	gContext.fillStyle = this.Color;
	gContext.fill();
}

 /**
 * @return {boolean}
 */
TPage.prototype.update = function () {
	if (!this.getVisible()) return false;
	if ((this.Left >= 5) || (this.Left <= -5)) this.setLeft(this.Left / 1.5);
	else if (this.Left != 0) this.setLeft(0);
	if ((this.Top >= 5) || (this.Top <= -5)) this.setTop(this.Top / 1.5);
	else if (this.Top != 0) this.setTop(0);
	for (var i = 0; i < this.Controls.length; i++) this.NeedRepaint = this.Controls[i].update() || this.NeedRepaint;
	return TControl.prototype.update.call(this);
}

/**
 * @constructor
 * @extends TControl
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
*/
function TPageControl(aLeft, aTop, aWidth, aHeight) {
	TControl.call(this, "", aLeft, aTop, aWidth, aHeight);
	this.Pages = [];
	this.Activepage = null;
}
TPageControl.prototype = Object.create(TControl.prototype);
TPageControl.prototype.constructor = TPageControl;

 /**
 * @param {TPage} aPage
 */
TPageControl.prototype.addPage = function (aPage) {
	if (!aPage) return;
	this.Pages.push(aPage);
	aPage.setParent(this);
	if (this.Pages.length == 1) this.Activepage = aPage;
}

 /**
 * @return {TPage}
 */
TPageControl.prototype.createPage = function () {
	var page = new TPage(0, 0, this.Width, this.Height);
	this.Pages.push(page);
	page.setParent(this);
	if (this.Pages.length == 1) this.Activepage = page;
	else page.hide();
	return page;
}

 /**
 * @return {TPoint}
 */
TPageControl.prototype.getPageOffset = function () {
	var point = new TPoint();
	while ((point.X == 0) && (point.Y == 0)) {
		point.X = (Math.floor(Math.random() * 3)) - 1;
		point.Y = (Math.floor(Math.random() * 3)) - 1;
	}
	point.X = point.X * this.Width;
	point.Y = point.Y * this.Height;
	return point;
}

 /**
 * @param {TPage} aPage
 */
TPageControl.prototype.setPage = function (aPage) {
	if (!aPage) return;
	if (this.Activepage != aPage) {
		this.Activepage = aPage;
		for (var i = 0; i < this.Pages.length; i++) this.Pages[i].hide();

		var offset = this.getPageOffset();
		aPage.setLeft(offset.X);
		aPage.setTop(offset.Y);
	}
	this.Activepage.show();
	this.Activepage.mouseMove(gMouse.X, gMouse.Y);
}

TPageControl.prototype.paint = function () {
	if (this.Activepage) this.Activepage.paint();
}

 /**
 * @return {boolean}
 */
TPageControl.prototype.update = function () {
	if (this.Activepage) return (this.Activepage.update());
	else return false;
}

TPageControl.prototype.mouseOut = function () {
	if (this.Activepage) this.Activepage.mouseOut();
}

 /**
 * @param {number} x
 * @param {number} y
 */
TPageControl.prototype.mouseMove = function (x, y) {
	if (this.Activepage) this.Activepage.mouseMove(x, y);
}

 /**
 * @param {boolean} left
 */
TPageControl.prototype.mouseDown = function (left) {
	if (this.Activepage) this.Activepage.mouseDown(left);
}

 /**
 * @param {boolean} left
 */
TPageControl.prototype.mouseUp = function (left) {
	if (this.Activepage) this.Activepage.mouseUp(left);
}

/**
 * @constructor
 * @extends TPage
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 */
function TSlide(aLeft, aTop, aWidth, aHeight) {
	TPage.call(this, aLeft, aTop, aWidth, aHeight);
	this.TargetLeft = aLeft;
}
TSlide.prototype = Object.create(TPage.prototype);
TSlide.prototype.constructor = TSlide;

TSlide.prototype.clear = function () {
	this.Controls = [];
}

 /**
 * @return {boolean}
 */
TSlide.prototype.update = function () {
	if (!this.getVisible()) return false;
	if (this.Left > this.TargetLeft + 50) this.setLeft(this.Left - 50);
	else if (this.Left > this.TargetLeft) this.setLeft(this.TargetLeft);
	if (this.Left < this.TargetLeft - 50) this.setLeft(this.Left + 50);
	else if (this.Left < this.TargetLeft) this.setLeft(this.TargetLeft);

	for (var i = 0; i < this.Controls.length; i++) this.NeedRepaint = this.Controls[i].update() || this.NeedRepaint;
	return TControl.prototype.update.call(this);
}

/**
 * @constructor
 * @extends TControl
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aWidth
 * @param {number} aHeight
 */
function TSlideControl(aLeft, aTop, aWidth, aHeight) {
	TControl.call(this, "", aLeft, aTop, aWidth, aHeight);
	this.Pages = [];
	this.Controls = [];
	this.PageIndex = -1;
	this.TabHeight = 70;
	this.Color = "black";
	this.TabTop = this.Height - this.TabHeight + 10;
	this.ShowPrevButton = new TStandardButton("<", 0, this.TabTop, 50, 50, this.showPrevPage.bind(this));
	this.ShowNextButton = new TStandardButton(">", 0, this.TabTop, 50, 50, this.showNextPage.bind(this));
	this.ShowPrevButton.setParent(this);
	this.ShowNextButton.setParent(this);
	this.ShowPrevButton.disable();
	this.ShowNextButton.disable();
	this.Controls.push(this.ShowPrevButton);
	this.Controls.push(this.ShowNextButton);
	this.DotSize = 20;
	this.DotSpace = 15;
}
TSlideControl.prototype = Object.create(TControl.prototype);
TSlideControl.prototype.constructor = TSlideControl;

 /**
 * @param {string} aCaption
 * @param {number} aID
 */
TSlideControl.prototype.showPrevPage = function (aCaption, aID) {
	if (this.PageIndex <= 0) {
		this.ShowPrevButton.disable();
		return;
	}
	var curLeft = this.Pages[this.PageIndex].Left;
	var curWidth = this.Pages[this.PageIndex].Width;
	this.Pages[this.PageIndex].TargetLeft = curWidth + 10;
	this.PageIndex--;
	this.Pages[this.PageIndex].TargetLeft = 0;
	this.Pages[this.PageIndex].Left = curLeft - (curWidth + 1);
	this.Pages[this.PageIndex].show();
	this.updateButtonState();
	this.NeedRepaint = true;
}

 /**
 * @param {string} aCaption
 * @param {number} aID
 */
TSlideControl.prototype.showNextPage = function (aCaption, aID) {
	if (this.PageIndex >= this.Pages.length - 1) {
		this.ShowNextButton.disable();
		return;
	}
	var curLeft = this.Pages[this.PageIndex].Left;
	var curWidth = this.Pages[this.PageIndex].Width;
	this.Pages[this.PageIndex].TargetLeft = -curWidth - 10;
	this.PageIndex++;
	this.Pages[this.PageIndex].TargetLeft = 0;
	this.Pages[this.PageIndex].Left = curLeft + curWidth + 1;
	this.Pages[this.PageIndex].show();
	this.updateButtonState();
	this.NeedRepaint = true;
}

TSlideControl.prototype.updateButtonState = function () {
	if (this.PageIndex > 0) this.ShowPrevButton.enable();
	else this.ShowPrevButton.disable();
	if (this.PageIndex < this.Pages.length - 1) this.ShowNextButton.enable();
	else this.ShowNextButton.disable();
}

 /**
 * @return {TSlide}
 */
TSlideControl.prototype.createPage = function () {
	var slide = new TSlide(0, 0, this.Width, this.Height - this.TabHeight);
	this.Pages.push(slide);
	slide.setParent(this);
	if (this.Pages.length == 1) this.PageIndex = 0;
	else slide.hide();
	if (this.Pages.length > 1) {
		if (this.PageIndex > 0) this.ShowPrevButton.enable();
		if (this.PageIndex < this.Pages.length - 1) this.ShowNextButton.enable();
	}
	this.adjustButtons();
	return slide;
}

TSlideControl.prototype.clearSlides = function () {
	for (var i = 0; i < this.Pages.length; i++) this.Pages[i].clear();
	this.Pages = [];
	this.adjustButtons();
}

TSlideControl.prototype.adjustButtons = function () {
	var slideCount = this.Pages.length;
	if (slideCount > 1) {
		if (slideCount > 7) {
			this.DotSize = 10;
			this.DotSpace = 7;

		} else {
			this.DotSize = 20;
			this.DotSpace = 15;
		}
		var dotWidth = (slideCount * this.DotSize + (slideCount - 1) * this.DotSpace);
		var dotLeft = (this.Width - dotWidth) / 2;
		var drawHCenter = this.TabTop + this.TabHeight / 2;
		this.ShowPrevButton.setPosition(dotLeft - this.DotSpace - this.ShowPrevButton.Width, drawHCenter - this.ShowPrevButton.Height / 2);
		this.ShowNextButton.setPosition(dotLeft + dotWidth + this.DotSpace, drawHCenter - this.ShowPrevButton.Height / 2);
		this.ShowPrevButton.show();
		this.ShowNextButton.show();
	} else {
		this.ShowPrevButton.hide();
		this.ShowNextButton.hide();
	}
	this.NeedRepaint = true;
}

 /**
 * @param {number} index
 */
TSlideControl.prototype.setPageIndex = function (index) {
	if (!((index >= 0) && (index < this.Pages.length))) return;
	if (this.PageIndex != index) {
		this.PageIndex = index;
		for (var i = 0; i < this.Pages.length; i++) this.Pages[i].hide();
	}
	this.Pages[this.PageIndex].setLeft(0);
	this.Pages[this.PageIndex].TargetLeft = 0;
	this.Pages[this.PageIndex].show();
	this.updateButtonState();
	this.Pages[this.PageIndex].mouseMove(gMouse.X, gMouse.Y);
}

TSlideControl.prototype.paint = function () {
	if (!this.getVisible()) return;
	gContext.fillStyle = this.Parent.Color;
	if (this.Pages.length == 0) gContext.fillRect(this.getLeft(), this.getTop(), this.Width, this.Height - this.TabHeight);
	gContext.fillRect(this.getLeft(), this.getTop() + this.TabTop, this.Width, this.TabHeight);

	gContext.beginPath();
	gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height - this.TabHeight);
	gContext.save();
	gContext.clip();
	try {
		for (var i = 0; i < this.Pages.length; i++) this.Pages[i].paint();
	} finally {
		gContext.restore();
	}
	if (this.Pages.length > 1) {
		var dotLeft = this.ShowPrevButton.getLeft() + this.ShowPrevButton.Width + this.DotSpace;
		var drawHCenter = this.ShowPrevButton.getTop() + this.ShowPrevButton.Height / 2;

		for (var i = 0; i < this.Controls.length; i++) this.Controls[i].paint();
		for (var i = 0; i < this.Pages.length; i++) {
			if (i == this.PageIndex) gContext.fillStyle = "#FFFFFF";
			else gContext.fillStyle = "#AAAAAA";
			gContext.fillRect(dotLeft, drawHCenter - this.DotSize / 2, this.DotSize, this.DotSize);
			dotLeft += (this.DotSize + this.DotSpace);
		}
	}
}

 /**
 * @return {boolean}
 */
TSlideControl.prototype.update = function () {
	for (var i = 0; i < this.Pages.length; i++) this.NeedRepaint = this.Pages[i].update() || this.NeedRepaint;
	for (var i = 0; i < this.Controls.length; i++) this.NeedRepaint = this.Controls[i].update() || this.NeedRepaint;
	return TControl.prototype.update.call(this);
}

TSlideControl.prototype.mouseOut = function () {
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseOut();
	for (var i = 0; i < this.Pages.length; i++) this.Pages[i].mouseOut();
}

 /**
 * @param {number} x
 * @param {number} y
 */
TSlideControl.prototype.mouseMove = function (x, y) {
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseMove(x, y);
	for (var i = 0; i < this.Pages.length; i++) this.Pages[i].mouseMove(x, y);
}

 /**
 * @param {boolean} left
 */
TSlideControl.prototype.mouseDown = function (left) {
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseDown(left);
	for (var i = 0; i < this.Pages.length; i++) this.Pages[i].mouseDown(left);
}

 /**
 * @param {boolean} left
 */
TSlideControl.prototype.mouseUp = function (left) {
	for (var i = 0; i < this.Controls.length; i++) this.Controls[i].mouseUp(left);
	for (var i = 0; i < this.Pages.length; i++) this.Pages[i].mouseUp(left);
}

/**
 * @constructor
 * @param {TGameGrid} aGrid
 * @param {number} aX
 * @param {number} aY
 * @param {number} aCellSize
 */
function TGameGridCell(aGrid, aX, aY, aCellSize) {
	this.X = aX;
	this.Y = aY;
	this.Left = aX * aCellSize;
	this.Top = aY * aCellSize;
	this.xCenter = this.Left + aCellSize / 2;
	this.yCenter = this.Top + aCellSize / 2;
	this.IsDot = false;
}

TGameGridCell.prototype.setDot = function () {
	this.IsDot = true;
}

/**
 * @constructor
 * @param {TGameGrid} aGrid
 * @param {TGameGridCell} aStartCell
 * @param {TGameGridCell} aEndCell
 * @param {string} aColor
 */
function TGameGridLine(aGrid, aStartCell, aEndCell, aColor) {
	this.Grid = aGrid;
	this.StartCell = aStartCell;
	this.EndCell = aEndCell;
	this.Cells = [];
	this.Closed = false;
	this.Color = aColor;
	this.Active = false;
	this.StartCell.setDot();
	this.EndCell.setDot();
}

 /**
 * @param {TGameGridCell} Cell1
 * @param {TGameGridCell} Cell2
 * @return {boolean}
 */
TGameGridLine.prototype.isSameCell = function (Cell1, Cell2) {
	if ((Cell1.X != Cell2.X) || (Cell1.Y != Cell2.Y)) return false;
	return true;
}

 /**
 * @param {TGameGridCell} aCell
 */
TGameGridLine.prototype.moveTo = function (aCell) {
	if (!aCell || this.Closed) return;
	this.Cells.push(aCell);
	this.checkClosed();
}

TGameGridLine.prototype.checkClosed = function () {
	if (this.Cells.length >= 2) {
		var cell1 = this.Cells[0];
		var cell2 = this.Cells[this.Cells.length - 1];
		this.Closed = (this.isSameCell(cell1, this.EndCell) || this.isSameCell(cell1, this.StartCell)) &&
			(this.isSameCell(cell2, this.EndCell) || this.isSameCell(cell2, this.StartCell)) && !this.isSameCell(cell1, cell2);
	} else this.Closed = false;
}

 /**
 * @param {TGameGridCell} downCell
 */
TGameGridLine.prototype.mouseDown = function (downCell) {
	this.Active = false;
	if (downCell.IsDot) {
		if (this.isSameCell(downCell, this.StartCell) || this.isSameCell(downCell, this.EndCell)) {
			this.Cells.length = 0;
			this.Cells.push(downCell);
			this.checkClosed();
			this.Active = true;
		}
		return;
	}
	for (var i = 0; i < this.Cells.length; i++) {
		if (this.isSameCell(this.Cells[i], downCell)) {
			this.Cells.length = i + 1;
			this.checkClosed();
			this.Active = true;
			return;
		}
	}
}

TGameGridLine.prototype.mouseUp = function () {
	if (this.Active && this.Closed) {
		this.Grid.flashGrid();
		var Cell = this.Cells[this.Cells.length - 1];
		if (Cell) this.Grid.doSplash(this.Grid.getLeft()+Cell.xCenter, this.Grid.getTop()+Cell.yCenter);
	}
	this.Active = false;
}

 /**
 * @param {TGameGridCell} moveCell
 * @return {Array<TGameGridCell>}
 */
TGameGridLine.prototype.canMove = function (moveCell) {
	if (!this.Active) return [];

	if (moveCell.IsDot && !(this.isSameCell(moveCell, this.StartCell) || this.isSameCell(moveCell, this.EndCell))) return [];
	var lastCell = this.Cells[this.Cells.length - 1];
	if (this.isSameCell(moveCell, lastCell)) return [];

	var firstCell = this.Cells[0];
	var lineEndCell = this.StartCell;
	if (this.isSameCell(firstCell, this.StartCell)) lineEndCell = this.EndCell;

	var stepCells = [];
	var startX = lastCell.X;
	var startY = lastCell.Y;
	var endX = moveCell.X;
	var endY = moveCell.Y;
	while ((startX != endX) || (startY != endY)) {
		if ((startX == endX) || (startY == endY)) {
			if (startX == endX) {
				if (startY < endY) startY += 1;
				else startY -= 1;
			} else if (startY == endY) {
				if (startX < endX) startX += 1;
				else startX -= 1;
			}
			var Cell = this.Grid.Cells[startY][startX];
			if (Cell.IsDot && !this.isSameCell(Cell, this.StartCell) && !this.isSameCell(Cell, this.EndCell)) return [];
			stepCells.push(Cell);
		} else {
			var diffX = 0;
			var diffY = 0;
			if (startX < endX) {
				diffX = 1;
				if (startY < endY) diffY = 1;
				else if (startY > endY) diffY = -1;
			} else if (startX > endX) {
				diffX = -1;
				if (startY < endY) diffY = 1;
				else if (startY > endY) diffY = -1;
			}
			var Cell1 = this.Grid.Cells[startY + diffY][startX];
			var Cell2 = this.Grid.Cells[startY][startX + diffX];
			if (!Cell1 || !Cell2) return [];
			if (Cell1.IsDot && Cell2.IsDot) return [];
			if (Cell1.IsDot) {
				stepCells.push(Cell2)
				startX += diffX;
			} else if (Cell2.IsDot) {
				stepCells.push(Cell1);
				startY += diffY;
			} else {
				if (Math.abs(startX - endX) > Math.abs(startY - endY)) {
					stepCells.push(Cell2);
					startX += diffX;
				} else {
					stepCells.push(Cell1);
					startY += diffY;
				}
			}
		}
		if (this.isSameCell(stepCells[stepCells.length - 1], lineEndCell)) return stepCells;
	}
	return stepCells;
}

 /**
 * @param {TGameGridCell} moveCell
 */
TGameGridLine.prototype.mouseMove = function (moveCell) {
	if (!this.Active && moveCell.IsDot) return;

	for (var i = 0; i < this.Cells.length; i++) {
		if (this.isSameCell(this.Cells[i], moveCell)) {
			this.Cells.length = i;
			this.checkClosed();
			break;
		}
	}
	if (!this.Active) return;
	this.moveTo(moveCell);
}

 /**
 * @param {number} x0
 * @param {number} y0
 */
TGameGridLine.prototype.paint = function (x0, y0) {
	var cellSize = this.Grid.CellSize;
	gContext.save();
	try {
		gContext.beginPath();
		gContext.rect(x0 + this.StartCell.Left + 1, y0 + this.StartCell.Top + 1, cellSize , cellSize );
		gContext.rect(x0 + this.EndCell.Left + 1, y0 + this.EndCell.Top + 1, cellSize , cellSize );
		for (var i = 0; i < this.Cells.length; i++) gContext.rect(x0 + this.Cells[i].Left + 1, y0 + this.Cells[i].Top, cellSize , cellSize );
		gContext.clip();

		if (!(this.Grid.IsMouseDown && this.Active)) {
			gContext.beginPath();
			gContext.globalAlpha = 0.2;
			try {
				for (var i = 0; i < this.Cells.length; i++) {
					gContext.rect(x0 + this.Cells[i].Left, y0 + this.Cells[i].Top, cellSize, cellSize);
				}
				gContext.fillStyle = this.Color;
				gContext.fill();
			} finally {
				gContext.globalAlpha = 1;
			}
		}

		gContext.beginPath();
		gContext.lineCap = "round";
		gContext.lineJoin = "round";
		gContext.lineWidth = cellSize / 4;

		var radius = this.Grid.DotRadius - gContext.lineWidth / 2;
		gContext.moveTo(x0 + this.StartCell.xCenter, y0 + this.StartCell.yCenter);
		gContext.arc(x0 + this.StartCell.xCenter, y0 + this.StartCell.yCenter, radius, 0, 2 * Math.PI);
		gContext.moveTo(x0 + this.EndCell.xCenter, y0 + this.EndCell.yCenter);
		gContext.arc(x0 + this.EndCell.xCenter, y0 + this.EndCell.yCenter, radius, 0, 2 * Math.PI);

		if (this.Cells.length > 0) {
			gContext.moveTo(x0 + this.Cells[0].xCenter, y0 + this.Cells[0].yCenter);
			for (var i = 1; i < this.Cells.length; i++) {
				gContext.lineTo(x0 + this.Cells[i].xCenter, y0 + this.Cells[i].yCenter);
			}
		}
		if (this.Closed && !(this.Grid.IsMouseDown && this.Active)) {
			gContext.shadowOffsetX = 0;
			gContext.shadowOffsetY = 0;
			gContext.shadowColor = this.Color;
			gContext.strokeStyle = "white";
			if (gSettings.HQ) {
				gContext.shadowBlur = cellSize * 1.5;
				gContext.stroke();
				gContext.shadowBlur = cellSize;
				gContext.stroke();
				gContext.shadowBlur = cellSize / 2;
				gContext.stroke();
				gContext.shadowBlur = cellSize / 3;
				gContext.stroke();
				gContext.shadowColor = "white";
				gContext.shadowBlur = cellSize / 3;
			}
			gContext.stroke();
		} else {
			gContext.strokeStyle = this.Color;
			gContext.stroke();
		}
	} finally {
		gContext.restore();
	}
}

/**
 * @constructor
 * @extends TControl
 * @param {number} aTop
 * @param {number} aParentWidth
 * @param {number} aTargetWidth
 * @param {function(number, number)=} aOnMovesChanged
 * @param {function(number, number, boolean)=} aOnGameEnd
 */

function TGameGrid(aTop, aParentWidth, aTargetWidth, aOnMovesChanged, aOnGameEnd) {
	this.Cells = [];
	this.Lines = [];
	this.LevelData = [];
	this.Size = 5;
	this.ParentWidth = aParentWidth;
	this.TargetWidth = aTargetWidth;
	this.calcGridSize();
	TControl.call(this, "", this.Left, aTop, this.Width, this.Width);
	this.TargetLeft = this.Left;
	this.IsMouseDown = false;
	this.ReadOnly = false;
	this.FlashStep = 0;
	this.ActiveIndex = -1;
	this.Moves = 0;
	this.OnMovesChanged = aOnMovesChanged;
	this.OnGameEnd = aOnGameEnd;
	this.OnMovedOut = null;
}
TGameGrid.prototype = Object.create(TControl.prototype);
TGameGrid.prototype.constructor = TGameGrid;

TGameGrid.prototype.calcGridSize = function () {
	this.CellSize = ~~ (this.TargetWidth / this.Size);
	this.Width = this.CellSize * this.Size + 1;
	this.Height = this.Width;
	this.DotRadius = ~~ (this.CellSize * 0.3);
	this.setLeft(~~((this.ParentWidth - this.Width) / 2));
}

TGameGrid.prototype.initCells = function () {
	this.Cells.length = 0;
	for (var i = 0; i < this.Size; i++) {
		var data = [];
		data.length = this.Size;
		this.Cells.push(data);
	}
	for (var y = 0; y < this.Size; y++) {
		for (var x = 0; x < this.Size; x++) this.Cells[y][x] = new TGameGridCell(this, x, y, this.CellSize);
	}
}

 /**
 * @param {number} aColumnCount
 * @param {Array} aLevelData
 */
TGameGrid.prototype.setLevelData = function (aColumnCount, aLevelData) {
	this.Size = aColumnCount || 5;
	this.LevelData = aLevelData;
	this.calcGridSize();
	this.initCells();
	this.Lines.length = 0;
	this.FlashStep = 0;
	this.ActiveIndex = -1;
	this.Moves = 0;
	this.SplashList = [];
	for (var i = 0; i < this.LevelData.length; i++) {
		var startXY = this.LevelData[i][0];
		var endXY = this.LevelData[i][1];
		this.Lines.push(new TGameGridLine(this, this.Cells[startXY[1]][startXY[0]], this.Cells[endXY[1]][endXY[0]], this.getColor(i)));
	}
	this.NeedRepaint = true;
}

 /**
 * @param {number} aColorIndex
 * @return {string}
 */
TGameGrid.prototype.getColor = function (aColorIndex) {
	var DotColors = ["#FFF22D", "#4b4eff", "#1fd61a", "#d66e1a", "#d7191d", "#16B2AB", "#B2169A", "#ef9279", "#8344e8", "#c8ef79", "#69B2FF", "#d579ef"];
	if ((aColorIndex < 0) || (aColorIndex >= DotColors.length)) return "white";
	else return DotColors[aColorIndex];
}

TGameGrid.prototype.paint = function () {
	if (!this.getVisible()) return;
	var x0 = this.getLeft();
	var y0 = this.getTop();
	this.paintBack(x0, y0);
	this.paintLines(x0, y0);

	if (this.IsMouseDown) {
		var activeColor = "white";
		for (var i = 0; i < this.Lines.length; i++) {
			if (this.Lines[i].Active) {
				var activeColor = this.Lines[i].Color;
				break;
			}
		}
		if (activeColor != "white") {
			gContext.beginPath();
			gContext.rect(x0, y0, this.Width, this.Height);
			gContext.save();
			gContext.clip();
			try {
				gContext.beginPath();
				gContext.fillStyle = activeColor;
				gContext.arc(gMouse.X, gMouse.Y, this.CellSize / 2, 0, 2 * Math.PI);
				gContext.globalAlpha = 0.3;
				try {
					gContext.fill();
				} finally {
					gContext.globalAlpha = 1;
				}
			} finally {
				gContext.restore();
			}
		}
	}

	if (this.FlashStep > 0) {
		gContext.beginPath();
		gContext.rect(x0, y0, this.Width, this.Height);
		gContext.fillStyle = "white";
		gContext.globalAlpha = this.FlashStep / 20;
		try {
			gContext.fill();
		} finally {
			gContext.globalAlpha = 1;
		}
	}
	
	for (var i = 0; i < this.SplashList.length; i++) this.SplashList[i].paint();
}

 /**
 * @param {number} x0
 * @param {number} y0
 */
TGameGrid.prototype.paintBack = function (x0, y0) {
	gContext.beginPath();
	var xCenter = x0 + (this.Width / 2);
	var yCenter = y0 + (this.Height / 2);
	gContext.rect(this.getLeft(), this.getTop(), this.Width, this.Height);
	if (gSettings.HQ) {
		var fillGradient = gContext.createRadialGradient(xCenter, yCenter, this.Width * 2 / 3, xCenter, yCenter, 0);
		fillGradient.addColorStop(0, '#222222');
		fillGradient.addColorStop(1, '#555555');
		gContext.fillStyle = fillGradient;
	} else gContext.fillStyle = "#303030";
	gContext.fill();
	gContext.lineWidth = 1;
	gContext.strokeStyle = "silver";

	for (var y = 1; y < this.Size; y++) {
		gContext.moveTo(x0, y0 + y * this.CellSize);
		gContext.lineTo(x0 + this.Width, y0 + y * this.CellSize);
	}

	for (var x = 1; x < this.Size; x++) {
		gContext.moveTo(x0 + x * this.CellSize, y0);
		gContext.lineTo(x0 + x * this.CellSize, y0 + this.Height);
	}
	gContext.stroke();
}

 /**
 * @param {number} x0
 * @param {number} y0
 */
TGameGrid.prototype.paintLines = function (x0, y0) {
	for (var i = 0; i < this.Lines.length; i++) this.Lines[i].paint(x0, y0);
}

 /**
 * @return {boolean}
 */
TGameGrid.prototype.update = function () {
	if (!this.getVisible()) return false;
	if (this.Left > this.TargetLeft) {
		this.Left = this.Left - 50;
		if (this.Left < this.TargetLeft) {
			this.Left = this.TargetLeft;
			if (this.Left > 0) this.flashGrid();
			else if (this.OnMovedOut) this.OnMovedOut(this);
		}
		this.NeedRepaint = true;
	} else if (this.Left < this.TargetLeft) {
		this.Left = this.Left + 50;
		if (this.Left > this.TargetLeft) this.Left = this.TargetLeft;
		this.NeedRepaint = true;
	}
	if (this.FlashStep > 0) {
		this.FlashStep -= 1;
		this.NeedRepaint = true;
		if (this.FlashStep == 0) this.checkGrid();
	}
	var i = 0;
	while (i < this.SplashList.length) {
		this.SplashList[i].update();
		if (this.SplashList[i].Alpha < 0.1) this.SplashList.splice(i, 1);
		else i++;
		this.NeedRepaint = true;
	}
	return TControl.prototype.update.call(this);
}

TGameGrid.prototype.flashGrid = function () {
	this.FlashStep = 5;
	this.NeedRepaint = true;
}

TGameGrid.prototype.checkGrid = function () {
	var totalFields = 0;
	for (var i = 0; i < this.Lines.length; i++) {
		if (!this.Lines[i].Closed) return;
		totalFields += this.Lines[i].Cells.length;
	}
	var Almost = (totalFields < this.Size * this.Size);
	if (this.OnGameEnd) this.OnGameEnd(this.Moves, this.Lines.length, Almost);
}

 /**
 * @param {number} x
 * @param {number} y
 */
TGameGrid.prototype.doSplash = function (x, y) {
	this.SplashList.push(new TSplash(x, y, this.CellSize / 3, this.CellSize * 0.8));
}

TGameGrid.prototype.moveOut = function () {
	if (!this.getVisible()) return;
	this.TargetLeft = -gCanvasWidth + this.Left;
}

TGameGrid.prototype.moveIn = function () {
	if (!this.getVisible()) return;
	this.TargetLeft = this.Left;
	this.Left = gCanvasWidth + this.Left;
}

TGameGrid.prototype.mouseOut = function () {
	if (!this.getVisible()) return;
	if (this.ReadOnly) return;
	this.IsMouseDown = false;
	for (var i = 0; i < this.Lines.length; i++) this.Lines[i].mouseUp();
	this.NeedRepaint = true;
}

 /**
 * @param {number} x
 * @param {number} y
 */
TGameGrid.prototype.mouseMove = function (x, y) {
	if (!this.getVisible()) return;
	if (this.ReadOnly) return;
	if (this.IsMouseDown) {
		var xMouse = x - this.getLeft();
		var yMouse = y - this.getTop();
		var xField = ~~(xMouse / this.CellSize);
		var yField = ~~(yMouse / this.CellSize);
		if ((xField >= 0) && (yField >= 0) && (xField < this.Size) && (yField < this.Size)) {
			var moveCell = this.Cells[yField][xField];
			var stepCells = [];
			for (var i = 0; i < this.Lines.length; i++) {
				if (this.Lines[i].Active) {
					stepCells = this.Lines[i].canMove(moveCell);
					break;
				}
			}
			if (stepCells && (stepCells.length > 0)) {
				for (var i = 0; i < this.Lines.length; i++) {
					for (var j = 0; j < stepCells.length; j++) this.Lines[i].mouseMove(stepCells[j]);
				}
			}
		}
		this.NeedRepaint = true;
	}
}

 /**
 * @param {boolean} left
 */
TGameGrid.prototype.mouseDown = function (left) {
	this.IsMouseDownX = -1;
	this.IsMouseDownY = -1;
	if (!this.getVisible()) return;
	if (this.ReadOnly) return;

	var xMouse = gMouse.X - this.getLeft();
	var yMouse = gMouse.Y - this.getTop();
	if ((xMouse < 0) || (yMouse < 0) || (xMouse > this.Width) || (yMouse > this.Height)) return;
	var xField = ~~ (xMouse / this.CellSize);
	var yField = ~~ (yMouse / this.CellSize);
	if ((xField < 0) || (yField < 0) || (xField >= this.Size) || (yField >= this.Size)) return;
	this.IsMouseDown = true;

	var downCell = this.Cells[yField][xField];
	for (var i = 0; i < this.Lines.length; i++) this.Lines[i].mouseDown(downCell);
	for (var i = 0; i < this.Lines.length; i++) if (this.Lines[i].Active){
		if (this.ActiveIndex != i) {
			this.ActiveIndex = i;
			this.Moves += 1;
			if (this.OnMovesChanged) this.OnMovesChanged(this.Moves, this.Lines.length);
		}
		break;
	}
	this.NeedRepaint = true;
}

 /**
 * @param {boolean} left
 */
TGameGrid.prototype.mouseUp = function (left) {
	if (!this.getVisible()) return;
	if (this.ReadOnly) return;
	this.IsMouseDown = false;
	for (var i = 0; i < this.Lines.length; i++) this.Lines[i].mouseUp();
	this.NeedRepaint = true;
}

/**
 * @constructor
 * @extends TGameGrid
 * @param {number} aLeft
 * @param {number} aTop
 */
function THelpGrid(aLeft, aTop) {
	TGameGrid.call(this, aTop, 0, 0);
	this.Size = 4;
	this.CellSize = 37;
	this.Left = aLeft;
	this.TargetLeft = this.Left;
	this.Top = aTop;
	this.Width = this.CellSize * this.Size + 1;
	this.Height = this.Width;
	this.setLevelData(this.Size, [[[0,0],[0,3]],[[1,1],[3,0]],[[1,2],[3,1]],[[1,3],[3,2]]]);
	this.setDefaultData();
	this.ReadOnly = true;
	this.DotRadius = ~~(this.CellSize * 0.3);
};
THelpGrid.prototype = Object.create(TGameGrid.prototype);
THelpGrid.prototype.constructor = THelpGrid;

THelpGrid.prototype.calcGridSize = function (){
}

THelpGrid.prototype.setDefaultData = function () {
	this.Lines[0].moveTo(this.Cells[0][0]);
	this.Lines[0].moveTo(this.Cells[1][0]);
	this.Lines[0].moveTo(this.Cells[2][0]);
	this.Lines[0].moveTo(this.Cells[3][0]);
	this.Lines[0].Closed = true;
	this.NeedRepaint = true;
};

/**
 * @constructor
 * @extends THelpGrid
 * @param {number} aLeft
 * @param {number} aTop
 */
function THelpGrid2(aLeft, aTop) {
	THelpGrid.call(this, aLeft, aTop);
}
THelpGrid2.prototype = Object.create(THelpGrid.prototype);
THelpGrid2.prototype.constructor = THelpGrid2;

THelpGrid2.prototype.setDefaultData = function () {
	this.Lines[0].moveTo(this.Cells[0][0]);
	this.Lines[0].moveTo(this.Cells[1][0]);
	this.Lines[0].moveTo(this.Cells[2][0]);
	this.Lines[0].moveTo(this.Cells[3][0]);
	this.Lines[0].Closed = true;

	this.Lines[1].moveTo(this.Cells[0][3]);
	this.Lines[1].moveTo(this.Cells[0][2]);
	this.Lines[1].moveTo(this.Cells[0][1]);
	this.Lines[1].moveTo(this.Cells[1][1]);
	this.Lines[1].Closed = true;

	this.Lines[2].moveTo(this.Cells[1][3]);
	this.Lines[2].moveTo(this.Cells[1][2]);
	this.Lines[2].moveTo(this.Cells[2][2]);
	this.Lines[2].moveTo(this.Cells[2][1]);
	this.Lines[2].Closed = true;

	this.Lines[3].moveTo(this.Cells[2][3]);
	this.Lines[3].moveTo(this.Cells[3][3]);
	this.Lines[3].moveTo(this.Cells[3][2]);
	this.Lines[3].moveTo(this.Cells[3][1]);
	this.Lines[3].Closed = true;
	this.NeedRepaint = true;
}

/**
 * @constructor
 * @param {number} aLeft
 * @param {number} aTop
 * @param {number} aStartSize
 * @param {number} aEndSize
 */
function TSplash(aLeft, aTop, aStartSize, aEndSize) {
	this.Left = aLeft;
	this.Top = aTop;
	this.Size = aStartSize;
	this.LineWidth = 1;
	this.Alpha = 1;
	this.SizeStep = (aEndSize - aStartSize) / 11;
}

TSplash.prototype.update = function () {
	this.Alpha *= 0.75;
	this.Size += this.SizeStep;
	this.LineWidth += (this.SizeStep / 3);
}

TSplash.prototype.paint = function () {
	gContext.save();
	gContext.beginPath();
	gContext.lineWidth = this.LineWidth;
	gContext.globalAlpha = this.Alpha;
	gContext.arc(this.Left, this.Top, this.Size, 0, 2 * Math.PI);
	gContext.strokeStyle = "white";
	gContext.stroke();
	gContext.globalAlpha = 1;
	gContext.restore();
}

/**
 * @constructor
 */
function TNeonFlowGame() {
	var self = this;
	var processing = false;
	gSettings = new TSettings();

	this.Level5Data = [
	[[[0,0],[3,2]],[[2,1],[4,1]],[[4,0],[2,2]],[[4,2],[0,4]]],
	[[[1,1],[1,3]],[[3,2],[4,4]],[[4,0],[3,4]],[[3,0],[2,4]]],
	[[[1,0],[4,4]],[[2,0],[2,2]],[[4,0],[2,3]],[[2,1],[1,3]]],
	[[[0,0],[2,0]],[[4,1],[2,2]],[[4,0],[2,3]],[[2,1],[0,4]]],
	[[[0,3],[2,4]],[[1,3],[3,3]],[[4,0],[3,4]],[[1,1],[4,1]]],
	[[[1,0],[4,2]],[[3,0],[2,3]],[[4,0],[3,3]],[[1,1],[1,3]]],
	[[[0,0],[3,4]],[[2,2],[0,4]],[[1,3],[2,4]],[[0,1],[2,3]]],
	[[[3,2],[1,3]],[[2,2],[4,4]],[[3,3],[1,4]]],
	[[[0,0],[2,3]],[[2,1],[3,3]],[[4,0],[3,2]],[[0,4],[4,4]]],
	[[[0,4],[3,4]],[[1,3],[4,4]],[[3,0],[4,2]],[[1,1],[2,3]]],

	[[[4,1],[1,2]],[[4,2],[3,3]],[[4,0],[4,4]],[[3,2],[1,3]]],
	[[[0,0],[2,2]],[[4,1],[1,2]],[[2,0],[4,0]],[[0,1],[4,2]]],
	[[[3,0],[0,1]],[[0,3],[4,3]],[[2,2],[1,3]],[[4,0],[0,2]],[[4,2],[3,3]]],
	[[[0,3],[4,4]],[[4,2],[1,3]],[[3,0],[3,2]],[[4,0],[1,1]]],
	[[[2,0],[0,3]],[[3,1],[1,2]],[[3,0],[3,4]],[[1,3],[0,4]],[[3,3],[4,4]]],
	[[[4,2],[1,3]],[[4,1],[4,4]],[[2,2],[4,3]]],
	[[[4,2],[0,4]],[[2,2],[4,3]],[[3,3],[1,4]]],
	[[[0,0],[4,1]],[[1,3],[3,3]],[[3,1],[3,4]],[[1,2],[4,4]]],
	[[[0,0],[3,3]],[[2,2],[0,4]],[[2,0],[4,3]],[[0,1],[0,3]],[[2,3],[4,4]]],
	[[[1,3],[0,4]],[[3,3],[1,4]],[[4,0],[3,2]],[[1,1],[3,4]]],

	[[[0,3],[4,4]],[[4,1],[1,3]],[[2,0],[3,2]],[[4,0],[1,1]]],
	[[[1,3],[3,3]],[[2,2],[4,3]],[[4,2],[1,4]]],
	[[[0,0],[2,4]],[[3,0],[2,3]],[[4,0],[3,3]],[[0,1],[0,4]],[[4,2],[3,4]]],
	[[[0,3],[1,4]],[[4,1],[1,3]],[[2,0],[3,3]],[[4,0],[1,1]]],
	[[[1,1],[4,1]],[[4,0],[3,2]],[[4,2],[3,4]],[[2,2],[1,3]]],
	[[[3,0],[0,4]],[[3,2],[3,4]],[[4,0],[4,2]],[[2,2],[4,4]]],
	[[[4,2],[0,3]],[[1,1],[4,4]],[[1,2],[3,4]],[[1,3],[0,4]]],
	[[[1,0],[3,1]],[[2,0],[4,4]],[[3,2],[3,4]],[[1,1],[1,3]]],
	[[[0,0],[1,3]],[[0,3],[4,4]],[[2,0],[4,3]],[[0,1],[2,2]]],
	[[[4,1],[0,2]],[[4,3],[1,4]],[[3,3],[4,4]],[[3,1],[0,4]]],

	[[[0,0],[2,0]],[[4,0],[1,2]],[[2,1],[3,3]],[[4,1],[0,4]]],
	[[[0,0],[1,1]],[[2,2],[0,3]],[[1,0],[4,0]],[[0,2],[3,2]]],
	[[[1,0],[3,1]],[[2,0],[3,3]],[[4,3],[3,4]],[[1,1],[1,3]]],
	[[[1,0],[0,4]],[[3,3],[1,4]],[[2,1],[2,4]],[[1,1],[3,1]]],
	[[[0,0],[4,1]],[[3,2],[0,4]],[[2,2],[1,3]],[[3,1],[0,3]],[[4,2],[3,4]]],
	[[[1,0],[0,4]],[[4,2],[2,3]],[[4,1],[4,4]],[[4,0],[1,1]]],
	[[[0,0],[1,3]],[[1,1],[4,4]],[[3,1],[2,3]],[[3,2],[0,3]]],
	[[[2,0],[0,2]],[[0,4],[3,4]],[[4,1],[2,3]],[[4,0],[0,3]],[[3,3],[4,4]]],
	[[[0,0],[4,1]],[[3,3],[0,4]],[[1,2],[4,2]],[[3,1],[0,2]],[[4,3],[1,4]]],
	[[[0,0],[2,2]],[[2,0],[1,2]],[[4,0],[4,3]],[[0,1],[4,4]]],

	[[[0,0],[2,2]],[[1,1],[1,4]],[[2,1],[0,4]]],
	[[[0,0],[3,3]],[[0,2],[0,4]],[[3,0],[1,3]],[[0,1],[2,3]]],
	[[[0,2],[4,4]],[[3,1],[1,4]],[[2,1],[2,3]],[[1,1],[0,4]]],
	[[[0,0],[4,4]],[[0,3],[3,4]],[[2,3],[0,4]],[[3,1],[0,2]]],
	[[[4,0],[0,2]],[[3,2],[1,4]],[[2,3],[4,4]],[[1,1],[0,4]]],
	[[[0,0],[3,1]],[[4,0],[3,4]],[[1,2],[4,4]]],
	[[[0,0],[1,3]],[[2,1],[3,4]],[[3,1],[3,3]],[[1,0],[0,4]]],
	[[[0,0],[1,3]],[[1,0],[3,0]],[[2,0],[3,2]],[[0,4],[4,4]]],
	[[[2,2],[0,3]],[[1,2],[3,3]],[[1,3],[0,4]]],
	[[[0,0],[1,3]],[[2,2],[1,4]],[[0,1],[3,3]]],

	[[[0,0],[1,3]],[[4,0],[1,2]],[[2,2],[4,4]],[[2,3],[0,4]],[[3,2],[3,4]]],
	[[[0,2],[4,4]],[[1,1],[3,4]],[[2,2],[1,4]],[[1,2],[0,4]]],
	[[[3,3],[0,4]],[[2,2],[4,4]],[[4,0],[4,3]],[[2,1],[1,4]]],
	[[[1,3],[4,3]],[[3,2],[4,4]],[[4,0],[2,2]]],
	[[[3,1],[0,4]],[[1,3],[4,3]],[[3,0],[3,3]],[[1,1],[2,3]]],
	[[[0,2],[2,2]],[[3,1],[1,3]],[[2,3],[4,4]],[[0,3],[3,4]]],
	[[[0,1],[1,3]],[[4,1],[0,4]],[[4,0],[3,3]],[[1,1],[0,3]]],
	[[[0,3],[4,3]],[[1,1],[3,1]],[[2,0],[1,3]],[[1,0],[2,2]]],
	[[[1,1],[3,2]],[[4,0],[4,2]],[[4,1],[2,2]]],
	[[[0,1],[2,2]],[[1,1],[3,1]],[[3,0],[0,2]]],

	[[[0,0],[1,3]],[[0,2],[3,4]],[[3,0],[4,4]],[[0,1],[2,2]]],
	[[[1,0],[4,3]],[[2,0],[3,3]],[[2,2],[2,3]],[[3,1],[1,3]]],
	[[[0,0],[3,2]],[[0,3],[3,3]],[[1,0],[0,4]],[[0,2],[2,2]]],
	[[[3,1],[0,2]],[[1,1],[3,3]],[[3,0],[1,3]],[[0,3],[1,4]]],
	[[[0,0],[1,2]],[[4,1],[1,3]],[[4,0],[3,2]],[[1,1],[4,4]]],
	[[[0,0],[2,1]],[[1,3],[3,3]],[[4,0],[3,4]],[[1,0],[2,4]]],
	[[[0,0],[1,2]],[[3,3],[0,4]],[[4,0],[2,4]],[[2,1],[0,3]]],
	[[[2,2],[0,3]],[[1,1],[3,4]],[[2,0],[4,4]],[[1,0],[1,3]]],
	[[[0,0],[2,3]],[[4,0],[1,2]],[[3,1],[4,4]],[[2,1],[0,4]]],
	[[[0,0],[1,3]],[[1,0],[3,1]],[[2,0],[3,2]],[[0,3],[4,4]],[[2,2],[4,3]]],

	[[[4,2],[0,3]],[[3,2],[1,3]],[[3,0],[2,2]],[[4,0],[1,1]]],
	[[[0,0],[2,2]],[[4,0],[2,3]],[[4,2],[4,4]],[[2,1],[4,1]]],
	[[[4,0],[1,2]],[[0,1],[2,1]],[[0,0],[2,2]],[[0,2],[4,4]]],
	[[[3,1],[3,3]],[[1,2],[0,4]],[[0,0],[1,4]],[[1,0],[2,4]]],
	[[[3,0],[0,4]],[[2,0],[2,2]],[[0,0],[2,3]],[[2,1],[3,3]]],
	[[[2,0],[4,0]],[[0,1],[2,2]],[[0,0],[2,3]],[[2,1],[4,4]]],
	[[[4,3],[2,4]],[[1,3],[3,3]],[[0,0],[1,4]],[[0,1],[3,1]]],
	[[[3,0],[0,2]],[[1,0],[2,3]],[[0,0],[1,3]],[[3,1],[3,3]]],
	[[[4,0],[1,4]],[[2,2],[4,4]],[[3,3],[2,4]],[[4,1],[2,3]]],
	[[[1,2],[3,3]],[[2,2],[0,4]],[[1,3],[3,4]]],

	[[[4,0],[2,3]],[[2,1],[1,3]],[[0,0],[1,2]],[[0,4],[4,4]]],
	[[[1,4],[4,4]],[[3,3],[0,4]],[[1,0],[0,2]],[[3,1],[2,3]]],
	[[[0,1],[3,2]],[[0,2],[1,3]],[[0,0],[0,4]],[[1,2],[3,3]]],
	[[[4,0],[2,2]],[[0,1],[3,2]],[[0,0],[2,0]],[[4,1],[0,2]]],
	[[[1,0],[4,1]],[[0,3],[4,3]],[[2,2],[3,3]],[[0,0],[4,2]],[[0,2],[1,3]]],
	[[[4,3],[0,4]],[[0,2],[3,3]],[[1,0],[1,2]],[[0,0],[3,1]]],
	[[[2,0],[4,3]],[[1,1],[3,2]],[[1,0],[1,4]],[[3,3],[4,4]],[[1,3],[0,4]]],
	[[[0,2],[3,3]],[[0,1],[0,4]],[[2,2],[0,3]]],
	[[[0,2],[4,4]],[[2,2],[0,3]],[[1,3],[3,4]]],

	[[[4,0],[0,1]],[[1,3],[3,3]],[[1,1],[1,4]],[[3,2],[0,4]]],
	[[[4,0],[1,3]],[[2,2],[4,4]],[[2,0],[0,3]],[[4,1],[4,3]],[[2,3],[0,4]]],
	[[[3,3],[4,4]],[[1,3],[3,4]],[[0,0],[1,2]],[[3,1],[1,4]]],
	[[[4,3],[0,4]],[[0,1],[3,3]],[[2,0],[1,2]],[[0,0],[3,1]]],
	[[[1,3],[3,3]],[[2,2],[0,3]],[[0,2],[3,4]]],
	[[[4,0],[2,4]],[[1,0],[2,3]],[[0,0],[1,3]],[[4,1],[4,4]],[[0,2],[1,4]]],
	[[[4,3],[3,4]],[[0,1],[3,3]],[[2,0],[1,3]],[[0,0],[3,1]]],
	[[[0,1],[3,1]],[[0,0],[1,2]],[[0,2],[1,4]],[[2,2],[3,3]]],
	[[[1,0],[4,4]],[[1,2],[1,4]],[[0,0],[0,2]],[[2,2],[0,4]]],
	[[[0,2],[4,3]],[[3,1],[0,4]],[[3,2],[1,4]],[[3,3],[4,4]]],
	[[[3,0],[1,1]],[[2,0],[0,4]],[[1,2],[1,4]],[[3,1],[3,3]]],

	[[[4,0],[3,3]],[[4,3],[0,4]],[[2,0],[0,3]],[[4,1],[2,2]]],
	[[[0,1],[4,2]],[[0,3],[3,4]],[[1,3],[0,4]],[[1,1],[4,4]]],
	[[[2,0],[4,0]],[[0,0],[3,2]],[[2,1],[1,3]],[[0,1],[4,4]]],
	[[[4,0],[3,1]],[[2,2],[4,3]],[[0,0],[3,0]],[[1,2],[4,2]]],
	[[[3,0],[1,1]],[[2,0],[1,3]],[[0,3],[1,4]],[[3,1],[3,3]]],
	[[[3,0],[4,4]],[[1,3],[3,4]],[[2,1],[2,4]],[[1,1],[3,1]]],
	[[[4,0],[0,1]],[[1,2],[4,4]],[[2,2],[3,3]],[[1,1],[4,3]],[[0,2],[1,4]]],
	[[[3,0],[4,4]],[[0,2],[2,3]],[[0,1],[0,4]],[[0,0],[3,1]]],
	[[[4,0],[3,3]],[[3,1],[0,4]],[[1,1],[2,3]],[[1,2],[4,3]]],
	[[[2,0],[4,2]],[[1,4],[4,4]],[[0,1],[2,3]],[[0,0],[4,3]],[[1,3],[0,4]]],

	[[[4,0],[0,1]],[[1,3],[4,4]],[[0,2],[3,2]],[[1,1],[4,2]],[[0,3],[3,4]]],
	[[[4,0],[2,2]],[[2,0],[3,2]],[[0,0],[0,3]],[[4,1],[0,4]]],
	[[[4,0],[2,2]],[[3,1],[3,4]],[[2,1],[4,4]]],
	[[[4,0],[1,3]],[[4,2],[4,4]],[[1,0],[3,3]],[[4,1],[2,3]]],
	[[[4,2],[0,4]],[[1,1],[3,4]],[[2,1],[2,3]],[[3,1],[4,4]]],
	[[[4,0],[0,4]],[[4,3],[1,4]],[[2,3],[4,4]],[[1,1],[4,2]]],
	[[[0,0],[4,2]],[[1,2],[3,4]],[[2,3],[0,4]],[[3,1],[4,4]]],
	[[[4,0],[1,1]],[[0,0],[1,4]],[[3,2],[0,4]]],
	[[[4,0],[3,3]],[[2,1],[1,4]],[[1,1],[1,3]],[[3,0],[4,4]]],
	[[[4,0],[3,3]],[[1,0],[3,0]],[[2,0],[1,2]],[[0,4],[4,4]]],

	[[[2,2],[4,3]],[[3,2],[1,3]],[[3,3],[4,4]]],
	[[[4,0],[3,3]],[[2,2],[3,4]],[[4,1],[1,3]]],
	[[[4,0],[3,3]],[[0,0],[3,2]],[[2,2],[0,4]],[[2,3],[4,4]],[[1,2],[1,4]]],
	[[[4,2],[0,4]],[[3,1],[1,4]],[[2,2],[3,4]],[[3,2],[4,4]]],
	[[[1,3],[4,4]],[[2,2],[0,4]],[[0,0],[0,3]],[[2,1],[3,4]]],
	[[[0,3],[3,3]],[[1,2],[0,4]],[[0,0],[2,2]]],
	[[[1,1],[4,4]],[[0,3],[3,3]],[[1,0],[1,3]],[[3,1],[2,3]]],
	[[[2,2],[4,2]],[[1,1],[3,3]],[[2,3],[0,4]],[[4,3],[1,4]]],
	[[[4,1],[3,3]],[[0,1],[4,4]],[[0,0],[1,3]],[[3,1],[4,3]]],
	[[[0,3],[4,3]],[[1,1],[3,1]],[[2,0],[3,3]],[[3,0],[2,2]]],

	[[[3,1],[1,2]],[[0,0],[0,2]],[[0,1],[2,2]]],
	[[[4,1],[2,2]],[[1,1],[3,1]],[[1,0],[4,2]]],
	[[[4,0],[3,3]],[[4,2],[1,4]],[[1,0],[0,4]],[[4,1],[2,2]]],
	[[[3,0],[0,3]],[[2,0],[1,3]],[[2,2],[2,3]],[[1,1],[3,3]]],
	[[[4,0],[1,2]],[[1,3],[4,3]],[[3,0],[4,4]],[[2,2],[4,2]]],
	[[[1,1],[4,2]],[[3,1],[1,3]],[[1,0],[3,3]],[[4,3],[3,4]]],
	[[[4,0],[3,2]],[[0,1],[3,3]],[[0,0],[1,2]],[[3,1],[0,4]]],
	[[[4,0],[2,1]],[[1,3],[3,3]],[[0,0],[1,4]],[[3,0],[2,4]]],
	[[[4,0],[3,2]],[[1,3],[4,4]],[[0,0],[2,4]],[[2,1],[4,3]]],
	[[[2,2],[4,3]],[[3,1],[1,4]],[[2,0],[0,4]],[[3,0],[3,3]]],

	[[[4,0],[2,3]],[[0,0],[3,2]],[[1,1],[0,4]],[[2,1],[4,4]]],
	[[[4,0],[3,3]],[[3,0],[1,1]],[[2,0],[1,2]],[[4,3],[0,4]],[[2,2],[0,3]]],
	[[[0,2],[4,3]],[[1,2],[3,3]],[[1,0],[2,2]],[[0,0],[3,1]]],
	[[[4,0],[2,2]],[[0,0],[2,3]],[[0,2],[0,4]],[[0,1],[2,1]]]
	];

	this.Level6Data = [
	[[[0,0],[2,4]],[[2,1],[1,4]],[[3,1],[2,3]],[[1,1],[3,4]],[[4,1],[4,4]]],
	[[[0,0],[5,0]],[[1,4],[3,5]],[[4,2],[2,5]],[[1,1],[1,5]],[[5,4],[4,5]]],
	[[[4,0],[0,3]],[[1,1],[3,1]],[[2,3],[4,3]],[[5,2],[0,4]],[[5,0],[4,4]]],
	[[[1,1],[2,5]],[[3,4],[1,5]],[[4,0],[3,5]],[[1,2],[1,4]],[[4,1],[4,4]]],
	[[[0,1],[5,5]],[[0,5],[2,5]],[[2,2],[4,2]],[[0,2],[3,5]],[[4,1],[2,3]],[[3,3],[4,5]]],
	[[[1,0],[0,2]],[[4,0],[1,4]],[[3,1],[2,3]],[[5,0],[0,3]],[[2,4],[4,4]],[[4,1],[3,3]]],
	[[[0,1],[1,5]],[[3,4],[2,5]],[[4,1],[3,5]],[[1,1],[0,5]],[[5,1],[4,4]]],
	[[[0,0],[2,5]],[[4,1],[2,4]],[[5,0],[3,1]],[[2,1],[4,4]],[[5,3],[3,4]]],
	[[[0,2],[2,3]],[[1,1],[3,2]],[[1,2],[4,4]],[[0,3],[1,5]],[[4,0],[2,5]]],
	[[[1,0],[3,5]],[[4,1],[1,2]],[[1,3],[4,4]],[[2,0],[1,1]],[[3,0],[2,3]],[[5,3],[4,5]]],

	[[[4,0],[1,1]],[[2,0],[4,1]],[[3,3],[5,4]],[[5,0],[1,4]]],
	[[[0,0],[4,2]],[[5,0],[2,3]],[[4,1],[3,5]],[[1,1],[2,5]]],
	[[[0,0],[4,1]],[[1,4],[4,4]],[[5,3],[1,5]],[[1,0],[5,2]],[[2,4],[5,4]]],
	[[[0,0],[5,3]],[[2,0],[5,0]],[[2,3],[4,4]],[[1,0],[2,1]],[[5,1],[2,4]],[[5,2],[4,3]]],
	[[[1,1],[1,4]],[[4,2],[3,4]],[[5,0],[5,5]],[[3,2],[4,4]],[[5,1],[5,4]]],
	[[[5,0],[0,1]],[[1,1],[5,2]],[[1,4],[3,5]],[[4,2],[0,5]],[[4,4],[1,5]]],
	[[[1,0],[0,5]],[[2,1],[2,5]],[[2,4],[5,5]],[[1,1],[3,1]],[[5,0],[4,4]]],
	[[[0,0],[1,1]],[[0,3],[2,3]],[[5,0],[0,4]],[[0,1],[2,1]],[[5,1],[0,5]],[[2,0],[3,3]]],
	[[[0,0],[1,4]],[[2,1],[4,3]],[[4,2],[5,3]],[[0,4],[5,5]],[[5,0],[5,2]]],
	[[[1,0],[3,5]],[[2,0],[5,0]],[[3,2],[5,5]],[[1,1],[3,4]],[[5,1],[4,5]]],

	[[[0,0],[1,5]],[[2,1],[5,2]],[[5,0],[3,4]],[[2,0],[1,4]],[[5,1],[4,4]]],
	[[[0,0],[3,0]],[[5,0],[0,5]],[[5,1],[1,5]],[[1,1],[0,3]],[[5,3],[3,5]],[[5,4],[4,5]]],
	[[[0,0],[4,0]],[[0,2],[2,2]],[[3,2],[0,4]],[[5,0],[0,1]],[[5,2],[0,5]],[[4,2],[1,4]]],
	[[[0,0],[1,2]],[[0,5],[5,5]],[[1,0],[1,3]],[[0,2],[4,3]],[[3,1],[4,4]],[[4,1],[3,2]]],
	[[[0,0],[2,2]],[[1,0],[2,3]],[[2,0],[1,3]],[[0,3],[2,5]],[[4,1],[4,4]]],
	[[[0,2],[3,3]],[[3,1],[5,5]],[[5,0],[3,4]],[[3,2],[1,4]]],
	[[[5,0],[0,5]],[[5,3],[1,5]],[[2,1],[2,3]],[[1,1],[5,2]],[[4,2],[4,4]]],
	[[[0,4],[4,5]],[[1,4],[5,5]],[[3,3],[5,3]],[[5,1],[1,2]],[[5,0],[5,2]]],
	[[[1,1],[4,1]],[[3,0],[3,5]],[[3,1],[3,4]],[[1,4],[4,4]]],
	[[[0,0],[4,4]],[[4,1],[4,5]],[[2,3],[5,5]]],

	[[[0,4],[4,5]],[[2,1],[1,3]],[[1,4],[5,5]],[[1,1],[2,3]],[[4,1],[5,3]],[[5,1],[4,3]]],
	[[[0,0],[4,3]],[[1,0],[3,3]],[[2,0],[4,2]],[[4,0],[0,5]],[[2,1],[3,2]]],
	[[[0,0],[4,1]],[[3,0],[2,3]],[[4,0],[5,5]],[[1,0],[3,1]]],
	[[[0,0],[2,5]],[[1,2],[1,5]],[[2,0],[5,5]],[[0,1],[0,5]],[[3,1],[3,5]],[[4,1],[4,5]]],
	[[[0,2],[3,3]],[[2,2],[0,5]],[[5,0],[1,5]],[[3,1],[0,3]],[[5,4],[4,5]]],
	[[[0,0],[0,1]],[[1,3],[2,5]],[[2,3],[1,5]],[[1,1],[3,2]],[[2,1],[4,3]],[[3,5],[5,5]]],
	[[[2,0],[0,2]],[[1,2],[2,5]],[[4,0],[1,4]],[[1,1],[4,4]],[[5,0],[4,1]]],
	[[[0,0],[0,4]],[[2,1],[5,5]],[[4,0],[4,1]],[[0,5],[3,5]],[[5,0],[4,5]]],
	[[[0,0],[1,3]],[[2,0],[0,5]],[[5,0],[5,5]],[[1,0],[0,3]],[[5,1],[5,4]]],
	[[[0,0],[4,3]],[[5,3],[1,4]],[[3,2],[5,5]],[[1,1],[5,2]]],

	[[[4,1],[2,2]],[[3,1],[5,5]],[[5,1],[4,5]],[[5,2],[2,3]]],
	[[[0,3],[2,4]],[[1,1],[3,5]],[[1,3],[3,3]],[[0,5],[2,5]],[[5,0],[3,2]],[[4,2],[3,4]]],
	[[[0,0],[1,5]],[[1,4],[4,4]],[[4,1],[2,3]],[[4,2],[1,3]],[[3,2],[2,5]]],
	[[[0,0],[2,3]],[[5,0],[1,1]],[[4,1],[4,4]],[[0,1],[5,1]]],
	[[[1,1],[4,2]],[[2,3],[5,3]],[[5,2],[4,3]],[[1,4],[4,4]]],
	[[[0,0],[2,5]],[[5,0],[1,4]],[[3,1],[3,5]],[[1,0],[3,3]]],
	[[[3,0],[0,1]],[[1,1],[5,1]],[[5,0],[2,4]],[[2,2],[0,5]]],
	[[[0,1],[1,4]],[[1,1],[0,5]],[[0,2],[3,3]]],
	[[[1,1],[4,5]],[[3,3],[3,5]],[[4,0],[4,1]],[[3,0],[5,5]],[[5,0],[5,2]]],
	[[[0,5],[1,5]],[[2,0],[5,0]],[[2,2],[3,4]],[[1,0],[4,5]],[[3,3],[5,5]]],

	[[[0,3],[3,4]],[[4,0],[1,1]],[[2,0],[4,1]],[[3,3],[0,4]],[[5,0],[5,5]]],
	[[[1,1],[2,3]],[[2,1],[5,3]],[[5,2],[3,3]],[[4,1],[1,3]]],
	[[[0,0],[2,4]],[[2,2],[0,5]],[[2,3],[1,5]],[[0,1],[3,4]],[[4,1],[4,4]]],
	[[[5,0],[0,1]],[[5,1],[1,5]],[[2,2],[3,4]],[[0,2],[0,5]],[[2,3],[2,5]],[[3,3],[4,4]]],
	[[[0,1],[4,2]],[[1,3],[5,4]],[[2,1],[3,2]],[[1,1],[0,5]],[[4,0],[2,3]],[[2,4],[4,4]]],
	[[[0,0],[5,3]],[[3,0],[2,3]],[[5,0],[3,3]],[[1,0],[4,3]],[[5,1],[4,2]]],
	[[[1,0],[3,5]],[[1,3],[4,4]],[[2,1],[4,1]],[[1,1],[4,5]],[[5,0],[3,1]]],
	[[[0,2],[3,5]],[[3,3],[0,5]],[[1,2],[2,5]],[[0,3],[0,4]],[[2,2],[3,2]]],
	[[[0,4],[4,4]],[[4,1],[1,2]],[[5,1],[2,2]],[[3,3],[0,5]],[[3,2],[5,2]]],
	[[[4,2],[0,4]],[[2,1],[4,4]],[[4,1],[2,2]],[[2,3],[0,5]]],

	[[[0,5],[2,5]],[[3,2],[1,5]],[[2,1],[2,4]],[[1,1],[2,2]]],
	[[[0,0],[4,4]],[[5,0],[1,5]],[[4,0],[2,3]],[[2,1],[0,5]]],
	[[[5,0],[0,5]],[[1,2],[2,5]],[[4,2],[3,5]],[[1,1],[3,4]],[[5,1],[4,4]]],
	[[[4,1],[0,2]],[[4,2],[1,4]],[[3,0],[4,4]],[[1,1],[2,5]],[[3,3],[5,4]]],
	[[[0,0],[3,3]],[[5,3],[1,5]],[[2,1],[5,2]],[[4,2],[0,5]],[[5,0],[3,2]]],
	[[[5,0],[0,2]],[[3,2],[0,4]],[[4,4],[4,5]],[[0,3],[0,5]],[[5,1],[5,5]]],
	[[[0,0],[1,5]],[[4,3],[1,4]],[[3,2],[5,5]],[[1,1],[3,4]]],
	[[[1,2],[0,4]],[[1,1],[5,4]],[[2,0],[1,3]],[[3,2],[0,5]],[[4,1],[5,5]]],
	[[[0,1],[3,3]],[[1,1],[2,3]],[[5,0],[1,2]],[[5,1],[0,2]]],
	[[[3,1],[1,4]],[[3,0],[4,4]],[[4,1],[5,4]],[[2,0],[2,2]]],

	[[[0,0],[4,2]],[[1,2],[5,2]],[[3,2],[1,4]],[[0,4],[4,4]],[[4,3],[5,5]]],
	[[[0,0],[5,3]],[[4,1],[3,3]],[[4,3],[3,4]],[[3,2],[2,4]]],
	[[[5,0],[3,4]],[[3,1],[4,4]],[[2,1],[3,3]],[[4,1],[2,4]],[[1,1],[1,4]]],
	[[[0,0],[5,0]],[[4,4],[2,5]],[[1,2],[3,5]],[[4,1],[4,5]],[[0,4],[1,5]]],
	[[[1,0],[5,3]],[[2,1],[4,1]],[[1,3],[3,3]],[[0,2],[5,4]],[[0,0],[1,4]]],
	[[[4,1],[3,5]],[[2,4],[4,5]],[[1,0],[2,5]],[[4,2],[4,4]],[[1,1],[1,4]]],
	[[[5,1],[0,5]],[[3,5],[5,5]],[[1,2],[3,2]],[[5,2],[2,5]],[[1,1],[3,3]],[[2,3],[1,5]]],
	[[[4,0],[5,2]],[[1,0],[4,4]],[[2,1],[3,3]],[[0,0],[5,3]],[[1,4],[3,4]],[[1,1],[2,3]]],
	[[[5,1],[4,5]],[[2,4],[3,5]],[[1,1],[2,5]],[[4,1],[5,5]],[[0,1],[1,4]]],
	[[[5,0],[3,5]],[[1,1],[3,4]],[[0,0],[2,1]],[[3,1],[1,4]],[[0,3],[2,4]]],

	[[[5,2],[3,3]],[[4,1],[2,2]],[[4,2],[1,4]],[[5,3],[4,5]],[[1,0],[3,5]]],
	[[[4,0],[2,5]],[[1,2],[4,2]],[[4,3],[1,4]],[[3,0],[4,1]],[[2,0],[3,3]],[[0,3],[1,5]]],
	[[[1,0],[4,1]],[[3,0],[1,1]],[[2,3],[0,4]],[[0,0],[4,4]]],
	[[[5,0],[1,2]],[[0,0],[3,3]],[[1,1],[2,5]],[[4,1],[3,5]]],
	[[[5,0],[1,1]],[[1,4],[4,4]],[[0,3],[4,5]],[[4,0],[0,2]],[[0,4],[3,4]]],
	[[[5,0],[0,3]],[[0,0],[3,0]],[[3,3],[1,4]],[[4,0],[3,1]],[[0,1],[3,4]],[[0,2],[1,3]]],
	[[[4,1],[4,4]],[[1,2],[2,4]],[[0,0],[0,5]],[[2,2],[1,4]],[[0,1],[0,4]]],
	[[[0,0],[5,1]],[[4,1],[0,2]],[[4,4],[2,5]],[[1,2],[5,5]],[[1,4],[4,5]]],
	[[[4,0],[5,5]],[[3,1],[3,5]],[[3,4],[0,5]],[[2,1],[4,1]],[[0,0],[1,4]]],
	[[[5,0],[4,1]],[[3,3],[5,3]],[[0,0],[5,4]],[[3,1],[5,1]],[[0,1],[5,5]],[[3,0],[2,3]]],

	[[[5,0],[4,4]],[[3,1],[1,3]],[[1,2],[0,3]],[[5,4],[0,5]],[[0,0],[0,2]]],
	[[[4,0],[2,5]],[[0,0],[3,0]],[[2,2],[0,5]],[[4,1],[2,4]],[[0,1],[1,5]]],
	[[[5,0],[4,5]],[[3,1],[0,2]],[[0,0],[2,4]],[[3,0],[4,4]],[[0,1],[1,4]]],
	[[[2,0],[5,0]],[[0,0],[5,5]],[[0,1],[4,5]],[[4,1],[5,3]],[[0,3],[2,5]],[[0,4],[1,5]]],
	[[[1,0],[5,0]],[[3,2],[5,2]],[[2,2],[5,4]],[[0,0],[5,1]],[[0,2],[5,5]],[[1,2],[4,4]]],
	[[[5,0],[4,2]],[[0,5],[5,5]],[[4,0],[4,3]],[[5,2],[1,3]],[[2,1],[1,4]],[[1,1],[2,2]]],
	[[[5,0],[3,2]],[[4,0],[3,3]],[[3,0],[4,3]],[[5,3],[3,5]],[[1,1],[1,4]]],
	[[[5,2],[2,3]],[[2,1],[0,5]],[[0,0],[2,4]],[[2,2],[4,4]]],
	[[[0,0],[5,5]],[[0,3],[4,5]],[[3,1],[3,3]],[[4,1],[0,2]],[[1,2],[1,4]]],
	[[[5,4],[1,5]],[[4,4],[0,5]],[[0,3],[2,3]],[[0,1],[4,2]],[[0,0],[0,2]]],

	[[[1,1],[4,1]],[[2,0],[2,5]],[[2,1],[2,4]],[[1,4],[4,4]]],
	[[[5,0],[1,4]],[[1,1],[1,5]],[[3,3],[0,5]]],
	[[[5,4],[1,5]],[[3,1],[4,3]],[[4,4],[0,5]],[[4,1],[3,3]],[[1,1],[0,3]],[[0,1],[1,3]]],
	[[[5,0],[1,3]],[[4,0],[2,3]],[[3,0],[1,2]],[[1,0],[5,5]],[[3,1],[2,2]]],
	[[[5,0],[1,1]],[[2,0],[3,3]],[[1,0],[0,5]],[[4,0],[2,1]]],
	[[[5,0],[3,5]],[[4,2],[4,5]],[[3,0],[0,5]],[[5,1],[5,5]],[[2,1],[2,5]],[[1,1],[1,5]]],
	[[[5,2],[2,3]],[[3,2],[5,5]],[[0,0],[4,5]],[[2,1],[5,3]],[[0,4],[1,5]]],
	[[[5,0],[5,1]],[[4,3],[3,5]],[[3,3],[4,5]],[[4,1],[2,2]],[[3,1],[1,3]],[[0,5],[2,5]]],
	[[[3,0],[5,2]],[[4,2],[3,5]],[[1,0],[4,4]],[[4,1],[1,4]],[[0,0],[1,1]]],
	[[[5,0],[5,4]],[[3,1],[0,5]],[[1,0],[1,1]],[[2,5],[5,5]],[[0,0],[1,5]]],

	[[[5,0],[4,3]],[[3,0],[5,5]],[[0,0],[0,5]],[[4,0],[5,3]],[[0,1],[0,4]]],
	[[[5,0],[1,3]],[[0,3],[4,4]],[[2,2],[0,5]],[[4,1],[0,2]]],
	[[[1,1],[3,2]],[[2,1],[0,5]],[[0,1],[1,5]],[[0,2],[3,3]]],
	[[[5,3],[3,4]],[[4,1],[2,5]],[[2,3],[4,3]],[[3,5],[5,5]],[[0,0],[2,2]],[[1,2],[2,4]]],
	[[[5,0],[4,5]],[[1,4],[4,4]],[[1,1],[3,3]],[[1,2],[4,3]],[[2,2],[3,5]]],
	[[[5,0],[3,3]],[[0,0],[4,1]],[[1,1],[1,4]],[[0,1],[5,1]]],
	[[[4,1],[1,2]],[[0,3],[3,3]],[[0,2],[1,3]],[[1,4],[4,4]]],
	[[[5,0],[3,5]],[[0,0],[4,4]],[[2,1],[2,5]],[[4,0],[2,3]]],
	[[[2,0],[5,1]],[[0,1],[4,1]],[[0,0],[3,4]],[[3,2],[5,5]]],
	[[[5,1],[4,4]],[[4,1],[5,5]],[[5,2],[2,3]]],

	[[[4,1],[1,5]],[[2,3],[2,5]],[[1,0],[1,1]],[[2,0],[0,5]],[[0,0],[0,2]]],
	[[[4,5],[5,5]],[[0,0],[3,0]],[[3,2],[2,4]],[[4,0],[1,5]],[[2,3],[0,5]]],
	[[[5,3],[2,4]],[[1,0],[4,1]],[[3,0],[1,1]],[[3,3],[5,4]],[[0,0],[0,5]]],
	[[[4,1],[3,3]],[[3,1],[0,3]],[[0,2],[2,3]],[[1,1],[4,3]]],
	[[[5,0],[3,4]],[[3,2],[5,5]],[[3,3],[4,5]],[[5,1],[2,4]],[[1,1],[1,4]]],
	[[[0,0],[5,1]],[[0,1],[4,5]],[[3,2],[2,4]],[[5,2],[5,5]],[[3,3],[3,5]],[[2,3],[1,4]]],
	[[[5,1],[1,2]],[[4,3],[0,4]],[[3,1],[2,2]],[[4,1],[5,5]],[[1,0],[3,3]],[[1,4],[3,4]]],
	[[[5,0],[0,3]],[[2,0],[3,3]],[[0,0],[2,3]],[[4,0],[1,3]],[[0,1],[1,2]]],
	[[[4,0],[2,5]],[[4,3],[1,4]],[[1,1],[3,1]],[[4,1],[1,5]],[[0,0],[2,1]]],
	[[[5,2],[2,5]],[[2,3],[5,5]],[[4,2],[3,5]],[[5,3],[5,4]],[[2,2],[3,2]]],

	[[[1,4],[5,4]],[[1,1],[4,2]],[[0,1],[3,2]],[[2,3],[5,5]],[[0,2],[2,2]]],
	[[[1,2],[5,4]],[[3,1],[1,4]],[[1,1],[3,2]],[[3,3],[5,5]]],
	[[[3,5],[5,5]],[[2,2],[4,5]],[[3,1],[3,4]],[[4,1],[3,2]]],
	[[[5,0],[1,4]],[[0,0],[4,5]],[[1,0],[3,3]],[[3,1],[5,5]]],
	[[[0,0],[5,5]],[[4,2],[3,5]],[[1,2],[2,5]],[[4,1],[2,4]],[[0,1],[1,4]]],
	[[[1,1],[5,2]],[[1,2],[4,4]],[[2,0],[1,4]],[[4,1],[3,5]],[[2,3],[0,4]]],
	[[[5,0],[2,3]],[[0,3],[4,5]],[[3,1],[0,2]],[[1,2],[5,5]],[[0,0],[2,2]]],
	[[[0,0],[5,2]],[[2,2],[5,4]],[[1,4],[1,5]],[[5,3],[5,5]],[[0,1],[0,5]]],
	[[[5,0],[4,5]],[[1,3],[4,4]],[[2,2],[0,5]],[[4,1],[2,4]]],
	[[[4,2],[5,4]],[[4,1],[0,4]],[[3,0],[4,3]],[[2,2],[5,5]],[[1,1],[0,5]]],

	[[[5,1],[2,3]],[[4,1],[3,3]],[[0,0],[4,2]],[[0,1],[5,2]]],
	[[[2,1],[4,4]],[[2,0],[1,4]],[[1,1],[0,4]],[[3,0],[3,2]]],
	[[[5,0],[1,2]],[[0,2],[4,2]],[[2,2],[4,4]],[[1,4],[5,4]],[[1,3],[0,5]]],
	[[[5,0],[0,3]],[[1,1],[2,3]],[[1,3],[2,4]],[[2,2],[3,4]]]
	];

	this.Level7Data = [
	[[[5,5],[0,6]],[[2,2],[5,4]],[[2,3],[5,3]],[[6,0],[1,6]],[[5,2],[2,4]]],
	[[[2,3],[0,6]],[[5,1],[2,2]],[[5,2],[2,5]],[[1,1],[3,6]],[[6,4],[4,6]],[[5,0],[6,2]],[[6,3],[5,5]]],
	[[[0,0],[1,3]],[[3,2],[1,4]],[[5,0],[1,5]],[[2,0],[1,2]],[[5,1],[4,5]],[[6,0],[4,6]],[[5,2],[5,5]]],
	[[[6,5],[0,6]],[[5,4],[1,6]],[[4,2],[3,3]],[[2,1],[1,5]],[[3,4],[6,6]],[[4,4],[5,5]]],
	[[[0,3],[6,6]],[[2,2],[1,4]],[[1,5],[4,5]],[[3,1],[0,4]],[[2,1],[3,3]]],
	[[[0,5],[4,6]],[[1,1],[1,5]],[[1,2],[1,4]],[[0,6],[3,6]],[[3,3],[2,4]],[[4,0],[5,6]]],
	[[[0,0],[4,4]],[[4,3],[0,6]],[[3,3],[1,6]],[[5,3],[0,4]],[[3,4],[6,6]],[[4,5],[6,5]]],
	[[[6,0],[0,1]],[[2,4],[6,4]],[[3,3],[5,5]],[[1,1],[6,2]],[[4,3],[6,3]],[[5,2],[6,5]]],
	[[[0,0],[5,6]],[[1,2],[1,5]],[[1,3],[3,3]],[[0,2],[3,2]],[[4,1],[3,6]],[[6,0],[6,6]]],
	[[[0,0],[4,0]],[[0,5],[3,6]],[[1,0],[4,2]],[[0,2],[0,4]],[[1,2],[2,4]],[[3,4],[1,5]],[[4,1],[5,5]]],

	[[[0,0],[0,6]],[[0,3],[3,5]],[[1,0],[3,2]],[[0,2],[4,5]],[[4,2],[6,6]],[[5,1],[5,2]]],
	[[[1,2],[0,4]],[[1,1],[0,6]],[[2,0],[2,3]],[[4,1],[0,5]],[[5,1],[5,5]]],
	[[[5,1],[1,5]],[[3,0],[5,5]],[[4,5],[3,6]],[[2,0],[2,2]],[[4,3],[4,6]]],
	[[[1,0],[2,3]],[[1,2],[3,3]],[[1,5],[5,5]],[[4,0],[1,1]],[[3,1],[5,4]],[[5,0],[3,4]],[[5,1],[5,3]]],
	[[[0,1],[6,2]],[[1,2],[5,2]],[[1,3],[1,5]],[[0,2],[6,3]],[[2,2],[3,5]],[[3,2],[4,5]],[[3,3],[4,4]]],
	[[[2,1],[4,5]],[[3,1],[4,4]],[[5,0],[4,3]],[[2,2],[2,4]],[[5,1],[5,4]],[[5,3],[6,6]]],
	[[[2,3],[4,5]],[[3,5],[6,6]],[[6,5],[4,6]],[[3,3],[2,4]]],
	[[[0,3],[0,6]],[[3,2],[1,3]],[[5,1],[2,5]],[[1,1],[4,2]],[[4,1],[4,4]]],
	[[[0,0],[1,6]],[[2,0],[2,2]],[[2,3],[4,5]],[[1,0],[2,6]],[[2,4],[3,6]],[[3,0],[5,1]],[[3,2],[4,6]],[[4,0],[6,6]]],
	[[[1,0],[0,5]],[[1,1],[4,6]],[[1,2],[4,5]],[[0,6],[3,6]],[[2,4],[2,6]],[[3,1],[5,5]]],

	[[[0,0],[0,2]],[[4,1],[1,3]],[[2,1],[3,3]],[[1,2],[0,3]],[[3,1],[2,4]]],
	[[[0,0],[6,3]],[[1,1],[0,6]],[[2,1],[6,5]],[[0,1],[1,5]],[[3,1],[5,5]],[[4,1],[6,4]]],
	[[[1,1],[1,5]],[[1,3],[5,4]],[[3,4],[2,5]],[[1,2],[5,3]],[[4,1],[3,3]],[[5,1],[5,5]]],
	[[[2,1],[1,5]],[[2,2],[6,6]],[[3,0],[6,0]],[[2,0],[2,3]],[[3,1],[6,1]],[[4,3],[3,6]],[[6,2],[5,5]]],
	[[[1,1],[5,1]],[[6,1],[3,2]],[[4,1],[6,2]],[[1,5],[5,5]]],
	[[[1,0],[6,0]],[[4,2],[5,4]],[[5,1],[4,4]],[[2,2],[3,4]],[[5,0],[5,5]]],
	[[[1,1],[5,2]],[[5,3],[2,5]],[[5,1],[2,6]],[[2,2],[1,5]],[[5,0],[3,6]],[[4,5],[6,5]],[[4,6],[6,6]]],
	[[[0,3],[4,4]],[[4,0],[1,1]],[[5,2],[1,4]],[[6,5],[0,6]],[[2,0],[6,0]],[[6,1],[3,5]]],
	[[[1,0],[2,1]],[[2,0],[4,2]],[[3,1],[3,3]],[[1,1],[1,5]],[[4,1],[4,5]]],
	[[[3,2],[0,6]],[[2,1],[4,5]],[[5,1],[2,4]],[[1,1],[2,6]],[[2,5],[6,6]]],

	[[[0,0],[1,3]],[[0,5],[6,6]],[[5,0],[1,5]],[[1,0],[0,3]],[[2,0],[4,3]],[[4,0],[3,3]]],
	[[[5,1],[0,3]],[[3,1],[1,5]],[[2,2],[3,5]],[[5,3],[1,4]],[[4,5],[6,6]],[[5,0],[6,1]]],
	[[[6,4],[0,6]],[[3,2],[2,4]],[[4,2],[5,4]],[[2,2],[3,4]],[[5,1],[4,4]],[[5,0],[6,3]]],
	[[[0,1],[4,1]],[[1,5],[4,5]],[[2,2],[5,3]],[[6,0],[0,2]],[[5,0],[5,5]]],
	[[[5,0],[0,4]],[[6,5],[0,6]],[[2,2],[5,2]],[[6,0],[0,5]],[[3,6],[6,6]],[[5,1],[5,3]]],
	[[[3,1],[1,6]],[[2,2],[4,3]],[[4,2],[3,3]],[[2,1],[2,6]],[[5,1],[3,6]],[[4,1],[4,6]]],
	[[[1,0],[4,6]],[[1,4],[3,5]],[[3,1],[6,6]],[[1,1],[4,5]],[[3,3],[4,3]],[[4,2],[6,3]],[[6,4],[6,5]]],
	[[[0,0],[5,5]],[[2,3],[0,6]],[[1,3],[1,6]],[[0,3],[2,6]],[[6,0],[5,6]]],
	[[[0,2],[3,5]],[[1,4],[5,4]],[[5,1],[2,5]],[[1,1],[0,6]],[[4,5],[6,6]]],
	[[[3,0],[0,4]],[[1,4],[5,5]],[[4,2],[1,5]],[[3,3],[0,5]],[[4,0],[4,3]]],

	[[[0,0],[3,3]],[[0,2],[1,5]],[[4,0],[0,6]],[[3,0],[0,1]],[[4,3],[3,4]],[[5,0],[6,0]]],
	[[[6,0],[0,6]],[[2,2],[1,6]],[[2,4],[6,5]],[[1,2],[6,3]],[[4,3],[6,4]],[[5,3],[5,5]]],
	[[[3,1],[0,3]],[[1,0],[3,3]],[[1,1],[2,4]],[[2,0],[0,5]]],
	[[[3,1],[0,2]],[[2,1],[4,2]],[[5,1],[2,3]],[[1,1],[0,6]],[[2,5],[6,6]],[[4,1],[5,4]],[[4,4],[5,6]]],
	[[[0,0],[2,5]],[[2,1],[1,4]],[[2,3],[6,6]],[[5,1],[0,6]],[[3,1],[3,5]]],
	[[[3,0],[0,6]],[[6,0],[2,5]],[[4,6],[6,6]],[[2,1],[1,2]],[[5,0],[6,2]],[[6,4],[5,5]]],
	[[[0,6],[2,6]],[[2,3],[2,5]],[[6,0],[3,3]],[[1,1],[6,6]],[[4,5],[6,5]],[[5,2],[5,5]]],
	[[[0,0],[6,0]],[[6,4],[0,6]],[[1,0],[3,0]],[[0,4],[5,5]],[[1,6],[3,6]],[[2,0],[4,1]]],
	[[[5,1],[0,3]],[[1,1],[5,2]],[[6,3],[1,5]],[[1,0],[2,2]],[[4,4],[3,6]],[[4,2],[5,5]]],
	[[[0,0],[6,6]],[[1,0],[6,0]],[[3,3],[2,5]],[[0,2],[0,6]],[[3,2],[2,6]],[[5,1],[4,5]],[[5,2],[5,4]]],

	[[[3,0],[0,4]],[[1,4],[1,6]],[[4,0],[6,2]],[[2,2],[0,6]],[[4,1],[5,5]],[[5,1],[4,5]]],
	[[[1,1],[3,1]],[[1,3],[2,6]],[[6,0],[1,4]],[[5,1],[1,2]],[[1,5],[6,6]],[[2,2],[4,3]],[[4,1],[3,2]],[[6,3],[4,5]]],
	[[[0,0],[2,2]],[[5,1],[1,5]],[[2,1],[5,5]],[[1,2],[3,6]]],
	[[[0,0],[6,0]],[[0,4],[0,6]],[[1,0],[4,3]],[[6,1],[0,2]],[[1,5],[6,6]],[[2,1],[3,2]]],
	[[[0,0],[1,3]],[[6,1],[0,5]],[[1,0],[2,3]],[[0,2],[3,4]],[[3,3],[1,5]],[[4,0],[5,5]],[[6,0],[4,1]]],
	[[[0,0],[3,3]],[[1,2],[6,3]],[[5,1],[1,5]],[[6,2],[0,3]]],
	[[[1,1],[6,6]],[[4,2],[1,5]],[[5,1],[2,3]],[[3,0],[1,4]]],
	[[[0,4],[5,4]],[[3,3],[1,5]],[[4,1],[2,2]],[[5,1],[1,2]],[[6,4],[3,6]],[[5,0],[4,4]]],
	[[[6,4],[1,5]],[[4,1],[6,5]],[[5,1],[4,4]],[[5,0],[2,2]]],
	[[[1,1],[1,5]],[[3,4],[2,6]],[[3,0],[3,3]],[[2,5],[1,6]],[[5,1],[3,5]]],

	[[[0,1],[6,6]],[[0,3],[3,3]],[[1,1],[3,4]],[[0,2],[1,5]]],
	[[[2,2],[4,4]],[[3,3],[6,4]],[[5,1],[6,6]],[[3,2],[6,3]],[[6,1],[5,6]]],
	[[[6,4],[1,5]],[[6,1],[2,3]],[[4,0],[6,5]],[[6,0],[2,2]],[[4,1],[4,4]]],
	[[[1,0],[2,1]],[[2,0],[3,2]],[[3,1],[4,5]],[[1,1],[1,5]],[[6,0],[6,6]]],
	[[[3,3],[0,6]],[[6,0],[1,5]],[[5,2],[1,6]],[[1,2],[6,6]]],
	[[[0,0],[6,1]],[[0,2],[2,3]],[[4,4],[1,5]],[[0,1],[3,4]],[[2,1],[4,2]],[[3,1],[6,6]]],
	[[[2,0],[0,6]],[[4,1],[1,6]],[[3,3],[4,6]],[[1,5],[5,6]],[[5,4],[6,6]],[[6,0],[6,3]]],
	[[[1,1],[3,3]],[[4,0],[2,4]],[[3,0],[5,2]],[[2,1],[6,5]],[[5,5],[6,6]]],
	[[[0,0],[1,4]],[[1,0],[3,3]],[[5,2],[1,5]],[[2,3],[0,4]],[[2,0],[3,1]],[[3,0],[6,0]]],
	[[[0,0],[6,0]],[[0,5],[1,6]],[[1,0],[4,4]],[[0,2],[3,6]],[[3,0],[5,2]],[[4,0],[5,1]],[[6,4],[4,6]]],

	[[[6,0],[0,3]],[[4,3],[3,5]],[[6,1],[3,6]],[[1,5],[4,5]],[[4,6],[6,6]],[[6,2],[5,5]]],
	[[[0,4],[6,5]],[[5,1],[1,6]],[[3,2],[5,2]],[[1,1],[0,6]],[[3,3],[3,6]],[[4,3],[4,6]],[[5,3],[6,6]]],
	[[[1,5],[6,6]],[[4,2],[1,4]],[[1,3],[4,3]],[[0,0],[5,6]],[[1,2],[4,4]]],
	[[[4,3],[6,6]],[[1,1],[4,2]],[[1,2],[4,5]],[[5,1],[3,6]],[[0,4],[2,6]],[[1,0],[0,2]],[[0,3],[1,5]]],
	[[[6,0],[5,3]],[[3,2],[5,4]],[[1,0],[5,5]],[[4,0],[5,2]],[[1,1],[2,5]],[[0,0],[2,6]],[[1,2],[1,5]]],
	[[[0,5],[6,6]],[[1,4],[5,6]],[[2,2],[3,3]],[[4,1],[5,5]],[[3,4],[0,6]],[[2,4],[1,5]]],
	[[[6,3],[0,6]],[[4,2],[5,4]],[[2,5],[5,5]],[[3,1],[6,4]],[[4,1],[3,3]]],
	[[[6,5],[2,6]],[[5,1],[5,5]],[[5,2],[5,4]],[[3,6],[6,6]],[[3,3],[4,4]],[[2,0],[1,6]]],
	[[[6,0],[2,4]],[[2,3],[6,6]],[[3,3],[5,6]],[[1,3],[6,4]],[[3,4],[0,6]],[[0,5],[2,5]]],
	[[[0,0],[6,1]],[[0,4],[4,4]],[[3,3],[1,5]],[[5,1],[0,2]],[[0,3],[2,3]],[[1,2],[0,5]]],

	[[[6,0],[1,6]],[[5,2],[5,5]],[[3,3],[5,3]],[[3,2],[6,2]],[[2,1],[3,6]],[[0,0],[0,6]]],
	[[[2,0],[6,0]],[[6,5],[3,6]],[[5,0],[2,2]],[[6,2],[6,4]],[[5,2],[4,4]],[[3,4],[5,5]],[[2,1],[1,5]]],
	[[[6,0],[6,6]],[[6,3],[3,5]],[[5,0],[3,2]],[[6,2],[2,5]],[[2,2],[0,6]],[[1,1],[1,2]]],
	[[[5,2],[6,4]],[[5,1],[6,6]],[[4,0],[4,3]],[[2,1],[6,5]],[[1,1],[1,5]]],
	[[[1,1],[5,5]],[[3,0],[1,5]],[[2,5],[3,6]],[[4,0],[4,2]],[[2,3],[2,6]]],
	[[[5,0],[4,3]],[[5,2],[3,3]],[[1,5],[5,5]],[[2,0],[5,1]],[[3,1],[1,4]],[[1,0],[3,4]],[[1,1],[1,3]]],
	[[[6,1],[0,2]],[[1,2],[5,2]],[[5,3],[5,5]],[[6,2],[0,3]],[[4,2],[3,5]],[[3,2],[2,5]],[[3,3],[2,4]]],
	[[[4,1],[2,5]],[[3,1],[2,4]],[[1,0],[2,3]],[[4,2],[4,4]],[[1,1],[1,4]],[[1,3],[0,6]]],
	[[[4,3],[2,5]],[[3,5],[0,6]],[[0,5],[2,6]],[[3,3],[4,4]]],
	[[[6,3],[6,6]],[[3,2],[5,3]],[[1,1],[4,5]],[[5,1],[2,2]],[[2,1],[2,4]]],

	[[[6,0],[5,6]],[[4,0],[4,2]],[[4,3],[2,5]],[[5,0],[4,6]],[[4,4],[3,6]],[[3,0],[1,1]],[[3,2],[2,6]],[[2,0],[0,6]]],
	[[[5,0],[6,5]],[[5,1],[2,6]],[[5,2],[2,5]],[[3,6],[6,6]],[[4,4],[4,6]],[[3,1],[1,5]]],
	[[[6,0],[6,2]],[[2,1],[5,3]],[[4,1],[3,3]],[[5,2],[6,3]],[[3,1],[4,4]]],
	[[[6,0],[0,3]],[[5,1],[6,6]],[[4,1],[0,5]],[[6,1],[5,5]],[[3,1],[1,5]],[[2,1],[0,4]]],
	[[[5,1],[5,5]],[[5,3],[1,4]],[[3,4],[4,5]],[[5,2],[1,3]],[[2,1],[3,3]],[[1,1],[1,5]]],
	[[[4,1],[5,5]],[[4,2],[0,6]],[[0,0],[3,0]],[[4,0],[4,3]],[[0,1],[3,1]],[[2,3],[3,6]],[[0,2],[1,5]]],
	[[[1,1],[5,1]],[[0,1],[3,2]],[[2,1],[0,2]],[[1,5],[5,5]]],
	[[[0,0],[5,0]],[[2,2],[1,4]],[[1,1],[2,4]],[[4,2],[3,4]],[[1,0],[1,5]]],
	[[[5,1],[1,2]],[[1,3],[4,5]],[[1,1],[4,6]],[[4,2],[5,5]],[[1,0],[3,6]],[[0,5],[2,5]],[[0,6],[2,6]]],
	[[[6,3],[2,4]],[[2,0],[5,1]],[[1,2],[5,4]],[[0,5],[6,6]],[[0,0],[4,0]],[[0,1],[3,5]]],

	[[[5,0],[4,1]],[[4,0],[2,2]],[[3,1],[3,3]],[[5,1],[5,5]],[[2,1],[2,5]]],
	[[[3,2],[6,6]],[[4,1],[2,5]],[[1,1],[4,4]],[[5,1],[4,6]],[[4,5],[0,6]]],
	[[[6,0],[5,3]],[[6,5],[0,6]],[[1,0],[5,5]],[[5,0],[6,3]],[[4,0],[2,3]],[[2,0],[3,3]]],
	[[[1,1],[6,3]],[[3,1],[5,5]],[[4,2],[3,5]],[[1,3],[5,4]],[[2,5],[0,6]],[[1,0],[0,1]]],
	[[[0,4],[6,6]],[[3,2],[4,4]],[[2,2],[1,4]],[[4,2],[3,4]],[[1,1],[2,4]],[[1,0],[0,3]]],
	[[[2,1],[6,1]],[[2,5],[5,5]],[[4,2],[1,3]],[[0,0],[6,2]],[[1,0],[1,5]]],
	[[[1,0],[6,4]],[[0,5],[6,6]],[[1,2],[4,2]],[[0,0],[6,5]],[[0,6],[3,6]],[[1,1],[1,3]]],
	[[[3,1],[5,6]],[[4,2],[2,3]],[[2,2],[3,3]],[[4,1],[4,6]],[[1,1],[3,6]],[[2,1],[2,6]]],
	[[[5,0],[2,6]],[[5,4],[3,5]],[[3,1],[0,6]],[[5,1],[2,5]],[[2,3],[3,3]],[[2,2],[0,3]],[[0,4],[0,5]]],
	[[[6,0],[1,5]],[[4,3],[6,6]],[[5,3],[5,6]],[[6,3],[4,6]],[[0,0],[1,6]]],

	[[[6,2],[3,5]],[[1,4],[5,4]],[[1,1],[4,5]],[[5,1],[6,6]],[[2,5],[0,6]]],
	[[[3,0],[6,4]],[[5,4],[1,5]],[[2,2],[5,5]],[[3,3],[6,5]],[[2,0],[2,3]]],
	[[[6,0],[3,3]],[[6,2],[5,5]],[[2,0],[6,6]],[[3,0],[6,1]],[[2,3],[3,4]],[[0,0],[1,0]]],
	[[[0,0],[6,6]],[[4,2],[5,6]],[[4,4],[0,5]],[[5,2],[0,3]],[[2,3],[0,4]],[[1,3],[1,5]]],
	[[[3,1],[6,3]],[[5,0],[3,3]],[[5,1],[4,4]],[[4,0],[6,5]]],
	[[[3,1],[6,2]],[[4,1],[2,2]],[[1,1],[4,3]],[[5,1],[6,6]],[[4,5],[0,6]],[[2,1],[1,4]],[[2,4],[1,6]]],
	[[[6,0],[4,5]],[[4,1],[5,4]],[[4,3],[0,6]],[[1,1],[6,6]],[[3,1],[3,5]]],
	[[[3,0],[6,6]],[[0,0],[4,5]],[[0,6],[2,6]],[[4,1],[5,2]],[[1,0],[0,2]],[[0,4],[1,5]]],
	[[[4,6],[6,6]],[[4,3],[4,5]],[[0,0],[3,3]],[[5,1],[0,6]],[[0,5],[2,5]],[[1,2],[1,5]]],
	[[[0,0],[6,0]],[[0,4],[6,6]],[[3,0],[5,0]],[[6,4],[1,5]],[[3,6],[5,6]],[[4,0],[2,1]]],

	[[[1,1],[6,3]],[[5,1],[1,2]],[[0,3],[5,5]],[[5,0],[4,2]],[[2,4],[3,6]],[[2,2],[1,5]]],
	[[[6,0],[0,6]],[[0,0],[5,0]],[[3,3],[4,5]],[[6,2],[6,6]],[[3,2],[4,6]],[[1,1],[2,5]],[[1,2],[1,4]]],
	[[[3,0],[6,4]],[[5,4],[5,6]],[[2,0],[0,2]],[[4,2],[6,6]],[[2,1],[1,5]],[[1,1],[2,5]]],
	[[[3,1],[5,1]],[[5,3],[4,6]],[[0,0],[5,4]],[[1,1],[5,2]],[[5,5],[0,6]],[[4,2],[2,3]],[[2,1],[3,2]],[[0,3],[2,5]]],
	[[[6,0],[4,2]],[[1,1],[5,5]],[[4,1],[1,5]],[[5,2],[3,6]]],
	[[[0,0],[6,0]],[[6,4],[6,6]],[[5,0],[2,3]],[[0,1],[6,2]],[[5,5],[0,6]],[[4,1],[3,2]]],
	[[[6,0],[5,3]],[[0,1],[6,5]],[[5,0],[4,3]],[[6,2],[3,4]],[[3,3],[5,5]],[[2,0],[1,5]],[[0,0],[2,1]]],
	[[[6,0],[3,3]],[[5,2],[0,3]],[[1,1],[5,5]],[[0,2],[6,3]]],
	[[[5,1],[0,6]],[[2,2],[5,5]],[[1,1],[4,3]],[[3,0],[5,4]]],
	[[[1,4],[6,4]],[[3,3],[5,5]],[[2,1],[4,2]],[[1,1],[5,2]],[[0,4],[3,6]],[[1,0],[2,4]]],

	[[[0,4],[5,5]],[[2,1],[0,5]],[[1,1],[2,4]],[[1,0],[4,2]]],
	[[[5,1],[5,5]],[[3,4],[4,6]],[[3,0],[3,3]],[[4,5],[5,6]],[[1,1],[3,5]]],
	[[[6,1],[0,6]],[[3,3],[6,3]],[[5,1],[3,4]],[[6,2],[5,5]]],
	[[[4,2],[2,4]],[[3,3],[0,4]],[[1,1],[0,6]],[[3,2],[0,3]],[[0,1],[1,6]]],
	[[[0,4],[5,5]],[[0,1],[4,3]],[[2,0],[0,5]],[[0,0],[4,2]],[[2,1],[2,4]]],
	[[[5,0],[4,1]],[[4,0],[3,2]],[[3,1],[2,5]],[[5,1],[5,5]],[[0,0],[0,6]]],
	[[[3,3],[6,6]],[[0,0],[5,5]],[[1,2],[5,6]],[[5,2],[0,6]]],
	[[[6,0],[0,1]],[[6,2],[4,3]],[[2,4],[5,5]],[[6,1],[3,4]],[[4,1],[2,2]],[[3,1],[0,6]]],
	[[[4,0],[6,6]],[[2,1],[5,6]],[[3,3],[2,6]],[[5,5],[1,6]],[[1,4],[0,6]],[[0,0],[0,3]]],
	[[[5,1],[3,3]],[[2,0],[4,4]],[[3,0],[1,2]],[[4,1],[0,5]],[[1,5],[0,6]]],

	[[[6,0],[5,4]],[[5,0],[3,3]],[[1,2],[5,5]],[[4,3],[6,4]],[[4,0],[3,1]],[[0,0],[3,0]]],
	[[[0,0],[6,0]],[[6,5],[5,6]],[[5,0],[2,4]],[[6,2],[3,6]],[[3,0],[1,2]],[[2,0],[1,1]],[[0,4],[2,6]]],
	[[[0,0],[6,3]],[[2,3],[3,5]],[[0,1],[3,6]],[[2,5],[5,5]],[[0,6],[2,6]],[[0,2],[1,5]]],
	[[[6,4],[0,5]],[[1,1],[5,6]],[[1,2],[3,2]],[[5,1],[6,6]],[[3,3],[3,6]],[[2,3],[2,6]],[[1,3],[0,6]]]
	];

	this.Level8Data = [
	[[[0,0],[5,5]],[[1,2],[4,4]],[[1,4],[3,6]],[[6,0],[1,1]],[[3,0],[2,5]],[[4,0],[6,1]]],
	[[[0,0],[5,6]],[[6,0],[2,3]],[[4,4],[2,5]],[[0,7],[3,7]],[[5,2],[3,6]],[[6,4],[6,6]]],
	[[[0,0],[4,4]],[[3,5],[1,6]],[[2,4],[6,7]],[[0,1],[5,2]],[[6,1],[6,6]]],
	[[[7,0],[0,3]],[[3,1],[1,6]],[[6,5],[2,6]],[[3,3],[0,4]],[[5,2],[7,7]],[[7,1],[5,4]]],
	[[[0,0],[3,5]],[[1,2],[4,7]],[[4,0],[7,5]],[[6,6],[0,7]],[[5,2],[7,6]]],
	[[[0,1],[1,4]],[[1,2],[7,7]],[[2,2],[4,3]],[[1,1],[1,3]],[[5,1],[2,5]]],
	[[[3,3],[4,4]],[[4,3],[5,4]],[[5,0],[6,3]],[[4,0],[6,0]],[[6,1],[5,5]]],
	[[[1,1],[5,2]],[[2,3],[3,5]],[[7,0],[2,5]],[[1,4],[6,6]],[[6,0],[3,2]],[[5,1],[3,3]]],
	[[[0,0],[0,3]],[[7,1],[2,2]],[[6,2],[2,6]],[[4,3],[0,7]],[[7,0],[7,2]]],
	[[[4,0],[0,1]],[[3,3],[2,5]],[[6,1],[2,7]],[[2,1],[1,7]],[[7,1],[3,5]],[[7,2],[5,3]],[[6,3],[6,6]]],

	[[[0,3],[2,4]],[[1,1],[5,5]],[[3,4],[1,6]],[[7,1],[0,4]],[[4,1],[5,2]],[[5,1],[7,2]]],
	[[[2,4],[0,7]],[[1,2],[5,4]],[[2,6],[5,7]],[[1,1],[2,5]],[[6,6],[4,7]]],
	[[[0,1],[3,2]],[[0,7],[7,7]],[[1,1],[6,5]],[[0,2],[4,4]],[[5,1],[2,4]],[[7,0],[5,4]]],
	[[[0,0],[6,1]],[[0,3],[1,5]],[[4,3],[0,4]],[[0,2],[5,5]],[[1,0],[1,6]]],
	[[[1,0],[6,1]],[[6,2],[2,6]],[[5,3],[4,7]],[[3,0],[1,6]],[[6,3],[5,5]],[[6,4],[6,6]]],
	[[[2,0],[5,1]],[[2,2],[2,5]],[[3,0],[5,5]],[[2,1],[7,2]],[[5,2],[5,3]]],
	[[[0,0],[2,1]],[[3,1],[1,2]],[[7,3],[1,5]],[[6,1],[0,7]],[[4,0],[5,4]],[[7,2],[4,4]]],
	[[[0,2],[2,5]],[[2,1],[2,6]],[[3,6],[7,6]],[[1,1],[6,1]],[[6,2],[7,7]]],
	[[[0,0],[0,7]],[[4,0],[1,4]],[[6,2],[2,4]],[[0,6],[6,6]],[[3,2],[5,2]],[[6,3],[3,4]],[[5,0],[4,4]]],
	[[[0,0],[5,7]],[[1,0],[5,4]],[[1,4],[4,7]],[[4,6],[0,7]],[[2,0],[7,1]],[[6,1],[7,7]],[[6,2],[7,6]]],

	[[[7,0],[0,4]],[[5,4],[1,5]],[[2,2],[1,6]],[[1,4],[6,6]],[[4,1],[6,5]],[[6,0],[6,2]]],
	[[[0,3],[3,5]],[[1,3],[2,5]],[[1,4],[4,4]],[[1,1],[7,7]],[[7,0],[6,7]]],
	[[[7,0],[0,1]],[[5,1],[2,5]],[[6,2],[3,3]],[[1,1],[6,5]],[[4,1],[5,7]]],
	[[[4,1],[1,2]],[[1,6],[6,7]],[[6,0],[2,2]],[[3,4],[1,5]],[[2,4],[5,5]],[[4,0],[7,7]],[[5,0],[6,1]]],
	[[[0,0],[1,7]],[[2,7],[4,7]],[[3,4],[3,7]],[[6,6],[0,7]]],
	[[[0,6],[5,7]],[[3,0],[5,3]],[[4,5],[6,7]],[[1,1],[0,7]],[[5,1],[7,4]],[[7,0],[6,6]]],
	[[[1,1],[0,3]],[[6,1],[1,6]],[[4,1],[2,2]],[[1,0],[1,2]],[[3,3],[2,4]],[[3,1],[4,4]]],
	[[[6,0],[1,1]],[[1,5],[6,6]],[[7,0],[2,3]],[[6,1],[1,4]],[[3,0],[2,5]],[[5,1],[4,2]]],
	[[[0,0],[1,1]],[[3,2],[0,6]],[[3,1],[0,7]],[[0,1],[6,1]],[[2,5],[6,6]],[[7,0],[2,6]],[[4,2],[5,4]]],
	[[[0,0],[0,2]],[[6,3],[1,4]],[[5,3],[3,4]],[[7,0],[1,1]],[[6,2],[6,4]],[[7,2],[7,6]]],

	[[[0,5],[0,7]],[[2,4],[5,5]],[[4,4],[2,6]],[[0,6],[3,6]],[[4,3],[3,4]]],
	[[[0,4],[1,5]],[[6,1],[1,2]],[[3,4],[1,6]],[[2,2],[0,6]],[[2,3],[6,6]],[[7,1],[4,5]]],
	[[[7,0],[0,7]],[[1,2],[5,2]],[[6,2],[1,5]],[[1,1],[6,4]],[[7,5],[1,6]],[[3,5],[1,7]],[[4,5],[7,7]],[[4,7],[6,7]]],
	[[[1,1],[6,5]],[[2,3],[4,3]],[[3,4],[3,7]],[[3,3],[1,6]],[[4,2],[6,6]]],
	[[[1,6],[0,7]],[[1,4],[2,6]],[[4,4],[1,7]],[[1,1],[5,4]],[[2,0],[4,1]],[[5,2],[2,5]],[[5,1],[6,6]]],
	[[[2,2],[4,3]],[[3,3],[7,6]],[[6,1],[7,7]],[[3,2],[4,6]]],
	[[[1,0],[6,5]],[[3,3],[4,4]],[[5,1],[3,7]],[[3,0],[1,6]],[[4,0],[5,3]],[[5,2],[4,3]],[[5,0],[7,3]]],
	[[[0,0],[2,6]],[[5,1],[3,2]],[[7,0],[3,6]],[[5,5],[1,6]],[[4,1],[4,6]],[[6,2],[5,4]]],
	[[[0,0],[2,2]],[[1,1],[3,1]],[[7,0],[1,3]],[[1,0],[5,2]],[[4,4],[2,5]],[[3,2],[5,5]],[[4,2],[5,4]],[[5,0],[5,6]]],
	[[[0,0],[4,2]],[[1,0],[0,4]],[[1,1],[1,6]],[[0,3],[4,4]],[[6,1],[2,5]]],

	[[[2,1],[1,2]],[[7,1],[2,4]],[[4,0],[7,4]],[[6,4],[1,6]],[[7,0],[4,4]],[[7,2],[5,5]]],
	[[[5,5],[0,6]],[[2,2],[2,7]],[[5,2],[3,4]],[[7,6],[0,7]],[[7,5],[4,6]]],
	[[[0,0],[5,3]],[[0,6],[6,6]],[[3,1],[0,7]],[[0,1],[3,2]],[[7,0],[4,3]]],
	[[[4,3],[0,7]],[[2,6],[6,6]],[[5,0],[7,1]],[[7,0],[1,2]],[[6,2],[5,5]]],
	[[[0,0],[1,2]],[[3,6],[0,7]],[[1,5],[6,6]],[[1,0],[0,2]],[[4,1],[7,3]],[[5,0],[5,7]],[[6,0],[6,2]],[[6,1],[6,3]]],
	[[[6,0],[0,1]],[[5,5],[0,7]],[[3,3],[1,7]],[[0,5],[3,5]],[[6,2],[3,6]],[[7,0],[6,1]]],
	[[[2,2],[0,5]],[[7,3],[0,7]],[[3,3],[6,6]],[[5,2],[0,6]],[[7,2],[3,4]]],
	[[[2,3],[0,6]],[[2,2],[7,7]],[[7,0],[4,5]],[[7,1],[1,6]]],
	[[[5,2],[0,7]],[[2,2],[1,6]],[[3,1],[1,7]],[[1,1],[3,2]]],
	[[[0,0],[7,6]],[[3,0],[4,6]],[[3,1],[3,4]],[[2,1],[5,5]],[[4,0],[5,4]],[[6,1],[4,4]],[[6,2],[5,3]],[[7,4],[5,6]]],

	[[[0,1],[7,3]],[[6,2],[2,5]],[[3,6],[7,7]],[[4,3],[2,4]],[[4,4],[5,7]],[[6,5],[6,7]]],
	[[[6,4],[0,5]],[[5,1],[0,7]],[[4,4],[6,7]],[[2,1],[0,6]],[[5,2],[5,7]],[[6,1],[5,3]],[[5,4],[7,7]]],
	[[[1,1],[6,3]],[[6,5],[2,7]],[[5,2],[3,4]],[[7,3],[1,7]],[[6,2],[3,5]],[[6,4],[5,5]],[[7,4],[6,7]]],
	[[[1,0],[4,1]],[[5,0],[2,5]],[[6,0],[6,6]],[[1,1],[3,1]],[[6,1],[7,3]]],
	[[[0,1],[4,3]],[[2,2],[5,7]],[[6,1],[4,4]],[[1,3],[1,6]],[[4,7],[7,7]],[[5,3],[7,6]]],
	[[[1,1],[7,2]],[[5,5],[2,7]],[[7,3],[4,4]],[[2,5],[6,5]],[[7,0],[6,4]]],
	[[[0,0],[0,5]],[[0,7],[7,7]],[[3,1],[4,5]],[[0,1],[1,5]],[[6,0],[4,3]],[[5,0],[4,4]]],
	[[[5,1],[0,7]],[[2,2],[6,6]],[[5,4],[2,6]],[[3,0],[1,6]],[[7,1],[4,5]]],
	[[[7,1],[0,4]],[[2,4],[1,6]],[[2,5],[1,7]],[[1,4],[2,7]],[[3,3],[4,4]],[[3,4],[5,4]]],
	[[[0,0],[2,1]],[[1,1],[4,3]],[[7,2],[1,3]],[[0,3],[7,6]],[[1,4],[6,6]],[[7,0],[3,1]],[[6,0],[5,3]]],

	[[[0,0],[2,0]],[[1,0],[2,2]],[[1,5],[1,7]],[[0,4],[5,4]],[[2,5],[1,6]],[[2,7],[7,7]],[[3,0],[3,6]],[[4,2],[3,5]],[[5,2],[4,3]]],
	[[[0,1],[3,2]],[[1,1],[2,6]],[[2,1],[1,4]],[[2,4],[0,6]],[[4,0],[3,3]],[[3,4],[4,6]],[[4,5],[5,6]],[[6,1],[6,6]]],
	[[[0,0],[7,2]],[[3,2],[7,7]],[[7,1],[5,5]],[[7,0],[1,6]],[[7,3],[7,6]]],
	[[[2,4],[7,5]],[[6,3],[7,6]],[[7,0],[6,6]],[[3,3],[2,5]]],
	[[[0,0],[3,3]],[[6,2],[1,6]],[[7,0],[7,7]],[[1,2],[7,6]]],
	[[[7,0],[0,1]],[[1,2],[2,3]],[[2,5],[4,5]],[[1,1],[0,2]],[[3,1],[7,1]],[[3,3],[5,6]],[[5,5],[4,6]]],
	[[[0,1],[6,3]],[[1,6],[4,6]],[[5,1],[4,3]],[[4,2],[1,3]],[[4,4],[6,5]],[[7,0],[5,2]],[[5,5],[6,6]]],
	[[[0,4],[0,6]],[[6,3],[1,4]],[[1,6],[3,6]],[[0,5],[3,5]],[[2,2],[4,2]],[[6,1],[4,3]],[[6,2],[5,5]]],
	[[[7,1],[0,5]],[[0,7],[4,7]],[[2,1],[7,5]],[[0,6],[2,6]],[[6,1],[3,3]],[[4,1],[7,3]],[[4,5],[6,6]],[[7,2],[5,3]]],
	[[[0,0],[6,7]],[[3,1],[1,7]],[[2,3],[3,7]],[[1,6],[3,6]],[[6,2],[4,4]],[[5,2],[4,7]],[[6,3],[5,7]]],

	[[[6,1],[0,4]],[[1,1],[6,3]],[[5,3],[1,7]],[[2,2],[0,7]],[[6,6],[2,7]],[[4,5],[7,7]],[[5,0],[5,6]]],
	[[[5,3],[0,7]],[[7,0],[2,6]],[[5,0],[6,7]],[[6,0],[1,2]],[[6,6],[7,7]]],
	[[[0,0],[5,2]],[[1,1],[3,6]],[[4,0],[2,3]],[[1,5],[0,7]],[[4,1],[7,5]],[[5,3],[7,7]],[[6,1],[7,6]]],
	[[[1,1],[4,3]],[[3,5],[2,7]],[[3,1],[5,4]],[[1,6],[3,6]],[[3,2],[3,7]],[[4,1],[6,6]],[[5,1],[6,2]]],
	[[[7,1],[0,2]],[[3,1],[6,5]],[[3,3],[4,4]],[[7,2],[0,3]],[[7,0],[4,3]]],
	[[[0,0],[2,4]],[[4,5],[0,7]],[[1,2],[3,5]],[[0,1],[2,5]],[[6,2],[7,5]],[[7,0],[6,7]],[[7,1],[7,7]]],
	[[[3,1],[1,2]],[[7,3],[1,7]],[[6,1],[2,5]],[[1,3],[7,7]],[[5,1],[3,3]],[[7,4],[3,6]],[[4,1],[5,4]]],
	[[[1,1],[6,6]],[[1,3],[5,5]],[[5,0],[2,5]],[[1,2],[5,2]],[[3,1],[3,4]],[[6,0],[5,4]]],
	[[[0,0],[4,6]],[[1,2],[2,6]],[[2,2],[1,6]],[[0,3],[3,4]],[[6,1],[4,5]],[[7,1],[6,6]]],
	[[[0,0],[6,4]],[[1,2],[2,6]],[[1,3],[6,6]],[[1,4],[0,7]],[[7,3],[1,5]],[[4,1],[6,5]],[[6,1],[5,4]]],

	[[[0,1],[6,2]],[[2,2],[3,5]],[[7,1],[2,5]],[[1,1],[3,6]],[[4,1],[6,6]],[[7,0],[6,1]]],
	[[[0,0],[6,6]],[[1,1],[0,6]],[[2,5],[3,6]],[[0,1],[2,6]],[[3,2],[7,3]],[[6,1],[4,3]]],
	[[[0,0],[2,1]],[[1,3],[7,3]],[[7,2],[3,4]],[[1,0],[7,7]],[[4,0],[7,0]],[[4,1],[6,2]]],
	[[[7,6],[0,7]],[[2,1],[3,4]],[[3,0],[2,2]],[[2,0],[5,5]],[[5,0],[2,4]],[[6,0],[4,5]],[[7,0],[6,4]],[[7,3],[6,6]]],
	[[[0,0],[3,4]],[[0,3],[7,4]],[[0,5],[3,7]],[[0,1],[7,1]],[[2,0],[4,2]],[[4,0],[4,3]],[[4,7],[6,7]],[[7,0],[6,2]],[[6,6],[7,7]]],
	[[[6,1],[0,2]],[[0,5],[2,7]],[[1,1],[3,5]],[[0,4],[3,7]],[[1,4],[1,6]],[[3,0],[7,3]],[[7,5],[4,7]],[[6,3],[5,5]],[[6,2],[7,4]]],
	[[[4,2],[0,3]],[[2,1],[1,2]],[[4,1],[1,3]],[[6,0],[0,4]],[[6,5],[1,6]],[[6,4],[6,6]],[[7,0],[7,7]]],
	[[[0,0],[0,4]],[[6,4],[0,7]],[[2,0],[3,2]],[[1,0],[0,6]],[[6,1],[2,5]],[[6,5],[2,7]],[[7,4],[3,5]],[[5,5],[6,6]]],
	[[[0,0],[4,2]],[[1,2],[5,5]],[[1,3],[5,7]],[[0,1],[2,5]],[[6,1],[3,2]],[[3,4],[7,7]]],
	[[[0,0],[3,5]],[[1,0],[1,3]],[[5,0],[1,6]],[[4,1],[0,7]],[[3,1],[5,1]],[[7,0],[6,7]],[[7,5],[7,7]]],

	[[[0,0],[7,0]],[[0,2],[7,3]],[[7,1],[1,6]],[[0,1],[5,5]],[[3,0],[3,2]],[[5,0],[3,3]]],
	[[[0,1],[2,2]],[[1,3],[4,3]],[[1,4],[2,6]],[[0,6],[6,6]],[[6,2],[1,6]],[[4,5],[7,7]],[[7,4],[5,6]]],
	[[[2,2],[2,5]],[[3,0],[6,1]],[[3,1],[3,4]],[[4,0],[2,4]],[[5,0],[7,7]]],
	[[[0,0],[2,6]],[[3,4],[5,6]],[[4,2],[7,7]],[[0,1],[7,5]],[[6,1],[7,6]]],
	[[[1,1],[2,2]],[[6,1],[1,6]],[[2,0],[6,0]],[[3,0],[1,2]],[[7,0],[5,1]]],
	[[[3,0],[0,3]],[[1,2],[7,7]],[[5,2],[1,3]],[[0,4],[6,6]],[[4,0],[2,5]]],
	[[[7,0],[2,5]],[[6,2],[3,4]],[[6,4],[4,6]],[[1,0],[6,1]],[[4,0],[5,5]],[[3,0],[1,1]]],
	[[[7,0],[2,6]],[[1,0],[5,3]],[[3,4],[5,5]],[[4,7],[7,7]],[[2,2],[4,6]],[[1,4],[1,6]]],
	[[[7,0],[3,4]],[[4,5],[6,6]],[[5,4],[1,7]],[[7,1],[2,2]],[[1,1],[1,6]]],
	[[[0,0],[7,3]],[[4,1],[6,6]],[[1,5],[5,6]],[[4,3],[7,4]],[[2,2],[0,7]],[[0,1],[2,4]]],

	[[[7,0],[4,5]],[[6,2],[3,7]],[[3,0],[0,5]],[[1,6],[7,7]],[[2,2],[0,6]]],
	[[[7,1],[6,4]],[[6,2],[0,7]],[[5,2],[3,3]],[[6,1],[6,3]],[[2,1],[5,5]]],
	[[[4,3],[3,4]],[[3,3],[2,4]],[[2,0],[1,3]],[[1,0],[3,0]],[[1,1],[2,5]]],
	[[[6,1],[2,2]],[[5,3],[4,5]],[[0,0],[5,5]],[[6,4],[1,6]],[[1,0],[4,2]],[[2,1],[4,3]]],
	[[[7,0],[7,3]],[[0,1],[5,2]],[[1,2],[5,6]],[[3,3],[7,7]],[[0,0],[0,2]]],
	[[[3,0],[7,1]],[[4,3],[5,5]],[[1,1],[5,7]],[[5,1],[6,7]],[[0,1],[4,5]],[[0,2],[2,3]],[[1,3],[1,6]]],
	[[[7,3],[5,4]],[[6,1],[2,5]],[[4,4],[6,6]],[[0,1],[7,4]],[[3,1],[2,2]],[[2,1],[0,2]]],
	[[[5,4],[7,7]],[[6,2],[2,4]],[[5,6],[2,7]],[[6,1],[5,5]],[[1,6],[3,7]]],
	[[[7,1],[4,2]],[[0,7],[7,7]],[[6,1],[1,5]],[[7,2],[3,4]],[[2,1],[5,4]],[[0,0],[2,4]]],
	[[[7,0],[1,1]],[[7,3],[6,5]],[[3,3],[7,4]],[[7,2],[2,5]],[[6,0],[6,6]]],

	[[[6,0],[1,1]],[[1,2],[5,6]],[[2,3],[3,7]],[[4,0],[6,6]],[[1,3],[2,5]],[[1,4],[1,6]]],
	[[[5,0],[2,1]],[[5,2],[5,5]],[[4,0],[2,5]],[[5,1],[0,2]],[[2,2],[2,3]]],
	[[[7,0],[5,1]],[[4,1],[6,2]],[[0,3],[6,5]],[[1,1],[7,7]],[[3,0],[2,4]],[[0,2],[3,4]]],
	[[[7,2],[5,5]],[[5,1],[5,6]],[[0,6],[4,6]],[[1,1],[6,1]],[[1,2],[0,7]]],
	[[[7,0],[7,7]],[[3,0],[6,4]],[[1,2],[5,4]],[[1,6],[7,6]],[[2,2],[4,2]],[[1,3],[4,4]],[[2,0],[3,4]]],
	[[[7,0],[2,7]],[[6,0],[2,4]],[[6,4],[3,7]],[[3,6],[7,7]],[[5,0],[0,1]],[[1,1],[0,7]],[[1,2],[0,6]]],
	[[[0,0],[7,4]],[[2,4],[6,5]],[[5,2],[6,6]],[[6,4],[1,6]],[[3,1],[1,5]],[[1,0],[1,2]]],
	[[[7,3],[4,5]],[[6,3],[5,5]],[[3,4],[6,4]],[[6,1],[0,7]],[[0,0],[1,7]]],
	[[[0,0],[7,1]],[[2,1],[5,5]],[[1,2],[4,3]],[[3,1],[2,7]],[[6,1],[1,5]]],
	[[[3,1],[6,2]],[[6,6],[1,7]],[[1,0],[5,2]],[[4,4],[6,5]],[[5,4],[2,5]],[[3,0],[0,7]],[[2,0],[1,1]]],

	[[[7,0],[6,7]],[[3,7],[5,7]],[[4,4],[4,7]],[[1,6],[7,7]]],
	[[[7,6],[2,7]],[[4,0],[2,3]],[[3,5],[1,7]],[[6,1],[7,7]],[[2,1],[0,4]],[[0,0],[1,6]]],
	[[[6,1],[7,3]],[[1,1],[6,6]],[[3,1],[5,2]],[[6,0],[6,2]],[[4,3],[5,4]],[[4,1],[3,4]]],
	[[[1,0],[6,1]],[[6,5],[1,6]],[[0,0],[5,3]],[[1,1],[6,4]],[[4,0],[5,5]],[[2,1],[3,2]]],
	[[[7,0],[6,1]],[[4,2],[7,6]],[[4,1],[7,7]],[[1,1],[7,1]],[[5,5],[1,6]],[[0,0],[5,6]],[[3,2],[2,4]]],
	[[[7,0],[7,2]],[[1,3],[6,4]],[[2,3],[4,4]],[[0,0],[6,1]],[[1,2],[1,4]],[[0,2],[0,6]]],
	[[[7,5],[7,7]],[[5,4],[2,5]],[[3,4],[5,6]],[[4,6],[7,6]],[[3,3],[4,4]]],
	[[[7,4],[6,5]],[[1,1],[6,2]],[[4,4],[6,6]],[[5,2],[7,6]],[[5,3],[1,6]],[[0,1],[3,5]]],
	[[[0,0],[7,7]],[[2,2],[6,2]],[[1,2],[6,5]],[[6,1],[1,4]],[[0,5],[6,6]],[[4,5],[6,7]],[[3,5],[0,7]],[[1,7],[3,7]]],
	[[[6,1],[1,5]],[[3,3],[5,3]],[[4,4],[4,7]],[[4,3],[6,6]],[[3,2],[1,6]]],

	[[[6,6],[7,7]],[[6,4],[5,6]],[[3,4],[6,7]],[[6,1],[2,4]],[[5,0],[3,1]],[[2,2],[5,5]],[[2,1],[1,6]]],
	[[[5,2],[3,3]],[[4,3],[0,6]],[[1,1],[0,7]],[[4,2],[3,6]]],
	[[[6,0],[1,5]],[[4,3],[3,4]],[[2,1],[4,7]],[[4,0],[6,6]],[[3,0],[2,3]],[[2,2],[3,3]],[[2,0],[0,3]]],
	[[[7,0],[5,6]],[[2,1],[4,2]],[[0,0],[4,6]],[[2,5],[6,6]],[[3,1],[3,6]],[[1,2],[2,4]]],
	[[[7,0],[5,2]],[[4,1],[6,1]],[[0,0],[6,3]],[[6,0],[2,2]],[[3,4],[5,5]],[[4,2],[2,5]],[[3,2],[2,4]],[[2,0],[2,6]]],
	[[[7,0],[3,2]],[[6,0],[7,4]],[[6,1],[6,6]],[[7,3],[3,4]],[[1,1],[5,5]]],
	[[[5,1],[6,2]],[[0,1],[5,4]],[[3,0],[0,4]],[[1,4],[6,6]],[[0,0],[3,4]],[[0,2],[2,5]]],
	[[[2,5],[7,6]],[[5,2],[5,7]],[[2,2],[4,4]],[[0,6],[7,7]],[[0,5],[3,6]]],
	[[[7,0],[2,3]],[[1,6],[7,6]],[[4,1],[7,7]],[[7,1],[4,2]],[[0,0],[3,3]]],
	[[[3,3],[7,7]],[[1,6],[5,6]],[[2,0],[0,1]],[[0,0],[6,2]],[[1,2],[2,5]]],

	[[[7,0],[6,2]],[[4,6],[7,7]],[[6,5],[1,6]],[[6,0],[7,2]],[[3,1],[0,3]],[[2,0],[2,7]],[[1,0],[1,2]],[[1,1],[1,3]]],
	[[[1,0],[7,1]],[[2,5],[7,7]],[[4,3],[6,7]],[[4,5],[7,5]],[[1,2],[4,6]],[[0,0],[1,1]]],
	[[[5,2],[7,5]],[[0,3],[7,7]],[[4,3],[1,6]],[[2,2],[7,6]],[[0,2],[4,4]]],
	[[[5,3],[7,6]],[[5,2],[0,7]],[[0,0],[3,5]],[[0,1],[6,6]]],
	[[[2,2],[7,7]],[[5,2],[6,6]],[[4,1],[6,7]],[[6,1],[4,2]]],
	[[[7,0],[0,6]],[[4,0],[3,6]],[[4,1],[4,4]],[[5,1],[2,5]],[[3,0],[2,4]],[[1,1],[3,4]],[[1,2],[2,3]],[[0,4],[2,6]]],
	[[[7,1],[0,3]],[[1,2],[5,5]],[[4,6],[0,7]],[[3,3],[5,4]],[[3,4],[2,7]],[[1,5],[1,7]]],
	[[[1,4],[7,5]],[[2,1],[7,7]],[[3,4],[1,7]],[[5,1],[7,6]],[[2,2],[2,7]],[[1,1],[2,3]],[[2,4],[0,7]]],
	[[[6,1],[1,3]],[[1,5],[5,7]],[[2,2],[4,4]],[[0,3],[6,7]],[[1,2],[4,5]],[[1,4],[2,5]],[[0,4],[1,7]]],
	[[[6,0],[3,1]],[[2,0],[5,5]],[[1,0],[1,6]],[[4,1],[6,1]],[[1,1],[0,3]]],

	[[[7,1],[3,3]],[[5,2],[2,7]],[[1,1],[3,4]],[[6,3],[6,6]],[[0,7],[3,7]],[[2,3],[0,6]]],
	[[[6,1],[0,2]],[[2,5],[5,7]],[[0,3],[3,4]],[[1,5],[5,5]],[[0,0],[1,4]]],
	[[[7,0],[7,5]],[[0,7],[7,7]],[[4,1],[3,5]],[[7,1],[6,5]],[[1,0],[3,3]],[[2,0],[3,4]]],
	[[[2,1],[7,7]],[[5,2],[1,6]],[[2,4],[5,6]],[[4,0],[6,6]],[[0,1],[3,5]]],
	[[[0,1],[7,4]],[[5,4],[6,6]],[[5,5],[6,7]],[[6,4],[5,7]],[[4,3],[3,4]],[[2,4],[4,4]]],
	[[[7,0],[5,1]],[[6,1],[3,3]],[[0,2],[6,3]],[[7,3],[0,6]],[[6,4],[1,6]],[[0,0],[4,1]],[[1,0],[2,3]]],
	[[[5,0],[7,0]],[[6,0],[5,2]],[[6,5],[6,7]],[[2,4],[7,4]],[[5,5],[6,6]],[[0,7],[5,7]],[[4,0],[4,6]],[[3,2],[4,5]],[[2,2],[3,3]]],
	[[[7,1],[4,2]],[[6,1],[5,6]],[[5,1],[6,4]],[[5,4],[7,6]],[[3,0],[4,3]],[[4,4],[3,6]],[[3,5],[2,6]],[[1,1],[1,6]]],
	[[[7,0],[0,2]],[[4,2],[0,7]],[[0,1],[2,5]],[[0,0],[6,6]],[[0,3],[0,6]]],
	[[[5,4],[0,5]],[[1,3],[0,6]],[[0,0],[1,6]],[[4,3],[5,5]]],

	[[[7,0],[4,3]],[[1,2],[6,6]],[[0,0],[0,7]],[[6,2],[0,6]]],
	[[[0,0],[7,1]],[[6,2],[5,3]],[[3,5],[5,5]],[[6,1],[7,2]],[[0,1],[4,1]],[[4,3],[2,6]],[[2,5],[3,6]]],
	[[[7,1],[1,3]],[[3,6],[6,6]],[[2,1],[3,3]],[[3,2],[6,3]],[[3,4],[1,5]],[[0,0],[2,2]],[[2,5],[1,6]]],
	[[[7,4],[7,6]],[[1,3],[6,4]],[[4,6],[6,6]],[[4,5],[7,5]],[[3,2],[5,2]],[[1,1],[3,3]],[[1,2],[2,5]]],
	[[[0,1],[7,5]],[[3,7],[7,7]],[[5,1],[0,5]],[[5,6],[7,6]],[[1,1],[4,3]],[[3,1],[0,3]],[[3,5],[1,6]],[[0,2],[2,3]]],
	[[[7,0],[1,7]],[[4,1],[6,7]],[[5,3],[4,7]],[[4,6],[6,6]],[[1,2],[3,4]],[[2,2],[3,7]],[[1,3],[2,7]]],
	[[[1,1],[7,4]],[[6,1],[1,3]],[[2,3],[6,7]],[[5,2],[7,7]],[[1,6],[5,7]],[[3,5],[0,7]],[[2,0],[2,6]]],
	[[[2,3],[7,7]],[[0,0],[5,6]],[[2,0],[1,7]],[[1,0],[6,2]],[[1,6],[0,7]]],
	[[[7,0],[2,2]],[[6,1],[4,6]],[[3,0],[5,3]],[[6,5],[7,7]],[[3,1],[0,5]],[[2,3],[0,7]],[[1,1],[0,6]]],
	[[[6,1],[3,3]],[[4,5],[5,7]],[[4,1],[2,4]],[[4,6],[6,6]],[[4,2],[4,7]],[[3,1],[1,6]],[[2,1],[1,2]]],

	[[[0,1],[7,2]],[[4,1],[1,5]],[[4,3],[3,4]],[[0,2],[7,3]],[[0,0],[3,3]]],
	[[[7,0],[5,4]],[[3,5],[7,7]],[[6,2],[4,5]],[[7,1],[5,5]],[[1,2],[0,5]],[[0,0],[1,7]],[[0,1],[0,7]]],
	[[[4,1],[6,2]],[[0,3],[6,7]],[[1,1],[5,5]],[[6,3],[0,7]],[[2,1],[4,3]],[[0,4],[4,6]],[[3,1],[2,4]]],
	[[[6,1],[1,6]],[[6,3],[2,5]],[[2,0],[5,5]],[[2,2],[6,2]],[[4,1],[4,4]],[[1,0],[2,4]]],
	[[[7,0],[3,6]],[[6,2],[5,6]],[[5,2],[6,6]],[[7,3],[4,4]],[[1,1],[3,5]],[[0,1],[1,6]]],
	[[[7,0],[1,4]],[[6,2],[5,6]],[[6,3],[1,6]],[[6,4],[7,7]],[[0,3],[6,5]],[[3,1],[1,5]],[[1,1],[2,4]]],
	[[[7,1],[1,2]],[[5,2],[4,5]],[[0,1],[5,5]],[[6,1],[4,6]],[[3,1],[1,6]],[[0,0],[1,1]]],
	[[[7,0],[1,6]],[[6,1],[7,6]],[[5,5],[4,6]],[[7,1],[5,6]],[[4,2],[0,3]],[[1,1],[3,3]]],
	[[[7,0],[5,1]],[[0,3],[6,3]],[[0,2],[4,4]],[[6,0],[0,7]],[[0,0],[3,0]],[[3,1],[1,2]]],
	[[[0,6],[7,7]],[[5,1],[4,4]],[[4,0],[5,2]],[[5,0],[2,5]],[[2,0],[5,4]],[[1,0],[3,5]],[[0,0],[1,4]],[[0,3],[1,6]]],

	[[[7,0],[4,4]],[[7,3],[0,4]],[[7,5],[4,7]],[[0,1],[7,1]],[[5,0],[3,2]],[[3,0],[3,3]],[[1,7],[3,7]],[[0,0],[1,2]],[[1,6],[0,7]]],
	[[[1,1],[7,2]],[[7,5],[5,7]],[[6,1],[4,5]],[[7,4],[4,7]],[[6,4],[6,6]],[[4,0],[0,3]],[[0,5],[3,7]],[[1,3],[2,5]],[[1,2],[0,4]]],
	[[[3,2],[7,3]],[[5,1],[6,2]],[[3,1],[6,3]],[[1,0],[7,4]],[[1,5],[6,6]],[[1,4],[1,6]],[[0,0],[0,7]]],
	[[[7,0],[7,4]],[[1,4],[7,7]],[[5,0],[4,2]],[[6,0],[7,6]],[[1,1],[5,5]],[[1,5],[5,7]],[[0,4],[4,5]],[[2,5],[1,6]]],
	[[[7,0],[3,2]],[[6,2],[2,5]],[[6,3],[2,7]],[[7,1],[5,5]],[[1,1],[4,2]],[[4,4],[0,7]]],
	[[[7,0],[4,5]],[[6,0],[6,3]],[[2,0],[6,6]],[[3,1],[7,7]],[[2,1],[4,1]],[[0,0],[1,7]],[[0,5],[0,7]]],
	[[[0,0],[7,0]],[[7,2],[0,3]],[[0,1],[6,6]],[[7,1],[2,5]],[[4,0],[4,2]],[[2,0],[4,3]]],
	[[[7,1],[5,2]],[[3,3],[6,3]],[[6,4],[5,6]],[[1,6],[7,6]],[[1,2],[6,6]],[[3,5],[0,7]],[[0,4],[2,6]]],
	[[[5,2],[5,5]],[[4,0],[1,1]],[[4,1],[4,4]],[[3,0],[5,4]],[[2,0],[0,7]]],
	[[[7,0],[5,6]],[[4,4],[2,6]],[[3,2],[0,7]],[[7,1],[0,5]],[[1,1],[0,6]]],

	[[[6,1],[5,2]],[[1,1],[6,6]],[[1,0],[5,0]],[[4,0],[6,2]],[[0,0],[2,1]]],
	[[[4,0],[7,3]],[[6,2],[0,7]],[[2,2],[6,3]],[[7,4],[1,6]],[[3,0],[5,5]]]
	];

	this.Level9Data = [
	[[[0,0],[7,1]],[[3,2],[7,2]],[[4,0],[5,6]],[[3,0],[1,7]],[[6,0],[8,8]]],
	[[[1,0],[0,8]],[[6,0],[2,7]],[[3,1],[8,3]],[[1,1],[6,4]],[[3,2],[8,8]],[[3,3],[6,3]],[[4,5],[7,5]],[[5,7],[8,7]],[[7,0],[8,0]]],
	[[[0,0],[0,8]],[[2,0],[3,5]],[[3,0],[2,5]],[[1,0],[5,5]],[[4,0],[4,5]]],
	[[[0,0],[8,1]],[[3,7],[0,8]],[[3,0],[1,1]],[[3,2],[0,3]],[[1,6],[6,6]],[[4,0],[7,5]],[[4,7],[8,7]],[[7,1],[8,2]]],
	[[[2,0],[8,6]],[[8,0],[2,6]],[[4,1],[8,1]],[[2,1],[7,7]],[[4,2],[4,5]],[[4,3],[7,6]],[[7,3],[4,4]],[[6,4],[7,4]]],
	[[[2,3],[0,6]],[[4,1],[0,8]],[[8,0],[1,1]],[[3,3],[0,7]],[[2,0],[7,3]],[[4,7],[7,7]],[[5,4],[7,6]],[[6,6],[6,8]],[[8,5],[7,8]]],
	[[[0,0],[1,4]],[[7,0],[0,8]],[[8,0],[1,8]],[[0,5],[0,6]],[[7,5],[7,7]],[[8,3],[3,8]],[[4,3],[4,4]],[[7,1],[4,5]],[[8,5],[4,8]],[[5,1],[2,2]]],
	[[[0,0],[1,4]],[[1,5],[8,8]],[[1,6],[8,7]],[[1,0],[5,3]],[[2,0],[4,4]],[[4,0],[2,4]],[[3,0],[3,4]]],
	[[[0,0],[3,7]],[[6,4],[0,8]],[[3,4],[1,8]],[[0,1],[1,3]],[[4,4],[3,8]],[[4,8],[8,8]],[[5,8],[7,8]]],
	[[[0,0],[3,8]],[[0,3],[3,3]],[[1,1],[6,6]],[[0,1],[2,8]],[[4,5],[1,7]],[[2,6],[4,6]],[[6,2],[2,7]],[[7,4],[6,7]]],

	[[[0,2],[6,5]],[[0,4],[8,8]],[[3,3],[0,5]],[[8,0],[0,3]],[[0,6],[1,8]],[[2,7],[0,8]],[[7,0],[6,6]]],
	[[[0,0],[1,3]],[[1,0],[2,6]],[[8,0],[1,7]],[[1,2],[0,3]],[[6,1],[3,7]],[[3,8],[7,8]],[[4,0],[6,3]],[[6,4],[4,5]],[[7,7],[8,8]]],
	[[[0,4],[0,8]],[[1,2],[1,4]],[[6,0],[1,7]],[[1,1],[0,5]],[[7,0],[1,8]],[[5,1],[5,6]],[[6,1],[6,6]],[[6,8],[7,8]],[[8,0],[8,8]]],
	[[[0,0],[1,8]],[[1,5],[1,7]],[[2,3],[8,8]],[[1,0],[7,4]],[[3,5],[2,8]],[[4,7],[6,7]],[[5,2],[6,4]],[[6,2],[6,6]]],
	[[[0,0],[4,8]],[[3,6],[7,6]],[[4,3],[6,6]],[[8,0],[1,7]],[[6,3],[5,4]],[[6,4],[5,5]],[[5,6],[8,8]]],
	[[[0,0],[8,0]],[[0,6],[7,8]],[[3,1],[1,2]],[[5,2],[0,4]],[[5,6],[1,7]],[[2,1],[6,4]],[[2,3],[6,3]],[[2,7],[5,7]],[[4,2],[6,6]],[[6,7],[8,8]]],
	[[[0,2],[6,8]],[[1,2],[6,2]],[[1,3],[5,8]],[[1,1],[4,3]],[[1,4],[4,7]],[[2,3],[7,7]],[[2,4],[5,7]],[[4,4],[7,6]]],
	[[[0,0],[3,1]],[[2,5],[1,6]],[[3,5],[1,7]],[[4,1],[1,2]],[[3,2],[3,7]],[[4,2],[6,8]],[[6,1],[8,7]],[[6,2],[6,6]],[[8,5],[8,8]]],
	[[[5,3],[0,4]],[[2,1],[1,4]],[[2,2],[6,4]],[[0,5],[0,8]],[[2,6],[5,8]],[[6,2],[2,8]],[[3,6],[7,7]],[[5,2],[4,7]],[[5,1],[7,2]]],
	[[[0,0],[4,3]],[[1,2],[6,5]],[[1,4],[7,7]],[[1,1],[2,5]],[[3,0],[5,1]],[[4,0],[8,0]],[[4,2],[5,3]],[[4,4],[8,4]],[[7,2],[8,5]]],

	[[[0,0],[6,8]],[[1,0],[2,1]],[[1,1],[2,2]],[[6,7],[0,8]],[[8,0],[1,4]],[[1,7],[5,7]],[[7,2],[3,6]],[[7,4],[8,8]]],
	[[[1,1],[1,4]],[[7,0],[3,4]],[[6,4],[3,5]],[[1,5],[6,7]],[[6,0],[3,6]],[[4,4],[7,4]],[[8,0],[8,4]],[[4,6],[7,7]],[[5,0],[8,7]],[[6,5],[8,6]]],
	[[[1,0],[0,5]],[[2,6],[0,8]],[[1,1],[5,5]],[[1,5],[0,7]],[[6,1],[2,8]],[[8,1],[3,2]],[[4,2],[8,2]],[[8,4],[4,8]],[[8,5],[5,8]],[[7,0],[8,0]]],
	[[[3,5],[6,6]],[[8,5],[4,6]],[[8,3],[8,7]],[[4,4],[4,5]],[[8,4],[8,6]]],
	[[[1,1],[4,4]],[[1,3],[2,7]],[[2,4],[2,6]],[[1,2],[2,3]],[[5,1],[3,7]],[[4,1],[4,3]],[[5,0],[6,2]],[[7,1],[5,7]],[[6,1],[5,8]]],
	[[[0,4],[3,5]],[[5,2],[1,7]],[[2,2],[6,6]],[[2,1],[1,4]],[[5,6],[2,7]],[[4,0],[7,3]],[[4,4],[8,8]],[[8,0],[7,4]],[[7,6],[7,8]]],
	[[[1,1],[7,7]],[[4,4],[2,5]],[[2,6],[7,6]],[[4,2],[2,4]],[[3,0],[4,3]],[[7,3],[4,5]],[[6,1],[5,3]],[[7,1],[5,4]]],
	[[[0,5],[7,8]],[[1,1],[1,3]],[[2,0],[4,3]],[[7,5],[0,8]],[[2,1],[3,5]],[[3,0],[3,4]],[[5,1],[7,4]],[[5,3],[5,5]]],
	[[[0,0],[8,0]],[[0,4],[5,4]],[[5,3],[0,5]],[[0,3],[5,5]],[[2,7],[0,8]],[[1,7],[2,8]],[[4,4],[8,7]],[[7,2],[5,6]]],
	[[[3,3],[4,6]],[[4,3],[5,6]],[[5,5],[7,8]],[[5,3],[3,6]],[[6,8],[8,8]],[[7,1],[7,6]]],

	[[[1,1],[4,7]],[[3,1],[3,4]],[[3,5],[6,6]],[[2,1],[6,7]],[[6,1],[4,4]],[[7,0],[5,2]],[[5,3],[7,3]],[[6,0],[5,4]],[[8,0],[8,8]]],
	[[[2,1],[1,4]],[[1,6],[3,8]],[[7,7],[1,8]],[[1,5],[4,6]],[[2,4],[6,4]],[[2,6],[8,7]],[[3,3],[6,7]],[[4,0],[8,0]],[[5,2],[6,6]]],
	[[[0,0],[8,0]],[[0,2],[6,7]],[[4,3],[0,8]],[[0,1],[6,8]],[[1,4],[2,6]],[[1,5],[6,6]],[[6,3],[2,7]],[[6,4],[4,7]]],
	[[[0,0],[7,2]],[[3,4],[0,8]],[[1,0],[4,2]],[[0,2],[0,7]],[[1,3],[2,3]],[[6,4],[2,8]],[[3,0],[4,0]],[[3,6],[8,7]],[[4,7],[7,7]],[[8,0],[5,3]]],
	[[[8,2],[0,6]],[[1,7],[4,7]],[[7,1],[2,2]],[[1,6],[6,7]],[[2,3],[5,3]],[[5,0],[2,5]],[[3,5],[7,7]],[[6,3],[8,4]]],
	[[[0,0],[3,3]],[[1,4],[3,6]],[[6,5],[1,7]],[[1,0],[7,1]],[[2,0],[6,8]],[[6,4],[2,5]],[[5,3],[7,7]],[[7,2],[6,7]]],
	[[[0,0],[0,1]],[[2,0],[1,4]],[[3,0],[1,7]],[[1,0],[2,4]],[[2,6],[8,8]],[[3,1],[5,4]],[[5,1],[3,4]],[[4,1],[4,4]]],
	[[[7,1],[1,3]],[[2,2],[4,2]],[[8,4],[4,7]],[[7,2],[1,7]],[[8,2],[5,4]],[[8,5],[6,7]],[[8,3],[8,8]]],
	[[[0,0],[3,7]],[[8,1],[1,7]],[[6,2],[2,7]],[[1,0],[1,4]],[[4,1],[4,5]],[[4,2],[4,4]],[[5,5],[4,7]],[[7,3],[5,7]]],
	[[[6,2],[0,4]],[[1,1],[6,3]],[[1,7],[6,8]],[[2,4],[0,5]],[[3,0],[6,5]],[[3,1],[3,6]],[[4,4],[3,8]],[[5,6],[8,8]],[[5,7],[7,8]]],

	[[[1,0],[0,1]],[[3,0],[1,8]],[[8,1],[2,4]],[[0,7],[0,8]],[[8,0],[3,7]],[[7,3],[3,8]],[[4,8],[8,8]],[[8,6],[7,7]]],
	[[[0,0],[8,3]],[[2,1],[8,4]],[[2,2],[2,7]],[[1,2],[1,7]],[[8,1],[5,5]],[[8,0],[8,2]]],
	[[[1,1],[2,7]],[[7,0],[1,4]],[[2,1],[3,3]],[[1,3],[4,5]],[[3,4],[2,6]],[[6,2],[4,8]],[[6,3],[5,7]],[[8,0],[6,1]],[[6,4],[6,6]]],
	[[[2,2],[6,6]],[[7,1],[4,3]],[[5,1],[8,5]],[[2,6],[5,6]],[[6,1],[6,8]],[[7,5],[7,8]],[[7,6],[8,8]]],
	[[[0,0],[6,2]],[[8,0],[0,8]],[[6,1],[1,5]],[[1,0],[0,7]],[[2,2],[2,5]],[[5,5],[5,7]],[[6,5],[5,6]]],
	[[[1,0],[0,2]],[[7,4],[0,8]],[[8,0],[1,5]],[[3,6],[0,7]],[[2,0],[5,8]],[[8,1],[3,4]],[[7,1],[4,2]],[[5,5],[6,5]],[[6,8],[8,8]]],
	[[[4,5],[0,7]],[[1,4],[1,7]],[[1,6],[5,7]],[[4,1],[1,3]],[[6,2],[2,4]],[[3,3],[7,6]],[[3,4],[4,6]],[[4,7],[8,8]],[[5,1],[7,5]]],
	[[[0,0],[6,3]],[[0,8],[2,8]],[[1,0],[1,1]],[[0,7],[3,8]],[[3,1],[2,4]],[[4,0],[4,8]],[[7,1],[4,2]],[[5,0],[5,8]],[[8,7],[7,8]]],
	[[[8,0],[1,1]],[[1,2],[5,2]],[[4,0],[3,6]],[[5,0],[7,7]],[[2,0],[0,8]]],
	[[[7,0],[8,8]],[[2,0],[6,7]],[[5,1],[0,3]],[[7,1],[2,4]],[[5,2],[0,8]],[[2,3],[5,3]],[[1,5],[4,5]],[[0,7],[3,7]],[[0,0],[1,0]]],

	[[[8,0],[8,8]],[[6,0],[5,5]],[[5,0],[6,5]],[[7,0],[3,5]],[[4,0],[4,5]]],
	[[[8,0],[0,1]],[[5,7],[8,8]],[[5,0],[7,1]],[[5,2],[8,3]],[[2,6],[7,6]],[[4,0],[1,5]],[[0,7],[4,7]],[[1,1],[0,2]]],
	[[[6,0],[0,6]],[[0,0],[6,6]],[[0,1],[4,1]],[[6,1],[1,7]],[[4,2],[4,5]],[[4,3],[1,6]],[[1,3],[4,4]],[[1,4],[2,4]]],
	[[[6,3],[8,6]],[[4,1],[8,8]],[[0,0],[7,1]],[[5,3],[8,7]],[[6,0],[1,3]],[[1,7],[4,7]],[[3,4],[1,6]],[[2,6],[2,8]],[[0,5],[1,8]]],
	[[[8,0],[7,4]],[[1,0],[8,8]],[[0,0],[7,8]],[[8,5],[8,6]],[[3,1],[6,2]],[[0,3],[5,8]],[[4,3],[4,4]],[[1,1],[4,5]],[[0,5],[4,8]],[[1,5],[1,7]]],
	[[[8,0],[7,4]],[[7,5],[0,8]],[[7,6],[0,7]],[[7,0],[3,3]],[[6,0],[4,4]],[[4,0],[6,4]],[[5,0],[5,4]]],
	[[[8,0],[5,7]],[[2,4],[8,8]],[[5,4],[7,8]],[[8,1],[7,3]],[[4,4],[5,8]],[[0,8],[4,8]],[[1,8],[3,8]]],
	[[[8,0],[5,8]],[[5,3],[8,3]],[[7,1],[2,6]],[[8,1],[6,8]],[[4,5],[7,7]],[[4,6],[6,6]],[[2,2],[6,7]],[[1,4],[2,7]]],
	[[[8,2],[2,5]],[[8,4],[0,8]],[[5,3],[8,5]],[[0,0],[8,3]],[[8,6],[7,8]],[[6,7],[8,8]],[[1,0],[2,6]]],
	[[[8,0],[7,3]],[[7,0],[6,6]],[[0,0],[7,7]],[[7,2],[8,3]],[[2,1],[5,7]],[[1,8],[5,8]],[[4,0],[2,3]],[[2,4],[4,5]],[[1,7],[0,8]]],

	[[[8,4],[8,8]],[[7,2],[7,4]],[[2,0],[7,7]],[[7,1],[8,5]],[[1,0],[7,8]],[[3,1],[3,6]],[[2,1],[2,6]],[[1,8],[2,8]],[[0,0],[0,8]]],
	[[[8,0],[7,8]],[[7,5],[7,7]],[[6,3],[0,8]],[[7,0],[1,4]],[[5,5],[6,8]],[[2,7],[4,7]],[[3,2],[2,4]],[[2,2],[2,6]]],
	[[[8,0],[4,8]],[[1,6],[5,6]],[[4,3],[2,6]],[[0,0],[7,7]],[[2,3],[3,4]],[[2,4],[3,5]],[[3,6],[0,8]]],
	[[[0,0],[8,0]],[[8,6],[1,8]],[[5,1],[7,2]],[[3,2],[8,4]],[[3,6],[7,7]],[[6,1],[2,4]],[[2,3],[6,3]],[[3,7],[6,7]],[[4,2],[2,6]],[[2,7],[0,8]]],
	[[[8,2],[2,8]],[[2,2],[7,2]],[[7,3],[3,8]],[[7,1],[4,3]],[[7,4],[4,7]],[[6,3],[1,7]],[[6,4],[3,7]],[[4,4],[1,6]]],
	[[[8,0],[5,1]],[[6,5],[7,6]],[[5,5],[7,7]],[[4,1],[7,2]],[[5,2],[5,7]],[[4,2],[2,8]],[[2,1],[0,7]],[[2,2],[2,6]],[[0,5],[0,8]]],
	[[[3,3],[8,4]],[[6,1],[7,4]],[[6,2],[2,4]],[[8,5],[8,8]],[[6,6],[3,8]],[[2,2],[6,8]],[[5,6],[1,7]],[[3,2],[4,7]],[[3,1],[1,2]]],
	[[[8,0],[4,3]],[[7,2],[2,5]],[[7,4],[1,7]],[[7,1],[6,5]],[[5,0],[3,1]],[[0,0],[4,0]],[[4,2],[3,3]],[[0,4],[4,4]],[[1,2],[0,5]]],
	[[[8,0],[2,8]],[[7,0],[6,1]],[[7,1],[6,2]],[[2,7],[8,8]],[[0,0],[7,4]],[[3,7],[7,7]],[[1,2],[5,6]],[[1,4],[0,8]]],
	[[[7,1],[7,4]],[[1,0],[5,4]],[[2,4],[5,5]],[[7,5],[2,7]],[[2,0],[5,6]],[[1,4],[4,4]],[[4,6],[1,7]],[[3,0],[0,7]],[[2,5],[0,6]],[[0,0],[0,4]]],

	[[[7,0],[8,5]],[[6,6],[8,8]],[[7,1],[3,5]],[[7,5],[8,7]],[[2,1],[6,8]],[[0,1],[5,2]],[[0,2],[4,2]],[[0,4],[4,8]],[[0,5],[3,8]],[[0,0],[1,0]]],
	[[[5,5],[2,6]],[[0,5],[4,6]],[[0,3],[0,7]],[[4,4],[4,5]],[[0,4],[0,6]]],
	[[[7,1],[4,4]],[[7,3],[6,7]],[[6,4],[6,6]],[[7,2],[6,3]],[[3,1],[5,7]],[[4,1],[4,3]],[[3,0],[2,2]],[[1,1],[3,7]],[[2,1],[3,8]]],
	[[[8,4],[5,5]],[[3,2],[7,7]],[[6,2],[2,6]],[[6,1],[7,4]],[[3,6],[6,7]],[[4,0],[1,3]],[[4,4],[0,8]],[[0,0],[1,4]],[[1,6],[1,8]]],
	[[[7,1],[1,7]],[[4,4],[6,5]],[[1,6],[6,6]],[[4,2],[6,4]],[[5,0],[4,3]],[[1,3],[4,5]],[[2,1],[3,3]],[[1,1],[3,4]]],
	[[[8,5],[1,8]],[[7,1],[7,3]],[[6,0],[4,3]],[[1,5],[8,8]],[[6,1],[5,5]],[[5,0],[5,4]],[[3,1],[1,4]],[[3,3],[3,5]]],
	[[[0,0],[8,0]],[[3,4],[8,4]],[[3,3],[8,5]],[[8,3],[3,5]],[[6,7],[8,8]],[[7,7],[6,8]],[[4,4],[0,7]],[[1,2],[3,6]]],
	[[[5,3],[4,6]],[[4,3],[3,6]],[[3,5],[1,8]],[[3,3],[5,6]],[[0,8],[2,8]],[[1,1],[1,6]]],
	[[[7,1],[4,7]],[[5,1],[5,4]],[[5,5],[2,6]],[[6,1],[2,7]],[[2,1],[4,4]],[[1,0],[3,2]],[[1,3],[3,3]],[[2,0],[3,4]],[[0,0],[0,8]]],
	[[[6,1],[7,4]],[[7,6],[5,8]],[[1,7],[7,8]],[[7,5],[4,6]],[[2,4],[6,4]],[[6,6],[0,7]],[[5,3],[2,7]],[[0,0],[4,0]],[[3,2],[2,6]]],

	[[[0,0],[8,0]],[[8,2],[2,7]],[[4,3],[8,8]],[[8,1],[2,8]],[[7,4],[6,6]],[[7,5],[2,6]],[[2,3],[6,7]],[[2,4],[4,7]]],
	[[[8,0],[1,2]],[[5,4],[8,8]],[[7,0],[4,2]],[[8,2],[8,7]],[[6,3],[7,3]],[[2,4],[6,8]],[[4,0],[5,0]],[[5,6],[0,7]],[[1,7],[4,7]],[[0,0],[3,3]]],
	[[[0,2],[8,6]],[[4,7],[7,7]],[[1,1],[6,2]],[[7,6],[2,7]],[[3,3],[6,3]],[[3,0],[6,5]],[[5,5],[1,7]],[[2,3],[0,4]]],
	[[[8,0],[5,3]],[[7,4],[5,6]],[[2,5],[7,7]],[[7,0],[1,1]],[[6,0],[2,8]],[[2,4],[6,5]],[[3,3],[1,7]],[[1,2],[2,7]]],
	[[[8,0],[8,1]],[[6,0],[7,4]],[[5,0],[7,7]],[[7,0],[6,4]],[[6,6],[0,8]],[[5,1],[3,4]],[[3,1],[5,4]],[[4,1],[4,4]]],
	[[[1,1],[7,3]],[[4,2],[6,2]],[[0,4],[4,7]],[[1,2],[7,7]],[[0,2],[3,4]],[[0,5],[2,7]],[[0,3],[0,8]]],
	[[[8,0],[5,7]],[[0,1],[7,7]],[[2,2],[6,7]],[[7,0],[7,4]],[[4,1],[4,5]],[[4,2],[4,4]],[[3,5],[4,7]],[[1,3],[3,7]]],
	[[[2,2],[8,4]],[[7,1],[2,3]],[[7,7],[2,8]],[[6,4],[8,5]],[[5,0],[2,5]],[[5,1],[5,6]],[[4,4],[5,8]],[[3,6],[0,8]],[[3,7],[1,8]]],
	[[[7,0],[8,1]],[[5,0],[7,8]],[[0,1],[6,4]],[[8,7],[8,8]],[[0,0],[5,7]],[[1,3],[5,8]],[[0,8],[4,8]],[[0,6],[1,7]]],
	[[[8,0],[0,3]],[[6,1],[0,4]],[[6,2],[6,7]],[[7,2],[7,7]],[[0,1],[3,5]],[[0,0],[0,2]]],

	[[[7,1],[6,7]],[[1,0],[7,4]],[[6,1],[5,3]],[[7,3],[4,5]],[[5,4],[6,6]],[[2,2],[4,8]],[[2,3],[3,7]],[[0,0],[2,1]],[[2,4],[2,6]]],
	[[[6,2],[2,6]],[[1,1],[4,3]],[[3,1],[0,5]],[[3,6],[6,6]],[[2,1],[2,8]],[[1,5],[1,8]],[[1,6],[0,8]]],
	[[[8,0],[2,2]],[[0,0],[8,8]],[[2,1],[7,5]],[[7,0],[8,7]],[[6,2],[6,5]],[[3,5],[3,7]],[[2,5],[3,6]]],
	[[[7,0],[8,2]],[[1,4],[8,8]],[[0,0],[7,5]],[[5,6],[8,7]],[[6,0],[3,8]],[[0,1],[5,4]],[[1,1],[4,2]],[[2,5],[3,5]],[[0,8],[2,8]]],
	[[[4,5],[8,7]],[[7,4],[7,7]],[[7,6],[3,7]],[[4,1],[7,3]],[[2,2],[6,4]],[[5,3],[1,6]],[[5,4],[4,6]],[[4,7],[0,8]],[[3,1],[1,5]]],
	[[[8,0],[2,3]],[[6,8],[8,8]],[[7,0],[7,1]],[[8,7],[5,8]],[[5,1],[6,4]],[[4,0],[4,8]],[[1,1],[4,2]],[[3,0],[3,8]],[[0,7],[1,8]]]
	];

	this.Level5Scores = [];
	this.Level6Scores = [];
	this.Level7Scores = [];
	this.Level8Scores = [];
	this.Level9Scores = [];

	this.LevelData = this.Level5Data;
	this.LevelScores = this.Level5Scores;
	this.Level = 0;
	this.Difficulty = 0;

	var CaptionTop = 70;
	var CaptionLeft = gCanvasWidth / 2;
	var ButtonWidth = 280;
	var ButtonHeight = 50;
	var ButtonLeft = (gCanvasWidth - ButtonWidth) / 2;
	var ButtonTop = CaptionTop * 2;
	var ButtonSpace = 20;

	this.ButtonImageList = new TImageList();
	this.BackImageIndex = this.ButtonImageList.add("images/back.png");
	this.PerfectlyImageIndex = this.ButtonImageList.add("images/star20.png");
	this.SolvedImageIndex = this.ButtonImageList.add("images/OK20.png");
	this.LevelImageIndex = this.ButtonImageList.add("images/level20.png");
	this.SizeImageIndex = this.ButtonImageList.add("images/size20.png");
	this.MenuImageIndex = this.ButtonImageList.add("images/menu.png");
	this.RestartImageIndex = this.ButtonImageList.add("images/restart.png");
	this.NextImageIndex = this.ButtonImageList.add("images/next.png");

	this.PerfectlyImage = new TStaticImage("images/star20.png", 185, 50);
	this.SolvedImage = new TStaticImage("images/OK20.png", 185, 50);

	var PageControl = new TPageControl(0, 0, gCanvasWidth, gCanvasHeight);
	var StartPage = PageControl.createPage();
	var HelpPage = PageControl.createPage();
	var DiffPage = PageControl.createPage();
	var LevelPage = PageControl.createPage();
	var GamePage = PageControl.createPage();
	var SettingsPage = PageControl.createPage();
	var SlideControl = new TSlideControl(50, 100, 380, 420);
	LevelPage.addControl(SlideControl);

	StartPage.setColor("#330033");
	DiffPage.setColor("#003300");
	HelpPage.setColor("#000033");
	LevelPage.setColor("#333300");
	SettingsPage.setColor("#330000");
	GamePage.setColor("#000000");

	/**
	 * @param {string} caption
	 * @param {number} aLevel
	 * @param {boolean=} aMove
	 */
	this.LevelButtonClick = function (caption, aLevel, aMove) {
	    this.Level = aLevel;
	    this.GameDiffLabel.setCaption(this.DiffButtons[this.Difficulty - 5].getCaption() + "  (" + this.Difficulty + " x " + this.Difficulty + ")");
	    this.GameLevelLabel.setCaption((this.Level + 1) + " / " + this.LevelData.length);
	    this.MovesLabel.setCaption("0 / " + this.LevelData[this.Level].length);
	    PageControl.setPage(GamePage);
	    this.GameEndPanel.hide();
	    this.NoMoreLevelsPanel.hide();
	    if (aMove) {
	        if (this.OldGameGrid) this.GameGridMovedOut(this.OldGameGrid);
	        this.OldGameGrid = this.GameGrid;
	        this.OldGameGrid.OnMovedOut = this.GameGridMovedOut.bind(this);
	        delete this.GameGrid;
	        this.GameGrid = new TGameGrid(97, gCanvasWidth, 470, this.GameMovesChanged.bind(this), this.GameEnd.bind(this));
	        this.GameGrid.setLevelData(this.Difficulty, this.LevelData[this.Level]);
	        GamePage.insertControl(this.GameGrid, 0);
	        this.OldGameGrid.moveOut();
	        this.GameGrid.moveIn();
	    } else this.GameGrid.setLevelData(this.Difficulty, this.LevelData[this.Level]);
	    this.GameGrid.ReadOnly = false;
	}

	this.CreateLevelButtons = function (aDifficulty) {
	    this.Difficulty = aDifficulty;
	    if (this.Difficulty == 6) {
	        this.LevelData = this.Level6Data;
	        this.LevelScores = this.Level6Scores;
	    } else if (this.Difficulty == 7) {
	        this.LevelData = this.Level7Data;
	        this.LevelScores = this.Level7Scores;
	    } else if (this.Difficulty == 8) {
	        this.LevelData = this.Level8Data;
	        this.LevelScores = this.Level8Scores;
	    } else if (this.Difficulty == 9) {
	        this.LevelData = this.Level9Data;
	        this.LevelScores = this.Level9Scores;
	    } else {
	        this.LevelData = this.Level5Data;
	        this.LevelScores = this.Level5Scores;
	    }
	    this.UpdateLevelButtons();
	}

	this.UpdateLevelButtons = function () {
	    SlideControl.clearSlides();
	    var xButtons = 4;
	    var yButtons = 6;
	    var buttonsPerSlide = xButtons * yButtons;
	    var levelCount = this.LevelData.length;
	    var slideCount = ~~ (levelCount / buttonsPerSlide);
	    if ((levelCount % buttonsPerSlide > 0) || (levelCount == 0)) slideCount += 1;

	    var slides = [];
	    for (var i = 0; i < slideCount; i++) {
	        var slide = SlideControl.createPage();
	        slides.push(slide);
			slide.setColor(LevelPage.Color);

	        var buttonCount = buttonsPerSlide;
	        var buttonSpace = 10;
	        var buttonWidth = ~~ (((slide.Width - buttonSpace) / xButtons) - buttonSpace);
	        var buttonHeight = ~~ (((slide.Height - buttonSpace) / yButtons) - buttonSpace);
	        if (i == slideCount - 1) {
				buttonCount = levelCount % buttonsPerSlide;
				if (buttonCount == 0) buttonCount = buttonsPerSlide;
			}
	        for (var j = 0; j < buttonCount; j++) {
	            var x = (j % xButtons);
	            var y = ~~ (j / xButtons);
	            var buttonNumber = i * buttonsPerSlide + j + 1;
	            var button = new TStandardButton(buttonNumber.toString(), buttonSpace + x * (buttonWidth + buttonSpace), buttonSpace + y * (buttonHeight + buttonSpace), buttonWidth, buttonHeight, this.LevelButtonClick.bind(this), buttonNumber - 1, this.ButtonImageList);
	            slide.addControl(button);
	            if (this.LevelScores[buttonNumber - 1] == 1) button.setImageIndex(this.SolvedImageIndex);
	            else if (this.LevelScores[buttonNumber - 1] == 2) button.setImageIndex(this.PerfectlyImageIndex);
	            else button.setImageIndex(-1);
	        }
	    }
	}

	this.GameMovesChanged = function (moves, totalMoves) {
	    this.MovesLabel.setCaption(moves + " / " + totalMoves);
	}

	this.GameEnd = function (moves, totalMoves, almost) {
	    this.PerfectlyImage.hide();
	    this.SolvedImage.hide();
	    if (almost) {
	        this.GameEndLabel1.setPosition(200, 70);
	        this.GameEndLabel2.setPosition(200, 140);
	        this.GameEndLabel1.setCaption("Almost solved.")
	        this.GameEndLabel2.setCaption("The entire board must be covered with lines or circles.");
	    } else {
	        this.GameEndLabel1.setPosition(200, 125);
	        this.GameEndLabel2.setPosition(200, 170);
	        this.GameEndLabel1.setCaption("Level completed.")
	        var s = "Moves: " + moves + " of " + totalMoves;
	        if (moves == totalMoves) {
	            this.GameEndLabel2.setCaption(s + " (perfect)");
	            this.PerfectlyImage.show();
	            if (this.LevelScores[this.Level] < 2) {
	                this.LevelScores[this.Level] = 2;
	                this.SaveScores();
	                this.UpdateLevelButtons();
	            }
	        } else {
	            this.GameEndLabel2.setCaption(s);
	            this.SolvedImage.show();
	            if (this.LevelScores[this.Level] == 0) {
	                this.LevelScores[this.Level] = 1;
	                this.SaveScores();
	                this.UpdateLevelButtons();
	            }
	        }
	    }
	    this.GameGrid.ReadOnly = true;
	    this.GameEndPanel.show();
	}

	this.CloseGameEnd = function () {
	    this.GameEndPanel.hide();
	    this.NoMoreLevelsPanel.hide();
	    this.GameGrid.ReadOnly = false;
	    GamePage.NeedRepaint = true;
	}

	this.NoMoreLevels = function () {
	    this.GameGrid.ReadOnly = true;
	    this.GameEndPanel.hide();
	    this.NoMoreLevelsPanel.show();
	}

	this.GameGridMovedOut = function (aGameGrid) {
	    GamePage.delControl(aGameGrid);
	}

	this.ShowLevelPage = function (caption, ID) {
	    this.CreateLevelButtons(ID);
	    PageControl.setPage(LevelPage);
	}

	this.ShowDiffPage = function (caption, ID) {
		this.updateDiffButtons();
	    PageControl.setPage(DiffPage);
	    SlideControl.setPageIndex(0);
	}

	this.ShowHelpPage = function (caption, ID) {
	    PageControl.setPage(HelpPage);
	}

	this.ShowStartPage = function (caption, ID) {
	    PageControl.setPage(StartPage);
	}

	this.ShowSettingsPage = function (caption, ID) {
	    PageControl.setPage(SettingsPage);
	}

	this.CancelLevel = function () {
	    SlideControl.setPageIndex(~~(this.Level / 24));
	    PageControl.setPage(LevelPage);
	}

	this.RestartLevel = function () {
	    this.LevelButtonClick("", this.Level, false);
	}

	this.StartNextLevel = function () {
	    if (this.Level < this.LevelData.length - 1) this.Level += 1;
	    else if (this.Level == this.LevelData.length - 1) {
	        if (this.Difficulty < 9) {
	            this.CreateLevelButtons(this.Difficulty + 1)
	            this.Level = 0;
	        }
			else {
				this.NoMoreLevels();
				return;
			}
	    }
	    this.LevelButtonClick("", this.Level, true);
	}

	this.ToggleHQ = function () {
		gSettings.setHQ(!this.HQCheckbox.Checked);
		PageControl.NeedRepaint = true;
	}

	this.LoadScores = function () {
	    var cookies = document.cookie.split(";");
	    for (var i = 0; i < cookies.length; i++) {
	        var name = cookies[i].substr(0, cookies[i].indexOf("="));
	        var value = cookies[i].substr(cookies[i].indexOf("=") + 1);
	        name = name.replace(/^\s+|\s+$/g, "");
			try {
				if (name == "neonflow_level5score") this.Level5Scores = JSON.parse(value);
				else if (name == "neonflow_level6score") this.Level6Scores = JSON.parse(value);
				else if (name == "neonflow_level7score") this.Level7Scores = JSON.parse(value);
				else if (name == "neonflow_level8score") this.Level8Scores = JSON.parse(value);
				else if (name == "neonflow_level9score") this.Level9Scores = JSON.parse(value);
			} catch (e) {
				this.InitScores();
				break;
			}
	    }
	    while (this.Level5Scores.length < this.Level5Data.length) this.Level5Scores.push(0);
	    while (this.Level6Scores.length < this.Level6Data.length) this.Level6Scores.push(0);
	    while (this.Level7Scores.length < this.Level7Data.length) this.Level7Scores.push(0);
	    while (this.Level8Scores.length < this.Level8Data.length) this.Level8Scores.push(0);
	    while (this.Level9Scores.length < this.Level9Data.length) this.Level9Scores.push(0);
	}

	this.SaveScores = function () {
	    var expiryDate = new Date();
	    expiryDate.setDate(expiryDate.getDate() + 100000);
	    sExpiry = "; expires=" + expiryDate.toUTCString();
	    document.cookie = "neonflow_level5score=" + JSON.stringify(this.Level5Scores) + sExpiry;
	    document.cookie = "neonflow_level6score=" + JSON.stringify(this.Level6Scores) + sExpiry;
	    document.cookie = "neonflow_level7score=" + JSON.stringify(this.Level7Scores) + sExpiry;
	    document.cookie = "neonflow_level8score=" + JSON.stringify(this.Level8Scores) + sExpiry;
	    document.cookie = "neonflow_level9score=" + JSON.stringify(this.Level9Scores) + sExpiry;
	}

	this.InitScores = function () {
	    this.Level5Scores.length = this.Level5Data.length;
	    this.Level6Scores.length = this.Level6Data.length;
	    this.Level7Scores.length = this.Level6Data.length;
	    this.Level8Scores.length = this.Level8Data.length;
	    this.Level9Scores.length = this.Level9Data.length;
	    for (var i = 0; i < this.Level5Scores.length; i++) this.Level5Scores[i] = 0;
	    for (var i = 0; i < this.Level6Scores.length; i++) this.Level6Scores[i] = 0;
	    for (var i = 0; i < this.Level7Scores.length; i++) this.Level7Scores[i] = 0;
	    for (var i = 0; i < this.Level8Scores.length; i++) this.Level8Scores[i] = 0;
	    for (var i = 0; i < this.Level9Scores.length; i++) this.Level9Scores[i] = 0;
	}

	this.ClearScores = function () {
	    sExpiry = "; expires=Thu, 01 Jan 1970 00:00:00 GMT";
	    document.cookie = "neonflow_level5score=" + sExpiry;
	    document.cookie = "neonflow_level6score=" + sExpiry;
	    document.cookie = "neonflow_level7score=" + sExpiry;
	    document.cookie = "neonflow_level8score=" + sExpiry;
	    document.cookie = "neonflow_level9score=" + sExpiry;
	}

	this.getSolvedCount = function (levelScores) {
		var count = 0;
	    for (var i = 0; i < levelScores.length; i++) if (levelScores[i] == 1) count += 1;
		return count;
	}

	this.getPerfectlyCount = function (levelScores) {
		var count = 0;
	    for (var i = 0; i < levelScores.length; i++) if (levelScores[i] == 2) count += 1;
		return count;
	}

	this.updateDiffButtons = function () {
		this.DiffButton5.setLevels(this.Level5Data.length);
		this.DiffButton6.setLevels(this.Level6Data.length);
		this.DiffButton7.setLevels(this.Level7Data.length);
		this.DiffButton8.setLevels(this.Level8Data.length);
		this.DiffButton9.setLevels(this.Level9Data.length);

		this.DiffButton5.setSolved(this.getSolvedCount(this.Level5Scores));
		this.DiffButton6.setSolved(this.getSolvedCount(this.Level6Scores));
		this.DiffButton7.setSolved(this.getSolvedCount(this.Level7Scores));
		this.DiffButton8.setSolved(this.getSolvedCount(this.Level8Scores));
		this.DiffButton9.setSolved(this.getSolvedCount(this.Level9Scores));

		this.DiffButton5.setPerfectly(this.getPerfectlyCount(this.Level5Scores));
		this.DiffButton6.setPerfectly(this.getPerfectlyCount(this.Level6Scores));
		this.DiffButton7.setPerfectly(this.getPerfectlyCount(this.Level7Scores));
		this.DiffButton8.setPerfectly(this.getPerfectlyCount(this.Level8Scores));
		this.DiffButton9.setPerfectly(this.getPerfectlyCount(this.Level9Scores));
	}

	this.LoadScores();

	this.DiffButton5 = new TDiffButton("Incredibly easy", ButtonLeft, ButtonTop, ButtonWidth, ButtonHeight, this.ShowLevelPage.bind(this), 5, this.ButtonImageList);
	this.DiffButton6 = new TDiffButton("Very easy", ButtonLeft, ButtonTop + ButtonHeight + ButtonSpace, ButtonWidth, ButtonHeight, this.ShowLevelPage.bind(this), 6, this.ButtonImageList);
	this.DiffButton7 = new TDiffButton("Easy", ButtonLeft, ButtonTop + 2 * (ButtonHeight + ButtonSpace), ButtonWidth, ButtonHeight, this.ShowLevelPage.bind(this), 7, this.ButtonImageList);
	this.DiffButton8 = new TDiffButton("Less easy", ButtonLeft, ButtonTop + 3 * (ButtonHeight + ButtonSpace), ButtonWidth, ButtonHeight, this.ShowLevelPage.bind(this), 8, this.ButtonImageList);
	this.DiffButton9 = new TDiffButton("Not completely easy", ButtonLeft, ButtonTop + 4 * (ButtonHeight + ButtonSpace), ButtonWidth, ButtonHeight, this.ShowLevelPage.bind(this), 9, this.ButtonImageList);
	this.DiffButtons = [this.DiffButton5, this.DiffButton6, this.DiffButton7, this.DiffButton8, this.DiffButton9];
	for (var i = 0; i < this.DiffButtons.length; i++) DiffPage.addControl(this.DiffButtons[i]);
	for (var i = 0; i < this.DiffButtons.length; i++) this.DiffButtons[i].setImageIndexes(this.SizeImageIndex, this.LevelImageIndex, this.SolvedImageIndex, this.PerfectlyImageIndex);

	this.updateDiffButtons();

	DiffPage.addControl(new TStandardButton("Back", ButtonLeft, gCanvasHeight - 100 - ButtonHeight, ButtonWidth, ButtonHeight, this.ShowStartPage.bind(this), 0, this.ButtonImageList, this.BackImageIndex));
	LevelPage.addControl(new TStandardButton("Back", ButtonLeft, gCanvasHeight - 35 - ButtonHeight, ButtonWidth, ButtonHeight, this.ShowDiffPage.bind(this), 0, this.ButtonImageList, this.BackImageIndex));
	StartPage.addControl(new TStandardButton("Play", ButtonLeft, ButtonTop + ButtonHeight + ButtonSpace, ButtonWidth, ButtonHeight, this.ShowDiffPage.bind(this)));
	StartPage.addControl(new TStandardButton("How to play", ButtonLeft, ButtonTop + 2 * (ButtonHeight + ButtonSpace), ButtonWidth, ButtonHeight, this.ShowHelpPage.bind(this)));
	StartPage.addControl(new TStandardButton("Settings", ButtonLeft, ButtonTop + 3 * (ButtonHeight + ButtonSpace), ButtonWidth, ButtonHeight, this.ShowSettingsPage.bind(this)));
	SettingsPage.addControl(new TStandardButton("Back", ButtonLeft, gCanvasHeight - 100 - ButtonHeight, ButtonWidth, ButtonHeight, this.ShowStartPage.bind(this), 0, this.ButtonImageList, this.BackImageIndex));
	
	var TitleLabel = new TNeonLabel("Neon Flow", CaptionLeft, CaptionTop, TNeonColor.nePink, true);
	TitleLabel.setFont("bold 50px Arial");
	StartPage.addControl(TitleLabel);
	HelpPage.addControl(new TNeonLabel("How to play", CaptionLeft, CaptionTop, TNeonColor.neBlue, false));
	DiffPage.addControl(new TNeonLabel("Select difficulty", CaptionLeft, CaptionTop, TNeonColor.neGreen, true));
	LevelPage.addControl(new TNeonLabel("Select level", CaptionLeft, 50, TNeonColor.neYellow, true));
	SettingsPage.addControl(new TNeonLabel("Settings", CaptionLeft, CaptionTop, TNeonColor.neOrange, true));
	PageControl.setPage(StartPage);

	SettingsPage.addControl(new TStandardButton("Clear scores", ButtonLeft, ButtonTop, ButtonWidth, ButtonHeight, this.ClearScores.bind(this)));
	this.HQCheckbox = new TCheckbox("Use low quality graphics", ButtonLeft, ButtonTop + ButtonHeight + 50, ButtonWidth, 30, this.ToggleHQ.bind(this), !gSettings.HQ);
	SettingsPage.addControl(this.HQCheckbox);
	this.SettingsLabel = new TLabel("Google Chrome is currently the only browser that is fast enough to handle high quality graphics.", ButtonLeft, this.HQCheckbox.Top + 70, TTextAlignment.taLeft);
	this.SettingsLabel.setFont("bold 20px Arial");
	SettingsPage.addControl(this.SettingsLabel);
	this.SettingsLabel.setMaxWidth(SettingsPage.Width - 2 * ButtonLeft);

	var tmp = new TLabel("Difficulty:", 16, 20, TTextAlignment.taLeft);
	var LabelWidth = tmp.getWidth();
	GamePage.addControl(tmp);
	GamePage.addControl(new TLabel("Level:", 16, 47, TTextAlignment.taLeft));
	GamePage.addControl(new TLabel("Moves:", 16, 74, TTextAlignment.taLeft));
	this.GameGrid = new TGameGrid(97, gCanvasWidth, 470, this.GameMovesChanged.bind(this), this.GameEnd.bind(this));
	GamePage.addControl(this.GameGrid);
	this.OldGameGrid = null;

	this.GameDiffLabel = new TLabel("", LabelWidth + 28, 20, TTextAlignment.taLeft);
	this.GameDiffLabel.setFontColor("yellow");
	GamePage.addControl(this.GameDiffLabel);
	this.GameLevelLabel = new TLabel("", LabelWidth + 28, 47, TTextAlignment.taLeft);
	this.GameLevelLabel.setFontColor("yellow");
	GamePage.addControl(this.GameLevelLabel);
	this.MovesLabel = new TLabel("", LabelWidth + 28, 74, TTextAlignment.taLeft);
	this.MovesLabel.setFontColor("yellow");
	GamePage.addControl(this.MovesLabel);

	var panelWidth = 400;
	var panelHeight = 400;
	var panelBorder = 50;
	var panelButtonWidth = panelWidth - 2 * panelBorder;
	var panelButtonHeight = 50;
	this.GameEndPanel = new TPanel((gCanvasWidth - panelWidth) / 2, (gCanvasHeight - panelHeight) / 2, panelWidth, panelHeight);
	this.GameEndPanel.hide();
	GamePage.addControl(this.GameEndPanel);
	this.GameEndPanel.addControl(new TStandardButton("Next level", panelBorder, panelHeight - panelBorder - 2 * panelButtonHeight - 20, panelButtonWidth, panelButtonHeight, this.StartNextLevel.bind(this)));
	this.GameEndPanel.addControl(new TStandardButton("Close", panelBorder, panelHeight - panelBorder - panelButtonHeight, panelButtonWidth, panelButtonHeight, this.CloseGameEnd.bind(this)));
	this.GameEndLabel1 = new TLabel("", panelWidth / 2, 125, TTextAlignment.taCenter);
	this.GameEndLabel2 = new TLabel("", panelWidth / 2, 170, TTextAlignment.taCenter);
	this.GameEndLabel1.setFont("bold 30px Arial");
	this.GameEndLabel2.setFont("bold 25px Arial");
	this.GameEndPanel.addControl(this.GameEndLabel1);
	this.GameEndPanel.addControl(this.GameEndLabel2);
	this.GameEndLabel2.setMaxWidth(panelWidth - 10);
	this.GameEndPanel.addControl(this.PerfectlyImage);
	this.GameEndPanel.addControl(this.SolvedImage);
	this.PerfectlyImage.setPosition((panelWidth - this.PerfectlyImage.Width) / 2, panelBorder);
	this.SolvedImage.setPosition((panelWidth - this.SolvedImage.Width) / 2, panelBorder);

	panelHeight = 350;
	this.NoMoreLevelsPanel = new TPanel((gCanvasWidth - panelWidth) / 2, (gCanvasHeight - panelHeight) / 2, panelWidth, panelHeight);
	this.NoMoreLevelsPanel.hide();
	GamePage.addControl(this.NoMoreLevelsPanel);
	this.NoMoreLevelsLabel1 = new TLabel("No more levels", panelWidth / 2, 55, TTextAlignment.taCenter);
	this.NoMoreLevelsLabel2 = new TLabel("If you want to play more levels, download the source code and add new levels on your own or play the original Android game.", panelWidth / 2, 110, TTextAlignment.taCenter);
	this.NoMoreLevelsLabel1.setFontColor("red");
	this.NoMoreLevelsLabel1.setFont("bold 30px Arial");
	this.NoMoreLevelsLabel2.setFont("bold 20px Arial");
	this.NoMoreLevelsPanel.addControl(this.NoMoreLevelsLabel1);
	this.NoMoreLevelsPanel.addControl(this.NoMoreLevelsLabel2);
	this.NoMoreLevelsLabel2.setMaxWidth(panelWidth - 30);
	this.NoMoreLevelsPanel.addControl(new TStandardButton("Close", panelBorder, panelHeight - panelBorder - panelButtonHeight, panelButtonWidth, panelButtonHeight, this.CloseGameEnd.bind(this)));

	var GameButtonHeight = 50;
	var GameButtonSpace = 10;
	var GameButtonWidth = ((gCanvasWidth - GameButtonSpace) / 3 - GameButtonSpace);
	GamePage.addControl(new TStandardButton("Menu", GameButtonSpace, gCanvasHeight - GameButtonSpace - GameButtonHeight, GameButtonWidth, GameButtonHeight, this.CancelLevel.bind(this), 0, this.ButtonImageList, this.MenuImageIndex));
	GamePage.addControl(new TStandardButton("Restart", GameButtonSpace + (GameButtonWidth + GameButtonSpace), gCanvasHeight - GameButtonSpace - GameButtonHeight, GameButtonWidth, GameButtonHeight, this.RestartLevel.bind(this), 0, this.ButtonImageList, this.RestartImageIndex));
	this.GameNextButton = new TStandardButton("Next", GameButtonSpace + 2 * (GameButtonWidth + GameButtonSpace), gCanvasHeight - GameButtonSpace - GameButtonHeight, GameButtonWidth, GameButtonHeight, this.StartNextLevel.bind(this), 0, this.ButtonImageList, this.NextImageIndex);
	GamePage.addControl(this.GameNextButton);

	HelpPage.addControl(new TStandardButton("Back", ButtonLeft, gCanvasHeight - 50 - ButtonHeight, ButtonWidth, ButtonHeight, this.ShowStartPage.bind(this), 0, this.ButtonImageList, this.BackImageIndex));
	var HelpLabel1 = new TLabel("Click on a circle and drag the mouse to draw a line.", 220, 170);
	var HelpLabel2 = new TLabel("Connect same-colored circles by lines and fill the entire board.", 220, 370);
	HelpLabel1.setMaxWidth(220);
	HelpLabel2.setMaxWidth(220);
	HelpPage.addControl(HelpLabel1);
	HelpPage.addControl(HelpLabel2);

	var HelpGrid1 = new THelpGrid(50, 130);
	var HelpGrid2 = new THelpGrid2(50, 330);
	HelpPage.addControl(HelpGrid1);
	HelpPage.addControl(HelpGrid2);

	var timer = setInterval(function () {
	    self.tick();
	}, 50);

	this.update = function () {
	    if (PageControl.update()) PageControl.paint();
	}

	this.mouseOut = function () {
	    PageControl.mouseOut();
	}

	this.mouseMove = function (x, y) {
	    PageControl.mouseMove(x, y);
	}

	this.mouseDown = function (left) {
	    PageControl.mouseDown(left);
	}

	this.mouseUp = function (left) {
	    PageControl.mouseUp(left);
	}

	this.tick = function () {
	    if (processing) return;
	    processing = true;
	    try {
	        this.update();
	    } finally {
	        processing = false;
	    }
	}
}

function doMouseMove(e) {
	gCanvasLeft = gCanvasTag.offsetLeft;
	gCanvasTop = gCanvasTag.offsetTop;
	gMouse.X = e.pageX - gCanvasLeft;
	gMouse.Y = e.pageY - gCanvasTop;
	gNeonFlowGame.mouseMove(gMouse.X, gMouse.Y);
}

function doMouseOut(e) {
	gNeonFlowGame.mouseOut();
}

function doMouseDown(e) {
	if (e.which == 1) gNeonFlowGame.mouseDown(true);
	else if (e.which == 3) gNeonFlowGame.mouseDown(false);
}

function doMouseUp(e) {
	if (e.which == 1) gNeonFlowGame.mouseUp(true);
	else if (e.which == 3) gNeonFlowGame.mouseUp(false);
}

function doContextMenu(e) {
	e.preventDefault();
}

var images = [
	{src: "images/OK30.png"},
	{src: "images/back.png"},
	{src: "images/star20.png"},
	{src: "images/OK20.png"},
	{src: "images/level20.png"},
	{src: "images/size20.png"},
	{src: "images/menu.png"},
	{src: "images/restart.png"},
	{src: "images/next.png"}
];
var loaded = 0;

function loadImages() {
	for (var i = 0; i < images.length; ++i) {
		var tmp = new Image;
		tmp.onload = function() {
			loaded++;
			if (loaded == images.length) {
				gNeonFlowGame = new TNeonFlowGame();
			}
		};
		tmp.src = images[i].src;
	}
}

function doOnLoad() {
	gCanvasTag = document.getElementById('game_canvas');
	if (!gCanvasTag) return;
	gContext = gCanvasTag.getContext('2d');
	if (!gContext) return;

	gCanvasWidth = gCanvasTag.width;
	gCanvasHeight = gCanvasTag.height;
	gCanvasLeft = gCanvasTag.offsetLeft;
	gCanvasTop = gCanvasTag.offsetTop;

	gMouse = new TPoint(0,0);
	loadImages();

	if (window.opera && opera.toString() == "[object Opera]") gSettings.HQ = false;
	else gContext.translate(0.5, 0.5);

	gCanvasTag.addEventListener("mousemove", doMouseMove, false);
	gCanvasTag.addEventListener("mouseout", doMouseOut, false);
	gCanvasTag.addEventListener("mousedown", doMouseDown, true);
	gCanvasTag.addEventListener("mouseup", doMouseUp, true);
	gCanvasTag.addEventListener("contextmenu", doContextMenu, false);
}

window.addEventListener('load', doOnLoad, false);

